@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -521,6 +521,28 @@ typedef void (APIENTRY * COIN_PFNGLGETQUERYOBJECTUIVPROC)(GLuint id, GLenum pnam
 typedef void *(APIENTRY * COIN_PFNGLXGETCURRENTDISPLAYPROC)(void);
 
 
+/* Typedefs for Framebuffer objects */
+
+typedef void (APIENTRY * COIN_PFNGLISRENDERBUFFERPROC)(GLuint renderbuffer);
+typedef void (APIENTRY * COIN_PFNGLBINDRENDERBUFFERPROC)(GLenum target, GLuint renderbuffer);
+typedef void (APIENTRY * COIN_PFNGLDELETERENDERBUFFERSPROC)(GLsizei n, const GLuint *renderbuffers);
+typedef void (APIENTRY * COIN_PFNGLGENRENDERBUFFERSPROC)(GLsizei n, GLuint *renderbuffers);
+typedef void (APIENTRY * COIN_PFNGLRENDERBUFFERSTORAGEPROC)(GLenum target, GLenum internalformat, GLsizei width, GLsizei height);
+typedef void (APIENTRY * COIN_PFNGLGETRENDERBUFFERPARAMETERIVPROC)(GLenum target, GLenum pname, GLint *params);
+typedef GLboolean (APIENTRY * COIN_PFNGLISFRAMEBUFFERPROC)(GLuint framebuffer);
+typedef void (APIENTRY * COIN_PFNGLBINDFRAMEBUFFERPROC)(GLenum target, GLuint framebuffer);
+typedef void (APIENTRY * COIN_PFNGLDELETEFRAMEBUFFERSPROC)(GLsizei n, const GLuint *framebuffers);
+typedef void (APIENTRY * COIN_PFNGLGENFRAMEBUFFERSPROC)(GLsizei n, GLuint *framebuffers);
+typedef GLenum (APIENTRY * COIN_PFNGLCHECKFRAMEBUFFERSTATUSPROC)(GLenum target);
+typedef void (APIENTRY * COIN_PFNGLFRAMEBUFFERTEXTURE1DPROC)(GLenum target, GLenum attachment, GLenum textarget, GLuint texture, GLint level);
+typedef void (APIENTRY * COIN_PFNGLFRAMEBUFFERTEXTURE2DPROC)(GLenum target, GLenum attachment, GLenum textarget, GLuint texture, GLint level);
+typedef void (APIENTRY * COIN_PFNGLFRAMEBUFFERTEXTURE3DPROC)(GLenum target, GLenum attachment, GLenum textarget, GLuint texture, GLint level, GLint zoffset);
+typedef void (APIENTRY * COIN_PFNGLFRAMEBUFFERRENDERBUFFERPROC)(GLenum target, GLenum attachment, GLenum renderbuffertarget, GLuint renderbuffer);
+typedef void (APIENTRY * COIN_PFNGLGETFRAMEBUFFERATTACHMENTPARAMETERIVPROC)(GLenum target, GLenum attachment, GLenum pname, GLint *params);
+typedef void (APIENTRY * COIN_PFNGLGENERATEMIPMAPPROC)(GLenum target);
+
+
+
 /* ********************************************************************** */
 
 /* Type specification for GLX info storage structure, embedded within
@@ -755,6 +777,25 @@ struct cc_glglue {
   COIN_PFNGLGETQUERYIVPROC glGetQueryiv;
   COIN_PFNGLGETQUERYOBJECTIVPROC glGetQueryObjectiv;
   COIN_PFNGLGETQUERYOBJECTUIVPROC glGetQueryObjectuiv;
+  
+  /* FBO */
+  COIN_PFNGLISRENDERBUFFERPROC glIsRenderbuffer;
+  COIN_PFNGLBINDRENDERBUFFERPROC glBindRenderbuffer;
+  COIN_PFNGLDELETERENDERBUFFERSPROC glDeleteRenderbuffers;
+  COIN_PFNGLGENRENDERBUFFERSPROC glGenRenderbuffers;
+  COIN_PFNGLRENDERBUFFERSTORAGEPROC glRenderbufferStorage;
+  COIN_PFNGLGETRENDERBUFFERPARAMETERIVPROC glGetRenderbufferParameteriv;
+  COIN_PFNGLISFRAMEBUFFERPROC glIsFramebuffer;
+  COIN_PFNGLBINDFRAMEBUFFERPROC glBindFramebuffer;
+  COIN_PFNGLDELETEFRAMEBUFFERSPROC glDeleteFramebuffers;
+  COIN_PFNGLGENFRAMEBUFFERSPROC glGenFramebuffers;
+  COIN_PFNGLCHECKFRAMEBUFFERSTATUSPROC glCheckFramebufferStatus;
+  COIN_PFNGLFRAMEBUFFERTEXTURE1DPROC glFramebufferTexture1D;
+  COIN_PFNGLFRAMEBUFFERTEXTURE2DPROC glFramebufferTexture2D;
+  COIN_PFNGLFRAMEBUFFERTEXTURE3DPROC glFramebufferTexture3D;
+  COIN_PFNGLFRAMEBUFFERRENDERBUFFERPROC glFramebufferRenderbuffer;
+  COIN_PFNGLGETFRAMEBUFFERATTACHMENTPARAMETERIVPROC glGetFramebufferAttachmentParameteriv;
+  COIN_PFNGLGENERATEMIPMAPPROC glGenerateMipmap;
 
   const char * versionstr;
   const char * vendorstr;
@@ -784,6 +825,7 @@ struct cc_glglue {
   SbBool has_arb_vertex_program;
   SbBool has_arb_shader_objects;
   SbBool has_texture_env_combine;
+  SbBool has_fbo;
 
   SbBool vbo_in_displaylist_ok;
   int max_lights;
