@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -341,6 +341,7 @@ typedef JSFunction * (* JS_DefineFunction_t)(JSContext *, JSObject *, const char
 typedef JSObject * (* JS_GetGlobalObject_t)(JSContext *);
 typedef JSGCCallback (* JS_SetGCCallback_t)(JSContext *, JSGCCallback);
 typedef void (* JS_GC_t)(JSContext *);
+typedef void (* JS_MaybeGC_t)(JSContext *);
 typedef JSBool (* JS_IsRunning_t)(JSContext *);
 typedef JSBool (* JS_DeleteProperty_t)(JSContext *, JSObject *, const char *);
 typedef JSScript * (* JS_CompileScript_t)(JSContext *, JSObject *,
@@ -361,6 +362,8 @@ typedef uint32_t (* JS_SetOptions_t)(JSContext *, uint32_t);
 typedef uint32_t (* JS_ToggleOptions_t)(JSContext *, uint32_t);
 typedef struct JSIdArray * (* JS_Enumerate_t)(JSContext *, JSObject *);
 typedef JSBool (* JS_IdToValue_t)(JSContext *, jsid, jsval *);
+typedef const char * (* JS_GetFunctionName_t)(JSFunction *);
+typedef JSObject * (* JS_GetConstructor_t)(JSContext *, JSObject *);
 
 
 /* Access interface. **************************************************** */
@@ -442,6 +445,7 @@ typedef struct {
   JS_GetGlobalObject_t JS_GetGlobalObject;
   JS_SetGCCallback_t JS_SetGCCallback;
   JS_GC_t JS_GC;
+  JS_MaybeGC_t JS_MaybeGC;
   JS_IsRunning_t JS_IsRunning;
   JS_DeleteProperty_t JS_DeleteProperty;
   JS_CompileScript_t JS_CompileScript;
@@ -459,6 +463,8 @@ typedef struct {
   JS_ToggleOptions_t JS_ToggleOptions;
   JS_Enumerate_t JS_Enumerate;
   JS_IdToValue_t JS_IdToValue;
+  JS_GetFunctionName_t JS_GetFunctionName;
+  JS_GetConstructor_t JS_GetConstructor;
 
 } SpiderMonkey_t;
 
