@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -185,7 +185,9 @@ private:
   static SbStringList * dirsearchlist;
 
   SbList<SoInput_FileInfo *> filestack;
-  SoInput_FileInfo * getTopOfStack(void) const;
+  SoInput_FileInfo * getTopOfStack(void) const {
+    return this->filestack[0];
+  }
 
   // OBSOLETE, only kept for Coin 2.x ABI compatibility.
   SbDict refdict;
