@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -51,6 +51,9 @@ public:
   virtual void pick(SoPickAction * action);
   virtual void getPrimitiveCount(SoGetPrimitiveCountAction * action);
   virtual void audioRender(SoAudioRenderAction * action);
+#ifdef COIN_NEXT_MINOR
+  virtual void search(SoSearchAction * action);
+#endif // COIN_NEXT_MINOR
 
   SoGroup * copyChildren(void) const;
   virtual SoChildList * getChildren(void) const;
@@ -59,6 +62,11 @@ public:
 
   const SbString & getFullName(void) const;
 
+#ifdef COIN_NEXT_MINOR
+  static void setSearchOK(SbBool dosearch);
+  static SbBool getSearchOK();
+#endif // COIN_NEXT_MINOR
+
 protected:
   virtual ~SoFile();
 
