@@ -4,7 +4,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -24,21 +24,7 @@
  *
 \**************************************************************************/
 
-/* This header file is supposed to take care of all operating system
- * dependent anomalies connected to the gl.h include file. */
-
-/**********************************************************************/
-
-/* This define is at least needed before inclusion of the header files
-   that are part of NVidia's Linux drivers v41.91. Without it, none of
-   the extension and OpenGL 1.1+ function prototypes will be set
-   up. */
-#define GL_GLEXT_PROTOTYPES 1
-
-/* #include <windows.h> - not needed on system */
-#include <GL/gl.h>
-#include <GL/glu.h>
-#include <GL/glext.h>
+#include <Inventor/system/gl-headers.h>
 
 /**********************************************************************/
 
@@ -1377,6 +1363,9 @@
 #ifndef GL_FRAGMENT_SHADER_ARB
 #define GL_FRAGMENT_SHADER_ARB 0x8B30
 #endif /* GL_FRAGMENT_SHADER_ARB */
+#ifndef GL_OBJECT_ACTIVE_UNIFORMS_ARB
+#define GL_OBJECT_ACTIVE_UNIFORMS_ARB 0x8B86
+#endif /* GL_OBJECT_ACTIVE_UNIFORMS_ARB */
 
 /* GL_ARB_shader_objects */
 #ifndef GL_INT_VEC2_ARB
@@ -1489,6 +1478,164 @@
 #define GL_ALL_CLIENT_ATTRIB_BITS 0xFFFFFFFF
 #endif /* GL_ALL_CLIENT_ATTRIB_BITS */
 
+/* FBO defines */
+#ifndef GL_INVALID_FRAMEBUFFER_OPERATION_EXT
+#define GL_INVALID_FRAMEBUFFER_OPERATION_EXT 0x0506
+#endif /* GL_INVALID_FRAMEBUFFER_OPERATION_EXT */
+#ifndef GL_MAX_RENDERBUFFER_SIZE_EXT
+#define GL_MAX_RENDERBUFFER_SIZE_EXT 0x84E8
+#endif /* GL_MAX_RENDERBUFFER_SIZE_EXT */
+#ifndef GL_FRAMEBUFFER_BINDING_EXT
+#define GL_FRAMEBUFFER_BINDING_EXT 0x8CA6
+#endif /* GL_FRAMEBUFFER_BINDING_EXT */
+#ifndef GL_RENDERBUFFER_BINDING_EXT
+#define GL_RENDERBUFFER_BINDING_EXT 0x8CA7
+#endif /* GL_RENDERBUFFER_BINDING_EXT */
+#ifndef GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE_EXT
+#define GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE_EXT 0x8CD0
+#endif /* GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE_EXT */
+#ifndef GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME_EXT
+#define GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME_EXT 0x8CD1
+#endif /* GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME_EXT */
+#ifndef GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL_EXT
+#define GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL_EXT 0x8CD2
+#endif /* GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL_EXT */
+#ifndef GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE_EXT
+#define GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE_EXT 0x8CD3
+#endif /* GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE_EXT */
+#ifndef GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_3D_ZOFFSET_EXT
+#define GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_3D_ZOFFSET_EXT 0x8CD4
+#endif /* GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_3D_ZOFFSET_EXT */
+#ifndef GL_FRAMEBUFFER_COMPLETE_EXT
+#define GL_FRAMEBUFFER_COMPLETE_EXT 0x8CD5
+#endif /* GL_FRAMEBUFFER_COMPLETE_EXT */
+#ifndef GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_EXT
+#define GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_EXT 0x8CD6
+#endif /* GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_EXT */
+#ifndef GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_EXT
+#define GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_EXT 0x8CD7
+#endif /* GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_EXT */
+#ifndef GL_FRAMEBUFFER_INCOMPLETE_DUPLICATE_ATTACHMENT_EXT
+#define GL_FRAMEBUFFER_INCOMPLETE_DUPLICATE_ATTACHMENT_EXT 0x8CD8
+#endif /* GL_FRAMEBUFFER_INCOMPLETE_DUPLICATE_ATTACHMENT_EXT */
+#ifndef GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT
+#define GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT 0x8CD9
+#endif /* GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_EXT */
+#ifndef GL_FRAMEBUFFER_INCOMPLETE_FORMATS_EXT
+#define GL_FRAMEBUFFER_INCOMPLETE_FORMATS_EXT 0x8CDA
+#endif /* GL_FRAMEBUFFER_INCOMPLETE_FORMATS_EXT */
+#ifndef GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER_EXT
+#define GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER_EXT 0x8CDB
+#endif /* GL_FRAMEBUFFER_INCOMPLETE_DRAW_BUFFER_EXT */
+#ifndef GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER_EXT
+#define GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER_EXT 0x8CDC
+#endif /* GL_FRAMEBUFFER_INCOMPLETE_READ_BUFFER_EXT */
+#ifndef GL_FRAMEBUFFER_UNSUPPORTED_EXT
+#define GL_FRAMEBUFFER_UNSUPPORTED_EXT 0x8CDD
+#endif /* GL_FRAMEBUFFER_UNSUPPORTED_EXT */
+#ifndef GL_MAX_COLOR_ATTACHMENTS_EXT
+#define GL_MAX_COLOR_ATTACHMENTS_EXT 0x8CDF
+#endif /* GL_MAX_COLOR_ATTACHMENTS_EXT */
+#ifndef GL_COLOR_ATTACHMENT0_EXT
+#define GL_COLOR_ATTACHMENT0_EXT 0x8CE0
+#endif /* GL_COLOR_ATTACHMENT0_EXT */
+#ifndef GL_COLOR_ATTACHMENT1_EXT
+#define GL_COLOR_ATTACHMENT1_EXT 0x8CE1
+#endif /* GL_COLOR_ATTACHMENT1_EXT */
+#ifndef GL_COLOR_ATTACHMENT2_EXT
+#define GL_COLOR_ATTACHMENT2_EXT 0x8CE2
+#endif /* GL_COLOR_ATTACHMENT2_EXT */
+#ifndef GL_COLOR_ATTACHMENT3_EXT
+#define GL_COLOR_ATTACHMENT3_EXT 0x8CE3
+#endif /* GL_COLOR_ATTACHMENT3_EXT */
+#ifndef GL_COLOR_ATTACHMENT4_EXT
+#define GL_COLOR_ATTACHMENT4_EXT 0x8CE4
+#endif /* GL_COLOR_ATTACHMENT4_EXT */
+#ifndef GL_COLOR_ATTACHMENT5_EXT
+#define GL_COLOR_ATTACHMENT5_EXT 0x8CE5
+#endif /* GL_COLOR_ATTACHMENT5_EXT */
+#ifndef GL_COLOR_ATTACHMENT6_EXT
+#define GL_COLOR_ATTACHMENT6_EXT 0x8CE6
+#endif /* GL_COLOR_ATTACHMENT6_EXT */
+#ifndef GL_COLOR_ATTACHMENT7_EXT
+#define GL_COLOR_ATTACHMENT7_EXT 0x8CE7
+#endif /* GL_COLOR_ATTACHMENT7_EXT */
+#ifndef GL_COLOR_ATTACHMENT8_EXT
+#define GL_COLOR_ATTACHMENT8_EXT 0x8CE8
+#endif /* GL_COLOR_ATTACHMENT8_EXT */
+#ifndef GL_COLOR_ATTACHMENT9_EXT
+#define GL_COLOR_ATTACHMENT9_EXT 0x8CE9
+#endif /* GL_COLOR_ATTACHMENT9_EXT */
+#ifndef GL_COLOR_ATTACHMENT10_EXT
+#define GL_COLOR_ATTACHMENT10_EXT 0x8CEA
+#endif /* GL_COLOR_ATTACHMENT10_EXT */
+#ifndef GL_COLOR_ATTACHMENT11_EXT
+#define GL_COLOR_ATTACHMENT11_EXT 0x8CEB
+#endif /* GL_COLOR_ATTACHMENT11_EXT */
+#ifndef GL_COLOR_ATTACHMENT12_EXT
+#define GL_COLOR_ATTACHMENT12_EXT 0x8CEC
+#endif /* GL_COLOR_ATTACHMENT12_EXT */
+#ifndef GL_COLOR_ATTACHMENT13_EXT
+#define GL_COLOR_ATTACHMENT13_EXT 0x8CED
+#endif /* GL_COLOR_ATTACHMENT13_EXT */
+#ifndef GL_COLOR_ATTACHMENT14_EXT
+#define GL_COLOR_ATTACHMENT14_EXT 0x8CEE
+#endif /* GL_COLOR_ATTACHMENT14_EXT */
+#ifndef GL_COLOR_ATTACHMENT15_EXT
+#define GL_COLOR_ATTACHMENT15_EXT 0x8CEF
+#endif /* GL_COLOR_ATTACHMENT15_EXT */
+#ifndef GL_DEPTH_ATTACHMENT_EXT
+#define GL_DEPTH_ATTACHMENT_EXT 0x8D00
+#endif /* GL_DEPTH_ATTACHMENT_EXT */
+#ifndef GL_STENCIL_ATTACHMENT_EXT
+#define GL_STENCIL_ATTACHMENT_EXT 0x8D20
+#endif /* GL_STENCIL_ATTACHMENT_EXT */
+#ifndef GL_FRAMEBUFFER_EXT
+#define GL_FRAMEBUFFER_EXT 0x8D40
+#endif /* GL_FRAMEBUFFER_EXT */
+#ifndef GL_RENDERBUFFER_EXT
+#define GL_RENDERBUFFER_EXT 0x8D41
+#endif /* GL_RENDERBUFFER_EXT */
+#ifndef GL_RENDERBUFFER_WIDTH_EXT
+#define GL_RENDERBUFFER_WIDTH_EXT 0x8D42
+#endif /* GL_RENDERBUFFER_WIDTH_EXT */
+#ifndef GL_RENDERBUFFER_HEIGHT_EXT
+#define GL_RENDERBUFFER_HEIGHT_EXT 0x8D43
+#endif /* GL_RENDERBUFFER_HEIGHT_EXT */
+#ifndef GL_RENDERBUFFER_INTERNAL_FORMAT_EXT
+#define GL_RENDERBUFFER_INTERNAL_FORMAT_EXT 0x8D44
+#endif /* GL_RENDERBUFFER_INTERNAL_FORMAT_EXT */
+#ifndef GL_STENCIL_INDEX1_EXT
+#define GL_STENCIL_INDEX1_EXT 0x8D46
+#endif /* GL_STENCIL_INDEX1_EXT */
+#ifndef GL_STENCIL_INDEX4_EXT
+#define GL_STENCIL_INDEX4_EXT 0x8D47
+#endif /* GL_STENCIL_INDEX4_EXT */
+#ifndef GL_STENCIL_INDEX8_EXT
+#define GL_STENCIL_INDEX8_EXT 0x8D48
+#endif /* GL_STENCIL_INDEX8_EXT */
+#ifndef GL_STENCIL_INDEX16_EXT
+#define GL_STENCIL_INDEX16_EXT 0x8D49
+#endif /* GL_STENCIL_INDEX16_EXT */
+#ifndef GL_RENDERBUFFER_RED_SIZE_EXT
+#define GL_RENDERBUFFER_RED_SIZE_EXT 0x8D50
+#endif /* GL_RENDERBUFFER_RED_SIZE_EXT */
+#ifndef GL_RENDERBUFFER_GREEN_SIZE_EXT
+#define GL_RENDERBUFFER_GREEN_SIZE_EXT 0x8D51
+#endif /* GL_RENDERBUFFER_GREEN_SIZE_EXT */
+#ifndef GL_RENDERBUFFER_BLUE_SIZE_EXT
+#define GL_RENDERBUFFER_BLUE_SIZE_EXT 0x8D52
+#endif /* GL_RENDERBUFFER_BLUE_SIZE_EXT */
+#ifndef GL_RENDERBUFFER_ALPHA_SIZE_EXT
+#define GL_RENDERBUFFER_ALPHA_SIZE_EXT 0x8D53
+#endif /* GL_RENDERBUFFER_ALPHA_SIZE_EXT */
+#ifndef GL_RENDERBUFFER_DEPTH_SIZE_EXT
+#define GL_RENDERBUFFER_DEPTH_SIZE_EXT 0x8D54
+#endif /* GL_RENDERBUFFER_DEPTH_SIZE_EXT */
+#ifndef GL_RENDERBUFFER_STENCIL_SIZE_EXT
+#define GL_RENDERBUFFER_STENCIL_SIZE_EXT 0x8D55
+#endif /* GL_RENDERBUFFER_STENCIL_SIZE_EXT */
+
 /*** GL enums, end ****************************************************/
 /**********************************************************************/
 
@@ -1684,19 +1831,32 @@
 /* Tesselator */
 #ifndef GLU_TESS_BEGIN
 #define GLU_TESS_BEGIN 100100
-#endif
+#endif /* ! GLU_TESS_BEGIN */
 #ifndef GLU_TESS_VERTEX
 #define GLU_TESS_VERTEX 100101
-#endif
-#ifndef GLU_TESS_END 
+#endif /* ! GLU_TESS_VERTEX */
+#ifndef GLU_TESS_END
 #define GLU_TESS_END 100102
-#endif
+#endif /* ! GLU_TESS_END */
 #ifndef GLU_TESS_ERROR
 #define GLU_TESS_ERROR 100103
-#endif
+#endif /* ! GLU_TESS_ERROR */
 #ifndef GLU_TESS_COMBINE
 #define GLU_TESS_COMBINE 100105
-#endif
+#endif /* ! GLU_TESS_COMBINE */
+#ifndef GLU_TESS_BEGIN_DATA
+#define GLU_TESS_BEGIN_DATA 100106
+#endif /* ! GLU_TESS_BEGIN_DATA */
+#ifndef GLU_TESS_VERTEX_DATA
+#define GLU_TESS_VERTEX_DATA 100107
+#endif /* ! GLU_TESS_VERTEX_DATA */
+#ifndef GLU_TESS_END_DATA
+#define GLU_TESS_END_DATA 100108
+#endif /* ! GLU_TESS_END_DATA */
+#ifndef GLU_TESS_ERROR_DATA
+#define GLU_TESS_ERROR_DATA 100109
+#endif /* ! GLU_TESS_ERROR_DATA */
+
 #ifndef GLU_TESS_MISSING_BEGIN_POLYGON
 #define GLU_TESS_MISSING_BEGIN_POLYGON 100151
 #endif /* ! GLU_TESS_MISSING_BEGIN_POLYGON */
