@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -202,6 +202,7 @@
 #include <Inventor/actions/SoActions.h>
 #include <Inventor/elements/SoOverrideElement.h>
 #include <Inventor/misc/SoState.h>
+#include <Inventor/misc/SoCompactPathList.h>
 #include <Inventor/lists/SbList.h>
 #include <Inventor/C/tidbitsp.h>
 #include <Inventor/SoDB.h>
@@ -328,11 +329,21 @@ public:
     struct {
       const SoPathList * pathlist;
       const SoPathList * origpathlist;
+      SoCompactPathList * compactlist;
     } pathlistdata;
   } applieddata;
   SbBool terminated;
   SbList <SbList<int> *> pathcodearray;
   int prevenabledelementscounter;
+
+  // Needed since popCurPath() is inline in Coin-2. This workaround is
+  // not needed in Coin development.
+  void fixMissingCompactPathListPops(const SoTempPath & currentpath) {
+    int len = currentpath.getLength();
+    while (this->applieddata.pathlistdata.compactlist->getDepth() > len) {
+      this->applieddata.pathlistdata.compactlist->pop();
+    }
+  }
 };
 
 #endif // DOXYGEN_SKIP_THIS
@@ -367,6 +378,7 @@ SoAction::~SoAction(void)
   for (int i = 0; i < n; i++) delete THIS->pathcodearray[i];
   delete this->state;
   
+  this->currentpath.unrefNoDelete(); // to match the ref() in the constructor
   delete THIS;
 }
 
@@ -653,15 +665,21 @@ SoAction::apply(const SoPathList & pathlist, SbBool obeysrules)
 
   THIS->applieddata.pathlistdata.origpathlist = &pathlist;
   THIS->applieddata.pathlistdata.pathlist = &pathlist;
+  THIS->applieddata.pathlistdata.compactlist = NULL;
   THIS->appliedcode = PATH_LIST;
   this->currentpathcode = pathlist[0]->getFullLength() > 1 ?
     SoAction::IN_PATH : SoAction::BELOW_PATH;
 
   if (obeysrules) {
     // GoGoGo
+    if (this->shouldCompactPathList()) {
+      THIS->applieddata.pathlistdata.compactlist = new SoCompactPathList(pathlist);
+    }
     this->currentpath.setHead(pathlist[0]->getHead());
     this->beginTraversal(pathlist[0]->getHead());
     this->endTraversal(pathlist[0]->getHead());
+    delete THIS->applieddata.pathlistdata.compactlist;
+    THIS->applieddata.pathlistdata.compactlist = NULL;
   }
   else {
     // make copy of path list and make sure it obeys rules
@@ -676,8 +694,16 @@ SoAction::apply(const SoPathList & pathlist, SbBool obeysrules)
     if (sortedlist[0]->getHead() == sortedlist[num-1]->getHead()) {
       this->currentpath.setHead(sortedlist[0]->getHead());
       THIS->applieddata.pathlistdata.pathlist = &sortedlist;
+      if (this->shouldCompactPathList()) {
+        THIS->applieddata.pathlistdata.compactlist = new SoCompactPathList(sortedlist);
+      }
+      else {
+        THIS->applieddata.pathlistdata.compactlist = NULL;
+      }
       this->beginTraversal(sortedlist[0]->getHead());
       this->endTraversal(sortedlist[0]->getHead());
+      delete THIS->applieddata.pathlistdata.compactlist;
+      THIS->applieddata.pathlistdata.compactlist = NULL;
     }
     else {
       // make one pass per head node. sortedlist is sorted on
@@ -698,7 +724,16 @@ SoAction::apply(const SoPathList & pathlist, SbBool obeysrules)
         this->currentpathcode = templist[0]->getFullLength() > 1 ?
           SoAction::IN_PATH : SoAction::BELOW_PATH;
         this->currentpath.setHead(templist[0]->getHead());
+
+        if (this->shouldCompactPathList()) {
+          THIS->applieddata.pathlistdata.compactlist = new SoCompactPathList(templist);
+        }
+        else {
+          THIS->applieddata.pathlistdata.compactlist = NULL;
+        }
         this->beginTraversal(templist[0]->getHead());
+        delete THIS->applieddata.pathlistdata.compactlist;
+        THIS->applieddata.pathlistdata.compactlist = NULL;
         templist.truncate(0);
       }
     }
@@ -712,7 +747,7 @@ SoAction::apply(const SoPathList & pathlist, SbBool obeysrules)
 /*!
   Applies this action object to the same as \a beingApplied is being
   applied to.
-
+  
   \COIN_FUNCTION_EXTENSION
   
   \since Coin 2.1
@@ -885,6 +920,14 @@ SoAction::traverse(SoNode * const node)
 void
 SoAction::pushCurPath(const int childindex, SoNode * node)
 {
+  // Detect missing popCurPath(PathCode) calls. See comment in
+  // fixMissingCompactPathListPops()
+  if (this->currentpathcode == IN_PATH &&
+      this->getWhatAppliedTo() != PATH &&
+      THIS->applieddata.pathlistdata.compactlist != NULL) {
+    THIS->fixMissingCompactPathListPops(this->currentpath);
+  }
+
   if (node) this->currentpath.simpleAppend(node, childindex);
   else {
     this->currentpath.append(childindex);
@@ -911,41 +954,54 @@ SoAction::pushCurPath(const int childindex, SoNode * node)
       }
     }
     else {
-      // test for below path by testing for one path that contains
-      // current path, and is longer than current.  at the same time,
-      // test for off path by testing if there is no paths that
-      // contains current path.  this is a lame and slow way to do it,
-      // but I don't think path list traversal is used that often,
-      // and not for a huge amount of paths. If I'm wrong, we'll just
-      // have to optimize this, for instance by implementing
-      // the SGI's SoCompactPathList class.
-      //                                       pederb, 2001-03-28
-      const SoPathList * pl = THIS->applieddata.pathlistdata.pathlist;
-      int i, n = pl->getLength();
-      int len = -1;
+      if (THIS->applieddata.pathlistdata.compactlist) {
+        SbBool inpath = THIS->applieddata.pathlistdata.compactlist->push(childindex);
+        assert(THIS->applieddata.pathlistdata.compactlist->getDepth() == this->currentpath.getLength());
 
-      for (i = 0; i < n; i++) {
-        const SoPath * path = (*pl)[i];
-        len = path->getFullLength();
-        // small optimization, no use testing if path is shorter
-        if (len >= curlen) {
-          if (path->containsPath(&this->currentpath)) break;
+        if (!inpath) {
+          this->currentpathcode = OFF_PATH;
+        }
+        else {
+          int	numchildren;
+          const int * dummy;
+          THIS->applieddata.pathlistdata.compactlist->getChildren(numchildren, dummy);
+          this->currentpathcode = numchildren == 0 ? BELOW_PATH : IN_PATH;
         }
       }
-      // if no path is found, we're off path
-      if (i == n) {
-        this->currentpathcode = OFF_PATH;
+      else {
+        // test for below path by testing for one path that contains
+        // current path, and is longer than current.  At the same time,
+        // test for off path by testing if there is no paths that
+        // contains current path.  This is a lame and slow way to do it,
+        // but SoCompactPathList will always be used. This is just backup
+        // code in case some action actually disables compact path list.
+        const SoPathList * pl = THIS->applieddata.pathlistdata.pathlist;
+        int i, n = pl->getLength();
+        int len = -1;
+        
+        for (i = 0; i < n; i++) {
+          const SoPath * path = (*pl)[i];
+          len = path->getFullLength();
+          // small optimization, no use testing if path is shorter
+          if (len >= curlen) {
+            if (path->containsPath(&this->currentpath)) break;
+          }
+        }
+        // if no path is found, we're off path
+        if (i == n) {
+          this->currentpathcode = OFF_PATH;
 #ifdef DEBUG_PATH_TRAVERSAL
-        fprintf(stderr,"off path at: %d (%s), depth: %d\n",
-                childindex, node->getName().getString(), curlen);
+          fprintf(stderr,"off path at: %d (%s), depth: %d\n",
+                  childindex, node->getName().getString(), curlen);
 #endif // DEBUG_PATH_TRAVERSAL
-      }
-      else if (len == curlen) {
-        this->currentpathcode = BELOW_PATH;
+        }
+        else if (len == curlen) {
+          this->currentpathcode = BELOW_PATH;
 #ifdef DEBUG_PATH_TRAVERSAL
-        fprintf(stderr,"below path at: %d (%s), depth: %d\n",
-                childindex, node->getName().getString(), curlen);
+          fprintf(stderr,"below path at: %d (%s), depth: %d\n",
+                  childindex, node->getName().getString(), curlen);
 #endif // DEBUG_PATH_TRAVERSAL
+        }
       }
     }
   }
@@ -1017,25 +1073,34 @@ SoAction::usePathCode(int & numindices, const int * & indices)
   myarray->truncate(0);
 
   if (this->getWhatAppliedTo() == PATH_LIST) {
-    // this might be very slow if the list contains a lot of
-    // paths. See comment in pushCurPath(int, SoNode*) about this.
-    const SoPathList * pl = THIS->applieddata.pathlistdata.pathlist;
-    int n = pl->getLength();
-    int previdx = -1;
-    myarray->truncate(0);
-    for (int i = 0; i < n; i++) {
-      const SoPath * path = (*pl)[i];
-      if (path->getFullLength() > curlen &&
-          path->containsPath(&this->currentpath)) {
-        int idx = path->getIndex(curlen);
-        if (idx != previdx) {
-          myarray->append(idx);
-          previdx = idx;
+    if (THIS->applieddata.pathlistdata.compactlist) {
+      // Detect missing popCurPath(PathCode) calls. See comment in
+      // fixMissingCompactPathListPops()
+      THIS->fixMissingCompactPathListPops(this->currentpath);
+      assert(THIS->applieddata.pathlistdata.compactlist->getDepth() == this->currentpath.getLength());
+      THIS->applieddata.pathlistdata.compactlist->getChildren(numindices, indices);
+    }
+    else {
+      // this might be very slow if the list contains a lot of
+      // paths. See comment in pushCurPath(int, SoNode*) about this.
+      const SoPathList * pl = THIS->applieddata.pathlistdata.pathlist;
+      int n = pl->getLength();
+      int previdx = -1;
+      myarray->truncate(0);
+      for (int i = 0; i < n; i++) {
+        const SoPath * path = (*pl)[i];
+        if (path->getFullLength() > curlen &&
+            path->containsPath(&this->currentpath)) {
+          int idx = path->getIndex(curlen);
+          if (idx != previdx) {
+            myarray->append(idx);
+            previdx = idx;
+          }
         }
       }
+      numindices = myarray->getLength();
+      indices = myarray->getArrayPtr();
     }
-    numindices = myarray->getLength();
-    indices = myarray->getArrayPtr();
   }
   else {
     numindices = 1;
