@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -952,8 +952,11 @@ SoGLRenderAction::beginTraversal(SoNode * node)
   if (COIN_GLBBOX) {
     THIS->bboxaction->apply(node);
   }
-  int err_before_init = glGetError();
+  int err_before_init = GL_NO_ERROR;
 
+  if (sogl_glerror_debugging()) {
+    err_before_init = glGetError();
+  }
   if (THIS->needglinit) {
     THIS->needglinit = FALSE;
 
@@ -972,7 +975,11 @@ SoGLRenderAction::beginTraversal(SoNode * node)
     }
   }
 
-  int err_after_init = glGetError();
+  int err_after_init = GL_NO_ERROR;
+
+  if (sogl_glerror_debugging()) {
+    err_after_init = glGetError();
+  }
 
   if (COIN_DEBUG && ((err_before_init != GL_NO_ERROR) || (err_after_init != GL_NO_ERROR))) {
     int err = (err_before_init != GL_NO_ERROR) ? err_before_init : err_after_init;
@@ -1487,17 +1494,21 @@ SoGLRenderActionP::render(SoNode * node)
 
   this->precblist.invokeCallbacks((void*) this->action);
 
-  // Check if the current OpenGL context has an accumulation buffer
-  // (rendering multiple passes doesn't make much sense otherwise).
-  GLint accumbits;
-  glGetIntegerv(GL_ACCUM_RED_BITS, &accumbits);
-
   if (this->action->getNumPasses() > 1) {
+    // Check if the current OpenGL context has an accumulation buffer
+    // (rendering multiple passes doesn't make much sense otherwise).
+    GLint accumbits;
+    glGetIntegerv(GL_ACCUM_RED_BITS, &accumbits);
+    
     if (accumbits == 0) {
-      SoDebugError::postWarning("SoGLRenderActionP::render", 
-                                "Multipass rendering requested,\nbut current "
-                                "GL context has no accumulation buffer - "
-                                "falling back to single pass\nrendering.");
+      static SbBool first = TRUE;
+      if (first) {
+        SoDebugError::postWarning("SoGLRenderActionP::render", 
+                                  "Multipass rendering requested,\nbut current "
+                                  "GL context has no accumulation buffer - "
+                                  "falling back to single pass\nrendering.");
+        first = FALSE;
+      }
       this->renderSingle(node);
     } else {
       this->renderMulti(node);
@@ -1615,9 +1626,7 @@ SoGLRenderActionP::renderSingle(SoNode * node)
     }
 
     // Render all transparent paths that should not be sorted
-    for (i = 0; i < this->transpobjpaths.getLength(); i++) {
-      this->action->apply(this->transpobjpaths[i]);
-    }
+    this->action->apply(this->transpobjpaths, TRUE);
 
     // enable depth buffer writes again
     glDepthMask(GL_TRUE);
@@ -1756,7 +1765,7 @@ SoGLRenderActionP::renderOneBlendLayer(const SoState * state,
   
 
   // Clear all errors before traversal, just in case.
-  GLenum glerror = glGetError();
+  GLenum glerror =  sogl_glerror_debugging() ? glGetError() : GL_NO_ERROR;
   while (glerror) {
     SoDebugError::postWarning("renderOneBlendLayer", 
                               "glError() = %d\n", glerror);
@@ -1861,7 +1870,7 @@ SoGLRenderActionP::initSortedLayersBlendRendering(const SoState * state)
     // FIXME: Maybe a wrapper for catching fragment program errors
     // should be a part of GLUE... (20031204 handegar)
     GLint errorPos;    
-    GLenum err = glGetError();
+    GLenum err = sogl_glerror_debugging() ? glGetError() : GL_NO_ERROR;
     if (err) {
       glGetIntegerv(GL_PROGRAM_ERROR_POSITION_ARB, &errorPos);
       SoDebugError::postWarning("initSortedLayersBlendRendering",
