@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See http://www.coin3d.org/ for more information.
+ *  See <URL:http://www.coin3d.org/> for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
+ *  <URL:http://www.sim.no/>.
  *
 \**************************************************************************/
 
@@ -148,8 +148,8 @@
   #include <Inventor/actions/SoReorganizeAction.h>
   #include <assert.h>
   #include <stdio.h>
-
-  static void strip_node(SoType type, SoNode * root)
+  
+  static void strip_node(SoType type, SoNode * root) 
   {
     SoSearchAction sa;
     sa.setType(type);
@@ -164,8 +164,8 @@
           SoGroup * g = (SoGroup*) p->getNodeFromTail(1);
           g->removeChild(p->getIndexFromTail(0));
         }
-      }
-    sa.reset();
+      } 
+    sa.reset();  
   }
 
   int
@@ -187,15 +187,15 @@
 
     SoDB::init();
     SoInteraction::init();
-
+    
     SoInput input;
     SbBool ok = input.openFile(argv[1]);
     if (!ok) {
       fprintf(stderr,"Unable to open file.\n");
       return -1;
     }
-    SoSeparator * root = SoDB::readAll(&input);
-
+    SoSeparator * root = SoDB::readAll(&input); 
+  
     SbBool vrml1 = input.isFileVRML1();
     SbBool vrml2 = input.isFileVRML2();
 
@@ -203,7 +203,7 @@
       fprintf(stderr,"VRML2 not supported yet\n");
       return -1;
     }
-
+  
     if (!root) {
       fprintf(stderr,"Unable to read file.\n");
       return -1;
@@ -216,7 +216,7 @@
     fprintf(stderr,"done\n");
 
     SoOutput out;
-    if (out.openFile(argv[2])) {
+    if (out.openFile(argv[2])) {      
       if (strip) { // strip coord3, texcoord and normal nodes
         fprintf(stderr,"stripping old nodes from scene graph\n");
         strip_node(SoCoordinate3::getClassTypeId(), root);
@@ -235,6 +235,7 @@
   \endvarbatim
 
   \since Coin 2.5
+
 */
 
 #include <Inventor/SbName.h>
@@ -280,10 +281,9 @@
 
 #include <coindefs.h> // COIN_STUB()
 
-
 class SoReorganizeActionP {
  public:
-  SoReorganizeActionP(SoReorganizeAction * masterin)
+  SoReorganizeActionP(SoReorganizeAction * masterin) 
     : master(masterin),
       gennormals(TRUE),
       gentexcoords(TRUE),
@@ -298,24 +298,24 @@ class SoReorganizeActionP {
 
     cbaction.addTriangleCallback(SoVRMLIndexedFaceSet::getClassTypeId(), triangle_cb, this);
     cbaction.addLineSegmentCallback(SoVRMLIndexedLineSet::getClassTypeId(), line_segment_cb, this);
-
+    
     cbaction.addPreCallback(SoVertexShape::getClassTypeId(),
                             pre_shape_cb, this);
     cbaction.addPostCallback(SoVertexShape::getClassTypeId(),
                              post_shape_cb, this);
-
+    
     cbaction.addPreCallback(SoVRMLIndexedFaceSet::getClassTypeId(),
                             pre_shape_cb, this);
     cbaction.addPostCallback(SoVRMLIndexedFaceSet::getClassTypeId(),
                              post_shape_cb, this);
-
+    
 
     cbaction.addPreCallback(SoVRMLIndexedLineSet::getClassTypeId(),
                             pre_shape_cb, this);
     cbaction.addPostCallback(SoVRMLIndexedLineSet::getClassTypeId(),
                              post_shape_cb, this);
-
-  }
+    
+  } 
   SoReorganizeAction * master;
   SbBool gennormals;
   SbBool gentexcoords;
@@ -349,7 +349,7 @@ class SoReorganizeActionP {
   static void line_segment_cb(void * userdata, SoCallbackAction * action,
                               const SoPrimitiveVertex * v1,
                               const SoPrimitiveVertex * v2);
-
+    
   SbBool initShape(SoCallbackAction * action);
   void replaceNode(SoFullPath * path);
   void replaceIfs(SoFullPath * path);
@@ -393,79 +393,79 @@ SoReorganizeAction::~SoReorganizeAction(void)
   delete PRIVATE(this);
 }
 
-SoSeparator *
+SoSeparator * 
 SoReorganizeAction::getSimplifiedSceneGraph(void) const
 {
   return NULL;
 }
 
-void
+void 
 SoReorganizeAction::generateNormals(SbBool onoff)
 {
   PRIVATE(this)->gennormals = onoff;
 }
 
-SbBool
+SbBool 
 SoReorganizeAction::areNormalGenerated(void) const
 {
   return PRIVATE(this)->gennormals;
 }
 
-void
+void 
 SoReorganizeAction::generateTriangleStrips(SbBool onoff)
 {
   PRIVATE(this)->gentristrips = onoff;
 }
 
-SbBool
+SbBool 
 SoReorganizeAction::areTriangleStripGenerated(void) const
 {
   return PRIVATE(this)->gentristrips;
 }
 
-void
+void 
 SoReorganizeAction::generateTexCoords(SbBool onoff)
 {
   PRIVATE(this)->gentexcoords = onoff;
 }
 
-SbBool
+SbBool 
 SoReorganizeAction::areTexCoordsGenerated(void) const
 {
   return PRIVATE(this)->gentexcoords;
 }
 
-void
+void 
 SoReorganizeAction::generateVPNodes(SbBool onoff)
 {
   PRIVATE(this)->genvp = onoff;
 }
 
-SbBool
+SbBool 
 SoReorganizeAction::areVPNodesGenerated(void)
 {
   return PRIVATE(this)->genvp;
 }
 
-void
+void 
 SoReorganizeAction::matchIndexArrays(SbBool onoff)
 {
   PRIVATE(this)->matchidx = onoff;
 }
 
-SbBool
+SbBool 
 SoReorganizeAction::areIndexArraysMatched(void) const
 {
   return PRIVATE(this)->matchidx;
 }
 
-SoSimplifier *
+SoSimplifier * 
 SoReorganizeAction::getSimplifier(void) const
 {
   return NULL;
 }
 
-void
+void 
 SoReorganizeAction::apply(SoNode * root)
 {
   int i;
@@ -478,7 +478,7 @@ SoReorganizeAction::apply(SoNode * root)
     this->apply(pl[i]);
   }
   PRIVATE(this)->sa.reset();
-
+  
   PRIVATE(this)->sa.setType(SoVRMLIndexedFaceSet::getClassTypeId());
   PRIVATE(this)->sa.setSearchingAll(TRUE);
   PRIVATE(this)->sa.setInterest(SoSearchAction::ALL);
@@ -500,14 +500,14 @@ SoReorganizeAction::apply(SoNode * root)
   PRIVATE(this)->sa.reset();
 }
 
-void
+void 
 SoReorganizeAction::apply(SoPath * path)
 {
   PRIVATE(this)->cbaction.apply(path);
   PRIVATE(this)->replaceNode((SoFullPath*) path);
 }
 
-void
+void 
 SoReorganizeAction::apply(const SoPathList & pathlist, SbBool obeysrules)
 {
   for (int i = 0; i < pathlist.getLength(); i++) {
@@ -515,13 +515,13 @@ SoReorganizeAction::apply(const SoPathList & pathlist, SbBool obeysrules)
   }
 }
 
-void
+void 
 SoReorganizeAction::startReport(const char * msg)
 {
   COIN_STUB();
 }
 
-void
+void 
 SoReorganizeAction::finishReport(void)
 {
   COIN_STUB();
@@ -535,7 +535,7 @@ SoReorganizeAction::beginTraversal(SoNode * /* node */)
 }
 
 
-SoCallbackAction::Response
+SoCallbackAction::Response 
 SoReorganizeActionP::pre_shape_cb(void * userdata, SoCallbackAction * action, const SoNode * node)
 {
   SoReorganizeActionP * thisp = (SoReorganizeActionP*) userdata;
@@ -547,7 +547,7 @@ SoReorganizeActionP::pre_shape_cb(void * userdata, SoCallbackAction * action, co
   return SoCallbackAction::CONTINUE;
 }
 
-SoCallbackAction::Response
+SoCallbackAction::Response 
 SoReorganizeActionP::post_shape_cb(void * userdata, SoCallbackAction * action, const SoNode * node)
 {
   SoReorganizeActionP * thisp = (SoReorganizeActionP*) userdata;
@@ -560,15 +560,15 @@ SoReorganizeActionP::post_shape_cb(void * userdata, SoCallbackAction * action, c
 #endif // debug
   return SoCallbackAction::CONTINUE;
 }
-
-void
+  
+void 
 SoReorganizeActionP::triangle_cb(void * userdata, SoCallbackAction * action,
                                  const SoPrimitiveVertex * v1,
                                  const SoPrimitiveVertex * v2,
-                                 const SoPrimitiveVertex * v3)
+                                 const SoPrimitiveVertex * v3) 
 {
   SoReorganizeActionP * thisp = (SoReorganizeActionP*) userdata;
-
+  
   if (!thisp->didinit) {
     if (thisp->initShape(action)) {
       assert(thisp->pvcache == NULL);
@@ -583,13 +583,13 @@ SoReorganizeActionP::triangle_cb(void * userdata, SoCallbackAction * action,
   }
 }
 
-void
+void 
 SoReorganizeActionP::line_segment_cb(void * userdata, SoCallbackAction * action,
                                      const SoPrimitiveVertex * v1,
                                      const SoPrimitiveVertex * v2)
 {
   SoReorganizeActionP * thisp = (SoReorganizeActionP*) userdata;
-
+  
   if (!thisp->didinit) {
     if (thisp->initShape(action)) {
       assert(thisp->pvcache == NULL);
@@ -610,25 +610,25 @@ SoReorganizeActionP::initShape(SoCallbackAction * action)
   this->didinit = TRUE;
   SoState * state = action->getState();
   SbBool canrenderasvertexarray = TRUE;
-
+  
   unsigned int shapeflags = SoShapeStyleElement::get(state)->getFlags();
-
+  
   this->lighting = SoLightModelElement::get(state) != SoLightModelElement::BASE_COLOR;
   this->normalsonstate = SoNormalElement::getInstance(state)->getNum() > 0;
-
+  
   SbBool texture0enabled =
     SoTextureEnabledElement::get(state) != FALSE;
 
   this->hastexture = texture0enabled;
 
   int lastenabled;
-  const SbBool * enabledunits =
+  const SbBool * enabledunits = 
     SoMultiTextureEnabledElement::getEnabledUnits(state, lastenabled);
 
   this->needtexcoords.truncate(0);
   this->needtexcoords.append(FALSE);
 
-  if (shapeflags &
+  if (shapeflags & 
       (SoShapeStyleElement::BUMPMAP|
        SoShapeStyleElement::BBOXCMPLX|
        SoShapeStyleElement::INVISIBLE|
@@ -655,7 +655,7 @@ SoReorganizeActionP::initShape(SoCallbackAction * action)
       break;
     }
   }
-
+  
   if (canrenderasvertexarray && enabledunits) {
     const SoMultiTextureCoordinateElement * melem =
       SoMultiTextureCoordinateElement::getInstance(state);
@@ -665,7 +665,7 @@ SoReorganizeActionP::initShape(SoCallbackAction * action)
         // FIXME: multitexturing is not supported yet, since it's not
         // supported by SoVertexProperty. We might fix this later by
         // inserting new nodes though. pederb, 2005-04-28
-        canrenderasvertexarray = FALSE;
+        canrenderasvertexarray = FALSE; 
 
         this->hastexture = TRUE;
         switch (melem->getType(i)) {
@@ -689,19 +689,19 @@ SoReorganizeActionP::initShape(SoCallbackAction * action)
 
   if (canrenderasvertexarray) {
     SbColor diffuse = SoLazyElement::getDiffuse(state, 0);
-    float transp = SoLazyElement::getTransparency(state, 0);
+    float transp = SoLazyElement::getTransparency(state, 0); 
     this->diffusecolor = SbColor4f(diffuse, 1.0f - transp);
   }
 
   return canrenderasvertexarray;
 }
 
-void
+void 
 SoReorganizeActionP::replaceNode(SoFullPath * path)
 {
-  if (this->pvcache == NULL) return;
+  if (this->pvcache == NULL) return;  
   this->pvcache->fit(); // needed to do optimize-sort of data
-
+  
   if (this->pvcache->getNumIndices()) {
     if (this->isvrml) {
       this->replaceVrmlIfs(path);
@@ -722,26 +722,26 @@ SoReorganizeActionP::replaceNode(SoFullPath * path)
   this->pvcache = NULL;
 }
 
-SoVertexProperty *
+SoVertexProperty * 
 SoReorganizeActionP::createVertexProperty(const SbBool forlines)
 {
   SoVertexProperty * vp = new SoVertexProperty;
   vp->ref();
   SoVertexProperty::Binding nbind = SoVertexProperty::PER_VERTEX_INDEXED;
 
-  if (!this->lighting ||
+  if (!this->lighting || 
       (forlines && !this->normalsonstate)) {
     nbind = SoVertexProperty::OVERALL;
-  }
+  }                                          
   vp->normalBinding = nbind;
 
   int numv = this->pvcache->getNumVertices();
-
+  
   if (this->hastexture) {
     vp->texCoord.setNum(numv);
     SbVec2f * dst = vp->texCoord.startEditing();
     const SbVec4f * src = this->pvcache->getTexCoordArray();
-
+    
     for (int i = 0; i < numv; i++) {
       SbVec4f tmp = src[i];
       if (tmp[3] != 0.0f) {
@@ -753,17 +753,17 @@ SoReorganizeActionP::createVertexProperty(const SbBool forlines)
     }
     vp->texCoord.finishEditing();
   }
-
+  
   vp->vertex.setValues(0, numv,
                        this->pvcache->getVertexArray());
   if (nbind == SoVertexProperty::PER_VERTEX_INDEXED) {
     vp->normal.setValues(0, numv,
                          this->pvcache->getNormalArray());
-  }
-
-  vp->materialBinding = SoVertexProperty::OVERALL;
+  }  
+  
+  vp->materialBinding = SoVertexProperty::OVERALL;  
   vp->orderedRGBA = this->diffusecolor.getPackedValue();
-
+  
   if (this->pvcache->colorPerVertex()) {
     vp->materialBinding = SoVertexProperty::PER_VERTEX_INDEXED;
     uint8_t * src = (uint8_t*) this->pvcache->getColorArray();
@@ -779,7 +779,7 @@ SoReorganizeActionP::createVertexProperty(const SbBool forlines)
   return vp;
 }
 
-void
+void 
 SoReorganizeActionP::replaceIfs(SoFullPath * path)
 {
   SoNode * parent = path->getNodeFromTail(1);
@@ -794,17 +794,17 @@ SoReorganizeActionP::replaceIfs(SoFullPath * path)
   ifs->normalIndex.setNum(0);
   ifs->materialIndex.setNum(0);
   ifs->textureCoordIndex.setNum(0);
-
+  
   int numtri = this->pvcache->getNumIndices() / 3;
-  const int32_t * indices = this->pvcache->getIndices();
+  const GLint * indices = this->pvcache->getIndices();
   ifs->coordIndex.setNum(numtri * 4);
   int32_t * ptr = ifs->coordIndex.startEditing();
 
-
+  
   for (int i = 0; i < numtri; i++) {
-    *ptr++ = indices[i*3];
-    *ptr++ = indices[i*3+1];
-    *ptr++ = indices[i*3+2];
+    *ptr++ = (int32_t) indices[i*3];
+    *ptr++ = (int32_t) indices[i*3+1];
+    *ptr++ = (int32_t) indices[i*3+2];
     *ptr++ = -1;
   }
   ifs->coordIndex.finishEditing();
@@ -817,7 +817,7 @@ SoReorganizeActionP::replaceIfs(SoFullPath * path)
   ifs->unrefNoDelete();
 }
 
-void
+void 
 SoReorganizeActionP::replaceVrmlIfs(SoFullPath * path)
 {
   SoNode * parent = path->getNodeFromTail(1);
@@ -828,7 +828,6 @@ SoReorganizeActionP::replaceVrmlIfs(SoFullPath * path)
 
   SoVRMLIndexedFaceSet * oldifs = (SoVRMLIndexedFaceSet*) path->getTail();
   assert(oldifs->isOfType(SoVRMLIndexedFaceSet::getClassTypeId()));
-
   SoVRMLIndexedFaceSet * ifs = new SoVRMLIndexedFaceSet;
   ifs->ref();
   ifs->normalPerVertex = this->lighting;
@@ -836,15 +835,15 @@ SoReorganizeActionP::replaceVrmlIfs(SoFullPath * path)
   ifs->ccw = oldifs->ccw;
   ifs->solid = oldifs->solid;
   ifs->creaseAngle = oldifs->creaseAngle;
-
+  
   int numv = this->pvcache->getNumVertices();
-
+  
   if (this->hastexture) {
     SoVRMLTextureCoordinate * tc = new SoVRMLTextureCoordinate;
     tc->point.setNum(numv);
     SbVec2f * dst = tc->point.startEditing();
     const SbVec4f * src = this->pvcache->getTexCoordArray();
-
+    
     for (int i = 0; i < numv; i++) {
       SbVec4f tmp = src[i];
       if (tmp[3] != 0.0f) {
@@ -868,7 +867,7 @@ SoReorganizeActionP::replaceVrmlIfs(SoFullPath * path)
     norm->vector.setValues(0, numv,
                            this->pvcache->getNormalArray());
     ifs->normal = norm;
-  }
+  }    
   if (this->pvcache->colorPerVertex()) {
     SoVRMLColor * col = new SoVRMLColor;
     col->color.setNum(numv);
@@ -887,16 +886,16 @@ SoReorganizeActionP::replaceVrmlIfs(SoFullPath * path)
   ifs->normalIndex.setNum(0);
   ifs->colorIndex.setNum(0);
   ifs->texCoordIndex.setNum(0);
-
+  
   int numtri = this->pvcache->getNumIndices() / 3;
-  const int32_t * indices = this->pvcache->getIndices();
+  const GLint * indices = this->pvcache->getIndices();
   ifs->coordIndex.setNum(numtri * 4);
   int32_t * ptr = ifs->coordIndex.startEditing();
 
   for (int i = 0; i < numtri; i++) {
-    *ptr++ = indices[i*3];
-    *ptr++ = indices[i*3+1];
-    *ptr++ = indices[i*3+2];
+    *ptr++ = (int32_t) indices[i*3];
+    *ptr++ = (int32_t) indices[i*3+1];
+    *ptr++ = (int32_t) indices[i*3+2];
     *ptr++ = -1;
   }
   ifs->coordIndex.finishEditing();
@@ -915,7 +914,7 @@ SoReorganizeActionP::replaceVrmlIfs(SoFullPath * path)
   ifs->unrefNoDelete();
 }
 
-void
+void 
 SoReorganizeActionP::replaceIls(SoFullPath * path)
 {
   SoNode * parent = path->getNodeFromTail(1);
@@ -930,20 +929,20 @@ SoReorganizeActionP::replaceIls(SoFullPath * path)
   ils->normalIndex.setNum(0);
   ils->materialIndex.setNum(0);
   ils->textureCoordIndex.setNum(0);
-
+  
   int numlines = this->pvcache->getNumLineIndices() / 2;
-  const int32_t * indices = this->pvcache->getLineIndices();
+  const GLint * indices = this->pvcache->getLineIndices();
   ils->coordIndex.setNum(numlines * 3);
   int32_t * ptr = ils->coordIndex.startEditing();
-
+  
   for (int i = 0; i < numlines; i++) {
-    *ptr++ = indices[i*2];
-    *ptr++ = indices[i*2+1];
+    *ptr++ = (int32_t) indices[i*2];
+    *ptr++ = (int32_t) indices[i*2+1];
     *ptr++ = -1;
   }
   ils->coordIndex.finishEditing();
 
-  int idx = path->getIndexFromTail(0);
+  int idx = path->getIndexFromTail(0);  
   path->pop();
   SoGroup * g = (SoGroup*)parent;
   g->replaceChild(idx, ils);
@@ -951,9 +950,65 @@ SoReorganizeActionP::replaceIls(SoFullPath * path)
   ils->unrefNoDelete();
 }
 
-void
+void 
 SoReorganizeActionP::replaceVrmlIls(SoFullPath * path)
 {
+  SoNode * parent = path->getNodeFromTail(1);
+  if (!parent->isOfType(SoGroup::getClassTypeId()) &&
+      !parent->isOfType(SoVRMLShape::getClassTypeId())) {
+    return;
+  }
+
+  SoVRMLIndexedLineSet * ils = new SoVRMLIndexedLineSet;
+  ils->ref();
+
+  int numv = this->pvcache->getNumVertices();
+  int numlines = this->pvcache->getNumLineIndices() / 2;
+  const GLint * indices = this->pvcache->getLineIndices();
+  ils->coordIndex.setNum(numlines * 3);
+  int32_t * ptr = ils->coordIndex.startEditing();
+  
+  for (int i = 0; i < numlines; i++) {
+    *ptr++ = (int32_t) indices[i*2];
+    *ptr++ = (int32_t) indices[i*2+1];
+    *ptr++ = -1;
+  }
+  ils->coordIndex.finishEditing();
+
+  SoVRMLCoordinate * c = new SoVRMLCoordinate;
+  c->point.setValues(0, numv,
+                     this->pvcache->getVertexArray());
+  ils->coord = c;
+  
+  if (this->pvcache->colorPerVertex()) {
+    ils->colorPerVertex = TRUE;
+    SoVRMLColor * col = new SoVRMLColor;
+    col->color.setNum(numv);
+    uint8_t * src = (uint8_t*) this->pvcache->getColorArray();
+    SbColor * dst = col->color.startEditing();
+    for (int i = 0; i < numv; i++) {
+      dst[i] = SbColor(src[0]/255.0f,
+                       src[1]/255.0f,
+                       src[2]/255.0f);
+      src += 4;
+    }
+    col->color.finishEditing();
+    ils->color = col;
+  }
+  ils->colorIndex.setNum(0);
+  
+  int idx = path->getIndexFromTail(0);
+  path->pop();
+  if (parent->isOfType(SoGroup::getClassTypeId())) {
+    SoGroup * g = (SoGroup*)parent;
+    g->replaceChild(idx, ils);
+  }
+  else {
+    SoVRMLShape * shape = (SoVRMLShape*) parent;
+    shape->geometry = ils;
+  }
+  path->push(idx);
+  ils->unrefNoDelete();
 }
 
 #endif // COIN_NEXT_MINOR
