@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -31,7 +31,7 @@
   of the defining corners of a box in 3D space, with the sides aligned
   with the 3 principal axes.
 
-  \sa SbBox2f, SbBox2s, SbXfBox3f
+  \sa SbBox2s, SbBox2f, SbBox2d, SbBox3s, SbXfBox3f.
 */
 
 
