@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -31,7 +31,7 @@
   floating point coordinates, and for a floating point 4x4 transformation
   matrix.
 
-  \sa SbBox3f, SbBox2f, SbBox2s, SbMatrix
+  \sa SbBox2s, SbBox2f, SbBox2d, SbBox3s, SbBox3f, SbMatrix.
 */
 
 #include <Inventor/SbXfBox3f.h>
