@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -122,7 +122,7 @@ public:
   void addVertex(const SoPrimitiveVertexCache::Vertex & v);
 
   void renderImmediate(const cc_glglue * glue,
-                       const int32_t * indices,
+                       const GLint * indices,
                        const int numindices,
                        const SbBool color, const SbBool normal,
                        const SbBool texture, const SbBool * enabled,
@@ -659,7 +659,7 @@ SoPrimitiveVertexCache::getNumIndices(void) const
   return PRIVATE(this)->triangleindexer ? PRIVATE(this)->triangleindexer->getNumIndices() : 0;
 }
 
-const int32_t *
+const GLint *
 SoPrimitiveVertexCache::getIndices(void) const
 {
   assert(PRIVATE(this)->triangleindexer);
@@ -699,14 +699,14 @@ SoPrimitiveVertexCache::getNumPointIndices(void) const
 }
 
 
-const int32_t *
+const GLint *
 SoPrimitiveVertexCache::getLineIndices(void) const
 {
   assert(PRIVATE(this)->lineindexer);
   return PRIVATE(this)->lineindexer->getIndices();
 }
 
-const int32_t *
+const GLint *
 SoPrimitiveVertexCache::getPointIndices(void) const
 {
   assert(PRIVATE(this)->pointindexer);
@@ -747,7 +747,7 @@ SoPrimitiveVertexCache::depthSortTriangles(SoState * state)
     PRIVATE(this)->prevsortplane = sortplane;
     float * darray = PRIVATE(this)->deptharray;
     const SbVec3f * vptr = PRIVATE(this)->vertexlist.getArrayPtr();
-    int32_t * iptr = PRIVATE(this)->triangleindexer->getWriteableIndices();
+    GLint * iptr = PRIVATE(this)->triangleindexer->getWriteableIndices();
     int i,j;
     for (i = 0; i < numtri; i++) {
       float acc = 0.0;
@@ -805,7 +805,16 @@ SoPrimitiveVertexCache::Vertex::operator unsigned long(void) const
 int 
 SoPrimitiveVertexCache::Vertex::operator==(const Vertex & v)
 {
-  return memcmp(this, &v, sizeof(v)) == 0;
+  return 
+    (this->vertex == v.vertex) &&
+    (this->normal == v.normal) &&
+    (this->texcoord0 == v.texcoord0) &&
+    (this->bumpcoord == v.bumpcoord) &&
+    (this->texcoordidx == v.texcoordidx) &&
+    (this->rgba[0] == v.rgba[0]) &&
+    (this->rgba[1] == v.rgba[1]) &&
+    (this->rgba[2] == v.rgba[2]) &&
+    (this->rgba[3] == v.rgba[3]);
 }
 
 void 
@@ -967,7 +976,7 @@ SoPrimitiveVertexCacheP::disableVBOs(const cc_glglue * glue,
 
 void 
 SoPrimitiveVertexCacheP::renderImmediate(const cc_glglue * glue,
-                                         const int32_t * indices,
+                                         const GLint * indices,
                                          const int numindices,
                                          const SbBool color, const SbBool normal,
                                          const SbBool texture, const SbBool * enabled,
