@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -164,7 +164,7 @@ create_matrix_bitmap(int intensity, unsigned char * bitmap,
 
 // HACK. Replacement for GLState since we needed some more data
 // members without breaking the ABI.  It's fixed properly in Coin-3
-// (Coin development CVS).
+// (Coin development SVN).
 typedef struct {
   // these are the same as GLState
   uint32_t cachebitmask;
@@ -193,12 +193,16 @@ typedef struct {
   uint32_t transpnodeid;
 } SoInternalGLState;
 
+/* FIXME: Horribly dirty void* cast added to calm down a gcc 4.1.1
+ * type-punned pointer compiler warning. 20070308 tamer. */
 // defines used to cast GLState to SoInternalGLState
 #define UPDATE_GLSTATE(_ptr_, _name_, _val_) \
-  ((SoInternalGLState*)&((_ptr_)->glstate))->_name_ = _val_  
+  ((SoInternalGLState*)(void*)&((_ptr_)->glstate))->_name_ = _val_
 
+/* FIXME: Horribly dirty void* cast added to calm down a gcc 4.1.1
+ * type-punned pointer compiler warning. 20070308 tamer. */
 #define GLSTATE(_ptr_) \
-  ((SoInternalGLState*)&((_ptr_)->glstate))
+  ((SoInternalGLState*)(void*)&((_ptr_)->glstate))
 
 #define OPENCACHEFLAGS(_ptr_) \
   (_ptr_)->forcediffuse
