@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -83,6 +83,7 @@ lazyelement_cleanup(void)
   delete lazy_defaultindex;
   delete lazy_defaultpacked;
   delete lazy_unpacked;
+  lazy_defaultdiffuse = NULL; // Only need to NULL this; see initClass().
 }
 
 // helper functions to handle default diffuse/transp values
@@ -127,7 +128,7 @@ SoLazyElement::initClass()
     *lazy_defaultindex = getDefaultColorIndex();
     *lazy_defaultpacked = getDefaultPacked();
 
-    coin_atexit((coin_atexit_f*) lazyelement_cleanup, 0);
+    coin_atexit((coin_atexit_f*) lazyelement_cleanup, CC_ATEXIT_NORMAL);
   }
 }
 
@@ -728,10 +729,12 @@ SoLazyElement::setMaterials(SoState * state, SoNode *node, uint32_t bitmask,
     if (welem) elem = welem;
     elem->lazyDidntSet((~eltbitmask) & bitmask);
   }
-  SoShapeStyleElement::setTransparentMaterial(state, istransparent);
-  SoLazyElement::setAlphaTest(state, 
-                              !istransparent && 
-                              elem->coinstate.glimageusealphatest);
+  if (bitmask & TRANSPARENCY_MASK) {
+    SoShapeStyleElement::setTransparentMaterial(state, istransparent);
+    SoLazyElement::setAlphaTest(state, 
+                                !istransparent && 
+                                elem->coinstate.glimageusealphatest);
+  }
 }
 
 void 
