@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -67,23 +67,17 @@ SO_ELEMENT_SOURCE(SoProfileCoordinateElement);
 void
 SoProfileCoordinateElement::initClass(void)
 {
-#if COIN_DEBUG
-  // Debugging for memory leaks will be easier if we can clean up the
-  // resource usage.
-  coin_atexit((coin_atexit_f *)SoProfileCoordinateElement::clean, 0);
-#endif // COIN_DEBUG
-
   SO_ELEMENT_INIT_CLASS(SoProfileCoordinateElement, inherited);
 
   SoProfileCoordinateElement::initdefaultcoords = new SbVec2f(0.0f, 0.0f);
+
+  coin_atexit((coin_atexit_f *)SoProfileCoordinateElement::clean, CC_ATEXIT_NORMAL);
 }
 
 void
 SoProfileCoordinateElement::clean(void)
 {
-#if COIN_DEBUG
   delete SoProfileCoordinateElement::initdefaultcoords;
-#endif // COIN_DEBUG
 }
 
 /*!
