@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -138,6 +138,8 @@
 #endif // !COIN_THREADSAFE
 
 static const int SOFIELD_GET_STACKBUFFER_SIZE = 1024;
+// need one static mutex for field_buffer in SoField::get(SbString &)
+static void * sofield_mutex = NULL;
 
 // flags for this->statusbits
 
@@ -381,12 +383,15 @@ SoFieldP::hashExitCleanup(void)
 void *
 SoFieldP::hashRealloc(void * bufptr, size_t size)
 {
+  CC_MUTEX_LOCK(sofield_mutex);
+  
   char ** bufptrptr = NULL;
   int ok = SoFieldP::ptrhash->get((char *)bufptr, bufptrptr);
   assert(ok);
-
+  
   // If *bufptrptr contains a NULL pointer, this is the first
   // invocation and the initial memory buffer was on the stack.
+  
   char * newbuf;
   if (*bufptrptr == NULL) {
     // if initial buffer was on the stack, we need to manually copy
@@ -403,7 +408,9 @@ SoFieldP::hashRealloc(void * bufptr, size_t size)
     *bufptrptr = newbuf;
     SoFieldP::ptrhash->put(newbuf, bufptrptr);
   }
-
+  
+  CC_MUTEX_UNLOCK(sofield_mutex);
+  
   return newbuf;
 }
 
@@ -577,9 +584,6 @@ SoField::~SoField()
   this->clearStatusBits(FLAG_ALIVE_PATTERN);
 }
 
-// need one static mutex for field_buffer in SoField::get(SbString &)
-static void * sofield_mutex = NULL;
-
 // atexit
 static void
 field_mutex_cleanup(void)
@@ -1278,18 +1282,16 @@ SoField::get(SbString & valuestring)
   // NOTE: this code has an almost verbatim copy in SoMField::get1(),
   // so remember to update both places if any fixes are done.
 
-  if (sofield_mutex) {
-    CC_MUTEX_LOCK(sofield_mutex); // need to lock since a static array is used
-  }
-
   // Initial buffer setup.
   SoOutput out;
   char initbuffer[SOFIELD_GET_STACKBUFFER_SIZE];
   char * bufferptr = NULL; // indicates that initial buffer is on the stack
-
+  
+  CC_MUTEX_LOCK(sofield_mutex);
   int ok = SoFieldP::getReallocHash()->put(initbuffer, &bufferptr);
   assert(ok);
-
+  CC_MUTEX_UNLOCK(sofield_mutex);
+  
   out.setBuffer(initbuffer, sizeof(initbuffer), SoFieldP::hashRealloc);
 
   // Record offset to skip header.
@@ -1308,15 +1310,14 @@ SoField::get(SbString & valuestring)
   size_t size;
   out.getBuffer(buffer, size);
   valuestring = ((char *)buffer) + offset;
-
+  
   // dealloc tmp memory buffer
   if (bufferptr) { free(bufferptr); }
+  
+  CC_MUTEX_LOCK(sofield_mutex);
   ok = SoFieldP::getReallocHash()->remove(bufferptr ? bufferptr : initbuffer);
   assert(ok);
-
-  if (sofield_mutex) {
-    CC_MUTEX_UNLOCK(sofield_mutex);
-  }
+  CC_MUTEX_UNLOCK(sofield_mutex);
 }
 
 /*!
@@ -1791,11 +1792,9 @@ SoField::read(SoInput * in, const SbName & name)
 
     if (c == IGNOREDCHAR) this->setIgnored(TRUE);
     else {
-      in->putBack(c);
-
       // First check if there's a field-to-field connection here as
       // the default value following the field can be omitted.
-      if (in->read(c)) { // if-check in case EOF on an SoField::set() invocation
+      if (c == CONNECTIONCHAR) { 
         // There's potential for an obscure bug to happen here: if the
         // field is an SoSFString where the string is unquoted and
         // starts with a CONNECTIONCHAR (i.e. '='), it will lead to a
@@ -1820,12 +1819,10 @@ SoField::read(SoInput * in, const SbName & name)
         // FIXME: it would be nice if we could improve the error
         // message, to let the app programmer actually stand a chance
         // of debugging this when it happens. 20030811 mortene.
-        if (c == CONNECTIONCHAR) { 
-          if (!this->readConnection(in)) { readok = FALSE; goto sofield_read_return; }
-          goto sofield_read_return;
-        }
-        else in->putBack(c);
+        if (!this->readConnection(in)) { readok = FALSE; goto sofield_read_return; }
+        goto sofield_read_return;
       }
+      else in->putBack(c);
       
       // Read field value(s).
       if (!this->readValue(in)) {
