@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -181,29 +181,48 @@ SoSFEnum::findEnumName(int valuearg, const SbName *& name) const
 SbBool
 SoSFEnum::readValue(SoInput * in)
 {
-  // FIXME: in this case, perhaps we should rather accept numeric
-  // values instead of demanding mnemonics? 20020630 mortene.
-  if (!this->legalValuesSet) {
-    SbName name;
-    SoFieldContainer * thecontainer = this->getContainer();
-    SbBool fname = thecontainer && thecontainer->getFieldName(this, name);
-    SoReadError::post(in,
-                      "no mappings available for SoSFEnum field %s",
-                      fname ? name.getString() : "");
-    return FALSE;
-  }
-
   SbName n;
+  int val;
+
   // Read mnemonic value as a character string identifier
   if (!in->read(n, TRUE)) {
-    SoReadError::post(in, "Couldn't read enumeration name");
-    return FALSE;
+    // If we don't have any legal values for this field,
+    // give some slack and accept integer values.
+    if (this->legalValuesSet || !in->read(val)) {
+      SoReadError::post(in, "Couldn't read enumeration name");
+      return FALSE;
+    }
   }
-
-  int val;
-  if (!this->findEnumValue(n, val)) {
-    SoReadError::post(in, "Unknown enumeration value \"%s\"", n.getString());
-    return FALSE;
+  else {
+    if (!this->findEnumValue(n, val)) {
+      // If we read an enum field written by an extension node,
+      // we won't have any defined enum values. This is indicated by
+      // this->legalValuesSet == FALSE. If this is the case, define
+      // any encountered enum values on the fly but keep legalValuesSet
+      // to FALSE in order not to fool built-in enum field to accept
+      // illegal values.
+      if (!this->legalValuesSet) {
+        int *newvalues = new int[this->numEnums+1];
+        SbName *newnames = new SbName[this->numEnums+1];
+        int i;
+        for (i = 0; i < this->numEnums; i++) {
+          newvalues[i] = this->enumValues[i];
+          newnames[i] = this->enumNames[i];
+        }
+        newvalues[i] = i;
+        newnames[i] = n;
+        delete[] this->enumValues;
+        delete[] this->enumNames;
+        this->enumValues = newvalues;
+        this->enumNames = newnames;
+        this->numEnums += 1;
+        val = i;
+      }
+      else {
+        SoReadError::post(in, "Unknown enumeration value \"%s\"", n.getString());
+        return FALSE;
+      }
+    }
   }
   this->value = val;
   return TRUE;
@@ -212,12 +231,16 @@ SoSFEnum::readValue(SoInput * in)
 void
 SoSFEnum::writeValue(SoOutput * out) const
 {
-  int val = this->getValue();
-  for (int i = 0; i < this->numEnums; i++) {
-    if (this->enumValues[i] == val) {
-      out->write((char *)this->enumNames[i].getString());
-      return;
-    }
+  const SbName *enumname;
+  if (findEnumName(this->getValue(), enumname)) {
+    out->write((char *)enumname->getString());
+    return;
+  }
+  // If we don't have any legal values for this field,
+  // pass through read integer values.
+  if (!this->legalValuesSet) {
+    out->write(this->getValue());
+    return;
   }
 
 #if COIN_DEBUG
@@ -233,7 +256,7 @@ SoSFEnum::writeValue(SoOutput * out) const
 
   SoDebugError::post("SoSFEnum::writeValue",
                      "Illegal enumeration value %d in field%s",
-                     val, s.getString());
+                     this->getValue(), s.getString());
 #endif // COIN_DEBUG
 }
 
