@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -132,9 +132,6 @@ static COIN_AGLTEXIMAGEPBUFFER aglglue_aglTexImagePBuffer = NULL;
 struct aglglue_contextdata;
 static SbBool (* aglglue_context_create)(struct aglglue_contextdata * ctx) = NULL;
 
-static cc_libhandle glhnd = NULL;
-static SbBool tried_fetching_handle = FALSE;
-
 /* 
  * Since AGL does not have an aglGetProcAddress() as such, this is
  * simply a wrapper around cc_dl_*. Present here for consistency with
@@ -143,11 +140,7 @@ static SbBool tried_fetching_handle = FALSE;
 void *
 aglglue_getprocaddress(const char * fname)
 {
-  if (!tried_fetching_handle) {
-    glhnd = cc_dl_handle_with_gl_symbols();
-    tried_fetching_handle = TRUE;
-  }
-
+  cc_libhandle glhnd = cc_dl_handle_with_gl_symbols();
   return glhnd ? cc_dl_sym(glhnd, fname) : NULL;
 }
 
@@ -230,6 +223,15 @@ aglglue_contextdata_cleanup(struct aglglue_contextdata * ctx)
 static SbBool
 aglglue_context_create_software(struct aglglue_contextdata * ctx)
 {
+  /* FIXME: we're currently not giving any hint as to whether we want
+     a single- / double- / quad-buffer visual. Using anything than a
+     single-buffer visual for offscreen rendering is probably
+     wasteful, so we're likely to often use up more resources than we
+     really need to do.
+
+     20061025 mortene.
+  */
+
   GLint attrib[] = {
     AGL_OFFSCREEN,
     AGL_RGBA,
@@ -654,13 +656,5 @@ aglglue_cleanup(void)
   aglglue_aglTexImagePBuffer = NULL;
 
   aglglue_context_create = NULL;
-
-  if (glhnd) { 
-    cc_dl_close(glhnd); 
-    glhnd = NULL; 
-  }
-  tried_fetching_handle = FALSE;
 }
 #endif /* HAVE_AGL */
-
-
