@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -168,8 +168,7 @@ static COIN_PFNWGLGETEXTENSIONSSTRING wglglue_wglGetExtensionsString = NULL;
 
 /* ********************************************************************** */
 
-static cc_libhandle glhnd = NULL;
-static SbBool tried_fetching_handle = FALSE;
+static SbBool attemptedextresolved = FALSE;
 
 /* ********************************************************************** */
 
@@ -183,16 +182,12 @@ coin_wgl_getprocaddress(const char * fname)
      (1.1+) functions. */
 
   if (ptr == NULL) {
-    if (!tried_fetching_handle) {
-      /* FIXME: never released -- one-off resource leak. 20041208 mortene.*/
-      glhnd = cc_dl_handle_with_gl_symbols();
-      tried_fetching_handle = TRUE;
+    cc_libhandle glhnd = cc_dl_handle_with_gl_symbols();
 
-      if (!glhnd && coin_glglue_debug()) {
-        cc_debugerror_postwarning("coin_wgl_getprocaddress",
-                                  "couldn't get hold of any workable module "
-                                  "handle for picking up OpenGL symbols");
-      }
+    if (!glhnd && coin_glglue_debug()) {
+      cc_debugerror_postwarning("coin_wgl_getprocaddress",
+                                "couldn't get hold of any workable module "
+                                "handle for picking up OpenGL symbols");
     }
 
     if (glhnd) {
@@ -253,10 +248,8 @@ wglglue_ext_supported(struct wglglue_contextdata * context, const char * reqext)
    * ..so we try to get hold of wglGetExtensionsString[ARB|EXT]() and
    * use that aswell.
    */
-  static SbBool attemptedresolved = FALSE;
-
-  if (!attemptedresolved) {
-    attemptedresolved = TRUE;
+  if (!attemptedextresolved) {
+    attemptedextresolved = TRUE;
 #ifdef WGL_ARB_extensions_string
     wglglue_wglGetExtensionsString = (COIN_PFNWGLGETEXTENSIONSSTRING)PROC(wglGetExtensionsStringARB);
 #endif /* WGL_ARB_extensions_string */
@@ -452,6 +445,15 @@ wglglue_context_create_context(struct wglglue_contextdata * ctx, DWORD bitWin)
   struct wglglue_contextdata * context = (struct wglglue_contextdata *)ctx;
   int pixelformat;
 
+  /* FIXME: we're currently not giving any hint as to whether we want
+     a single- / double- / quad-buffer visual. Using anything than a
+     single-buffer visual for offscreen rendering is probably
+     wasteful, so we're likely to often use up more resources than we
+     really need to do.
+
+     20061025 mortene.
+  */
+
   PIXELFORMATDESCRIPTOR pfd = {
     sizeof(PIXELFORMATDESCRIPTOR),   /* size of this pfd */
     1,                     /* version number */
@@ -465,13 +467,39 @@ wglglue_context_create_context(struct wglglue_contextdata * ctx, DWORD bitWin)
     0,                     /* no accumulation buffer */
     0, 0, 0, 0,            /* accum bits ignored */
     32,                    /* 32-bit z-buffer */
-    0,                     /* no stencil buffer */
+    1,                     /* minimum size stencil buffer */
     0,                     /* no auxiliary buffer */
     PFD_MAIN_PLANE,        /* main layer */
     0,                     /* reserved */
     0, 0, 0                /* layer masks ignored */
   };
 
+  /* FIXME: the following is a hack to get around a problem which
+     really demands more effort to be solved properly.
+
+     The problem is that there is no way in the API of the
+     SoOffscreenRenderer class to specify what particular attributes
+     to request. This most often manifests itself as a problem for app
+     programmers in that they have made some kind of extension node
+     which uses the OpenGL stencil buffer. If no stencil buffer
+     happens to be part of the GL context format for the offscreen
+     renderer, these will not work properly. At the same time, we
+     don't want to default to requesting a stencil buffer, as that
+     takes a non-trivial amount of extra memory resources on the gfx
+     card.
+
+     So until we have implemented the proper solution for making it
+     possible to pass in a detailed specification of which attributes
+     to request from offscreen GL contexts, we provide this temporary
+     work-around: the app programmer can set an envvar with a value
+     specifying the number of stencil buffer bits he/she wants.
+
+     20060223 mortene.
+  */
+  const int v = coin_glglue_stencil_bits_hack();
+  if (v != -1) { pfd.cStencilBits = (BYTE)v; }
+
+
   /* get the best available match of pixel format for the device
      context: */
 
@@ -643,6 +671,8 @@ wglglue_context_create_pbuffer(struct wglglue_contextdata * ctx, SbBool warnoner
   else { context->noappglcontextavail = TRUE; }
 
   if (context->noappglcontextavail) {
+    /* FIXME: This should be reset in wglglue_cleanup() once we
+       also properly unregister there... 20060207 kyrah */
     static int didregister = 0;
     if (!didregister) {
       WNDCLASS wc;
@@ -729,14 +759,22 @@ wglglue_context_create_pbuffer(struct wglglue_contextdata * ctx, SbBool warnoner
     unsigned int numFormats;
     const float fAttribList[] = { 0 };
 
-    const int nontex_attrs[] = {
+    int nontex_attrs[] = {
+      WGL_STENCIL_BITS_ARB, 1, /* FIXME: must be first, since we may
+                                  want to modify it later, and need to
+                                  know where it is. This is a
+                                  wart. 20060307 mortene. */
       WGL_DRAW_TO_PBUFFER_ARB, TRUE,
       WGL_COLOR_BITS_ARB, 32,
       WGL_ALPHA_BITS_ARB, 8,
       WGL_DEPTH_BITS_ARB, 24,
       0
     };
-    const int tex_attrs[] = {
+    int tex_attrs[] = {
+      WGL_STENCIL_BITS_ARB, 1, /* FIXME: must be first, since we may
+                                  want to modify it later, and need to
+                                  know where it is. This is a
+                                  wart. 20060307 mortene. */
       WGL_DRAW_TO_PBUFFER_ARB, TRUE,
       WGL_BIND_TO_TEXTURE_RGBA_ARB, TRUE,
       WGL_COLOR_BITS_ARB, 32,
@@ -754,8 +792,25 @@ wglglue_context_create_pbuffer(struct wglglue_contextdata * ctx, SbBool warnoner
     };
     const int * pbufferflags[] = { nontex_pbufferflags, tex_pbufferflags };
 
+    unsigned int try;
+
+    {
+      /* FIXME: the following is a hack to get around a problem which
+         really demands more effort to be solved properly. See further
+         up in this source file, in the function to create a software
+         context, for a more elaborate explanation. 20060307 mortene.
+      */
+      const int v = coin_glglue_stencil_bits_hack();
+      if (v != -1) {
+        assert(nontex_attrs[0] == WGL_STENCIL_BITS_ARB);
+        nontex_attrs[1] = v;
+        assert(tex_attrs[0] == WGL_STENCIL_BITS_ARB);
+        tex_attrs[1] = v;
+      }
+    }
+
     /* iterate from end of arrays, which contains "best" option */
-    unsigned int try = (sizeof(attrs) / sizeof(attrs[0]));
+    try = (sizeof(attrs) / sizeof(attrs[0]));
     /* if render-to-texture extension not supported, don't attempt to
        set up a pbuffer with those capabilities (could in theory cause
        nasty WGL errors): */
@@ -1113,6 +1168,34 @@ wglglue_context_pbuffer_max(void * c, unsigned int * lims)
   return TRUE;
 }
 
+/* ********************************************************************** */
+
+void wglglue_cleanup(void)
+{
+  wglglue_context_create = NULL;
+
+  wglglue_wglCreatePbuffer = NULL;
+  wglglue_wglGetPbufferDC = NULL;
+  wglglue_wglReleasePbufferDC = NULL;
+  wglglue_wglDestroyPbuffer = NULL;
+  wglglue_wglQueryPbuffer = NULL;
+  wglglue_wglBindTexImageARB = NULL;
+  wglglue_wglReleaseTexImageARB = NULL;
+
+  wglglue_wglChoosePixelFormat = NULL;
+  wglglue_wglGetPixelFormatAttribiv = NULL;
+
+  wglglue_wglGetExtensionsString = NULL;
+
+  attemptedextresolved = FALSE;
+
+  /* FIXME: We should probably do an UnregisterClass() here (see
+     wglglue_context_create_pbuffer()) -- but since I don't know the
+     details of how to do this, I'll leave it to one of the Windows
+     gurus...  20060207 kyrah */
+}
+
+
 #endif /* HAVE_WGL */
 
 /* ********************************************************************** */
