@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -289,6 +289,7 @@ spidermonkey(void)
   REGISTER_FUNC(JS_DefineFunction, JS_DefineFunction_t);
   REGISTER_FUNC(JS_GetGlobalObject, JS_GetGlobalObject_t);
   REGISTER_FUNC(JS_SetGCCallback, JS_SetGCCallback_t);
+  REGISTER_FUNC(JS_MaybeGC, JS_MaybeGC_t);
   REGISTER_FUNC(JS_GC, JS_GC_t);
   REGISTER_FUNC(JS_IsRunning, JS_IsRunning_t);
   REGISTER_FUNC(JS_DeleteProperty, JS_DeleteProperty_t);
@@ -308,6 +309,8 @@ spidermonkey(void)
   REGISTER_FUNC(JS_ToggleOptions, JS_ToggleOptions_t);
   REGISTER_FUNC(JS_Enumerate, JS_Enumerate_t);
   REGISTER_FUNC(JS_IdToValue, JS_IdToValue_t);
+  REGISTER_FUNC(JS_GetFunctionName, JS_GetFunctionName_t);
+  REGISTER_FUNC(JS_GetConstructor, JS_GetConstructor_t);
 
 wrapperexit:
   CC_SYNC_END(spidermonkey);
