@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -173,7 +173,6 @@ soinput_delete_private_data(const SoInput * thisp)
   }
 }
 
-#undef PRIVATE
 #define PRIVATE(obj) (soinput_get_private_data(obj))
 
 // *************************************************************************
@@ -236,7 +235,7 @@ SoInputP::debugBinary(void)
 // Helper function that pops the stack when the current file is at
 // EOF.  Then it returns the file at the top of the stack.
 SoInput_FileInfo * 
-SoInputP::getTopOfStackPopOnEOF(void)
+SoInputP::getTopOfStackPopOnEOF(SoInput * owner)
 {
   SoInput_FileInfo * fi = owner->getTopOfStack();
   assert(fi); // Should always have a top of stack, because the last
@@ -254,7 +253,7 @@ SoInputP::getTopOfStackPopOnEOF(void)
   return fi;
 }
 
-// Helperfunction to handle different filetypes (Inventor, VRML 1.0
+// Helperfunctions to handle different filetypes (Inventor, VRML 1.0
 // and VRML 2.0).
 //
 // VRML 1.0 identifiers are defined as:
@@ -294,136 +293,182 @@ SoInputP::getTopOfStackPopOnEOF(void)
 //  IdRestChars ::= Any number of ISO-10646 characters except:
 //  0x0-0x20, 0x22, 0x23, 0x27, 0x2c, 0x2e, 0x5b, 0x5c, 0x5d,
 //  0x7b, 0x7d, 0x7f ;
+
 SbBool
 SoInputP::isNameStartChar(unsigned char c, SbBool validIdent)
 {
-  // Cannot be unsigned when using strchr, but since all values are
-  // below 128, it does not matter if they are signed or unsigned.
-  const char invalid_vrml1[] = { 0x22, 0x23, 0x27, 0x2b, 0x2c, 0x2e, 
-                                 0x5c, 0x7b, 0x7d, 0x00 }; // 0x7d = 125
-                               // '"',  '#',  ''',  '+',  ',',  '.',
-                               // '\',  '{',  '}'
+  if (validIdent) return SbName::isIdentStartChar(c);
+  return (c > 0x20); // Not control characters
+}
 
-  const char invalid_vrml2[] = { 0x22, 0x23, 0x27, 0x2b, 0x2c, 0x2d, 0x2e, 
-                                 0x5b, 0x5c, 0x5d, 0x7b, 0x7d, 0x7f, 0x00 }; // 0x7f = 127
-                               // '"',  '#',  ''',  '+',  ',',  '-',  '.',
-                               // '[',  '\',  ']',  '{',  '}',   ''
+SbBool
+SoInputP::isNameStartCharVRML1(unsigned char c, SbBool validIdent)
+{
+  static unsigned char invalid_vrml1_table[256];
+  static unsigned char valid_ident_invalid_vrml1_table[256];
 
-  // Differences from invalid_vrml1: '&' , '[', and ']' are now invalid
-  const char valid_ident_invalid_vrml1[] = { 0x22, 0x23, 0x26, 0x27, 0x2b, 0x2c, 0x2e, 
-                                             0x5b, 0x5c, 0x5d, 0x7b, 0x7d, 0x00 }; // 0x7d = 125
-                                           // '"',  '#',   '&', ''',  '+',  ',',  '.',
-                                           // '[',  '\',   ']',  '{',  '}'
+  static int isNameStartCharVRML1Initialized = FALSE;
+  if (!isNameStartCharVRML1Initialized) {
+    const unsigned char invalid_vrml1[] = {
+      0x22, 0x23, 0x27, 0x2b, 0x2c, 0x2e, 0x5c, 0x7b, 0x7d, 0x00 }; // 0x7d = 125
+    //'"',  '#',  ''',  '+',  ',',  '.',  '\',  '{',  '}'
 
-  const char * valid_ident_invalid_vrml2 = invalid_vrml2;
+    // Differences from invalid_vrml1: '&' , '[', and ']' are now invalid
+    const unsigned char valid_ident_invalid_vrml1[] = {
+      0x22, 0x23, 0x26, 0x27, 0x2b, 0x2c, 0x2e, 0x5b, 0x5c, 0x5d, 0x7b, 0x7d, 0x00 }; // 0x7d = 125
+    //'"',  '#',   '&', ''',  '+',  ',',  '.',  '[',  '\',   ']',  '{',  '}'
 
+    for (int c = 0; c < 256; ++c) {
+      invalid_vrml1_table[c] = 0;
+      valid_ident_invalid_vrml1_table[c] = 0;
+    }
+    const unsigned char * ptr = invalid_vrml1;
+    while (*ptr) { invalid_vrml1_table[*ptr] = 1; ++ptr; }
+    ptr = valid_ident_invalid_vrml1;
+    while (*ptr) { valid_ident_invalid_vrml1_table[*ptr] = 1; ++ptr; }
+
+    isNameStartCharVRML1Initialized = TRUE;
+  }
 
   // VRML1, VRML2 and Inventor files are kept separate for ease of
   // modification if more differences are found
-  if (owner->isFileVRML1()) {
-    if (c <= 0x20) return FALSE; // Control characters
-    if (c >= 0x30 && c <= 0x39) return FALSE; // Digits
+  if (c <= 0x20) return FALSE; // Control characters
+  if (c >= 0x30 && c <= 0x39) return FALSE; // Digits
 
-    if (validIdent) {
-      return (strchr(valid_ident_invalid_vrml1, (int)c) == NULL ? TRUE : FALSE);
-    }
-    else {
-      return (strchr(invalid_vrml1, (int)c) == NULL ? TRUE : FALSE);
+  if (validIdent) return (valid_ident_invalid_vrml1_table[c] == 0);
+  return (invalid_vrml1_table[c] == 0);
+}
+
+SbBool
+SoInputP::isNameStartCharVRML2(unsigned char c, SbBool validIdent)
+{
+  static unsigned char invalid_vrml2_table[256];
+  static unsigned char valid_ident_invalid_vrml2_table[256];
+
+  static int isNameStartCharVRML2Initialized = FALSE;
+  if (!isNameStartCharVRML2Initialized) {
+    const unsigned char invalid_vrml2[] = {
+      0x22, 0x23, 0x27, 0x2b, 0x2c, 0x2d, 0x2e, 0x5b, 0x5c, 0x5d, 0x7b, 0x7d, 0x7f, 0x00 }; // 0x7f = 127
+    //'"',  '#',  ''',  '+',  ',',  '-',  '.',  '[',  '\',  ']',  '{',  '}',  ''
+
+    const unsigned char * valid_ident_invalid_vrml2 = invalid_vrml2;
+
+    for (int c = 0; c < 256; ++c) {
+      invalid_vrml2_table[c] = 0;
+      valid_ident_invalid_vrml2_table[c] = 0;
     }
+
+    const unsigned char * ptr = invalid_vrml2;
+    while (*ptr) { invalid_vrml2_table[*ptr] = 1; ++ptr; }
+    ptr = valid_ident_invalid_vrml2;
+    while (*ptr) { valid_ident_invalid_vrml2_table[*ptr] = 1; ++ptr; }
+
+    isNameStartCharVRML2Initialized = TRUE;
   }
-  else if (owner->isFileVRML2()) {
-    if (c <= 0x20) return FALSE; // Control characters
-    if (c >= 0x30 && c <= 0x39) return FALSE; // Digits
 
-    if (validIdent) {
-      return (strchr(valid_ident_invalid_vrml2, (int)c) == NULL ? TRUE : FALSE);
-    }
-    else {
-      // For Coin to be able to load VRML97 (invalid) files that have
-      // been generated with illegal names, '+' is considered a valid
-      // startcharacter.
-      static int non_strict = -1;
-      if (non_strict == -1) {
-        const char * env = coin_getenv("COIN_NOT_STRICT_VRML97");
-        non_strict = env && (atoi(env) > 0);
-      }
+  if (c <= 0x20) return FALSE; // Control characters
+  if (c >= 0x30 && c <= 0x39) return FALSE; // Digits
 
-      if (c == 0x2b && non_strict) { // '+' is considered valid
-        return TRUE;
-      }
+  if (validIdent) return (valid_ident_invalid_vrml2_table[c] == 0);
 
-      return (strchr(invalid_vrml2, (int)c) == NULL ? TRUE : FALSE);
-    }
-  }
-  else { // Inventor files
-    if (validIdent) {
-      return SbName::isIdentStartChar(c);
-    }
-    else {
-      return (c > 0x20); // Not control characters
-    }
+  // For Coin to be able to load VRML97 (invalid) files that have
+  // been generated with illegal names, '+' is considered a valid
+  // startcharacter.
+  static int non_strict = -1;
+  if (non_strict == -1) {
+    const char * env = coin_getenv("COIN_NOT_STRICT_VRML97");
+    non_strict = env && (atoi(env) > 0);
   }
+  if (c == '+' && non_strict) // '+' is considered valid
+    return TRUE;
+
+  return (invalid_vrml2_table[c] == 0);
 }
 
 // Helperfunction to handle different filetypes (Inventor, VRML 1.0
 // and VRML 2.0).
 // 
 // See SoInputP::isIdentStartChar for more information
+
 SbBool
 SoInputP::isNameChar(unsigned char c, SbBool validIdent)
 {
-  // Cannot be unsigned when using strchr, but since all values are
-  // below 128, it does not matter if they are signed or unsigned.
-  const char invalid_vrml1[] = { 0x22, 0x23, 0x27, 0x2b, 0x2c, 0x2e, 
-                                 0x5c, 0x7b, 0x7d, 0x00 }; // 0x7d = 125
-                               // '"',  '#',  ''',  '+',  ',',  '.',
-                               // '\',  '{',  '}'
+  if (validIdent) return SbName::isIdentChar(c);
+  return (c > 0x20); // Not control characters
+}
 
-  // Compared to isIdentStartChar, '+' and '-' have now become valid
-  // characters.
-  const char invalid_vrml2[] = { 0x22, 0x23, 0x27, 0x2c, 0x2e, 0x5b, 
-                                 0x5c, 0x5d, 0x7b, 0x7d, 0x7f, 0x00 }; // 0x7f = 127
-                               // '"',  '#',  ''',  ',',  '.',  '[',
-                               // '\',  ']',  '{',  '}',   ''
+SbBool
+SoInputP::isNameCharVRML1(unsigned char c, SbBool validIdent)
+{
+  static unsigned char invalid_vrml1_table[256];
+  static unsigned char valid_ident_invalid_vrml1_table[256];
 
-  // Differences from invalid_vrml1: '&' , '[', and ']' are now invalid
-  const char valid_ident_invalid_vrml1[] = { 0x22, 0x23, 0x26, 0x27, 0x2b, 0x2c, 0x2e, 
-                                             0x5b, 0x5c, 0x5d, 0x7b, 0x7d, 0x00 }; // 0x7d = 125
-                                           // '"',  '#',   '&', ''',  '+',  ',',  '.',
-                                           // '[',  '\',   ']',  '{',  '}'
+  static int isNameCharVRML1Initialized = FALSE;
+  if (!isNameCharVRML1Initialized) {
+    const unsigned char invalid_vrml1[] = {
+      0x22, 0x23, 0x27, 0x2b, 0x2c, 0x2e, 0x5c, 0x7b, 0x7d, 0x00 }; // 0x7d = 125
+    //'"',  '#',  ''',  '+',  ',',  '.',  '\',  '{',  '}'
 
-  const char * valid_ident_invalid_vrml2 = invalid_vrml2;
+    // Differences from invalid_vrml1: '&' , '[', and ']' are now invalid
+    const unsigned char valid_ident_invalid_vrml1[] = {
+      0x22, 0x23, 0x26, 0x27, 0x2b, 0x2c, 0x2e, 0x5b, 0x5c, 0x5d, 0x7b, 0x7d, 0x00 }; // 0x7d = 125
+    //'"',  '#',   '&', ''',  '+',  ',',  '.',  '[',  '\',   ']',  '{',  '}'
+
+    for (int c = 0; c < 256; ++c) {
+      invalid_vrml1_table[c] = 0;
+      valid_ident_invalid_vrml1_table[c] = 0;
+    }
+
+    const unsigned char * ptr = invalid_vrml1;
+    while (*ptr) { invalid_vrml1_table[*ptr] = 1; ++ptr; }
+    ptr = valid_ident_invalid_vrml1;
+    while (*ptr) { valid_ident_invalid_vrml1_table[*ptr] = 1; ++ptr; }
+
+    isNameCharVRML1Initialized = TRUE;
+  }
 
   // VRML1, VRML2 and Inventor files are kept separate for ease of
   // modification if more differences are found
-  if (owner->isFileVRML1()) {
-    if (c <= 0x20) return FALSE; // Control characters
+  if (c <= 0x20) return FALSE; // Control characters
 
-    if (validIdent) {
-      return (strchr(valid_ident_invalid_vrml1, (int)c) == NULL ? TRUE : FALSE);
-    }
-    else {
-      return (strchr(invalid_vrml1, (int)c) == NULL ? TRUE : FALSE);
-    }
-  }
-  else if (owner->isFileVRML2()) {
-    if (c <= 0x20) return FALSE; // Control characters
+  if (validIdent) return (valid_ident_invalid_vrml1_table[c] == 0);
+  return (invalid_vrml1_table[c] == 0);
+}
 
-    if (validIdent) {
-      return (strchr(valid_ident_invalid_vrml2, (int)c) == NULL ? TRUE : FALSE);
-    }
-    else {
-      return (strchr(invalid_vrml2, (int)c) == NULL ? TRUE : FALSE);
-    }
-  }
-  else { // Inventor files
-    if (validIdent) {
-      return SbName::isIdentChar(c);
-    }
-    else {
-      return (c > 0x20); // Not control characters
+SbBool
+SoInputP::isNameCharVRML2(unsigned char c, SbBool validIdent)
+{
+  static unsigned char invalid_vrml2_table[256];
+  static unsigned char valid_ident_invalid_vrml2_table[256];
+
+  static int isNameCharVRML2Initialized = FALSE;
+  if (!isNameCharVRML2Initialized) {
+    // Compared to isIdentStartChar, '+' and '-' have now become valid characters.
+    const unsigned char invalid_vrml2[] = {
+      0x22, 0x23, 0x27, 0x2c, 0x2e, 0x5b, 0x5c, 0x5d, 0x7b, 0x7d, 0x7f, 0x00 }; // 0x7f = 127
+    //'"',  '#',  ''',  ',',  '.',  '[',  '\',  ']',  '{',  '}',  ''
+
+    const unsigned char * valid_ident_invalid_vrml2 = invalid_vrml2;
+
+    for (int c = 0; c < 256; ++c) {
+      invalid_vrml2_table[c] = 0;
+      valid_ident_invalid_vrml2_table[c] = 0;
     }
+
+    const unsigned char * ptr = invalid_vrml2;
+    while (*ptr) { invalid_vrml2_table[*ptr] = 1; ++ptr; }
+    ptr = valid_ident_invalid_vrml2;
+    while (*ptr) { valid_ident_invalid_vrml2_table[*ptr] = 1; ++ptr; }
+
+    isNameCharVRML2Initialized = TRUE;
   }
+
+  // VRML1, VRML2 and Inventor files are kept separate for ease of
+  // modification if more differences are found
+  if (c <= 0x20) return FALSE; // Control characters
+
+  if (validIdent) return (valid_ident_invalid_vrml2_table[c] == 0);
+  return (invalid_vrml2_table[c] == 0);
 }
 
 // *************************************************************************
@@ -1074,7 +1119,7 @@ SoInput::get(char & c)
 {
   // It is essential that this method pops on EOF, because this
   // feature is used in e.g SoFile::readNamedFile
-  SoInput_FileInfo * fi = PRIVATE(this)->getTopOfStackPopOnEOF();
+  SoInput_FileInfo * fi = SoInputP::getTopOfStackPopOnEOF(this);
   return (this->checkHeader() && // Strip off file header, if any.
           fi->get(c));
 }
@@ -1139,7 +1184,7 @@ SoInput::readHex(uint32_t & l)
 SbBool
 SoInput::read(char & c)
 {
-  SoInput_FileInfo * fi = PRIVATE(this)->getTopOfStackPopOnEOF();
+  SoInput_FileInfo * fi = SoInputP::getTopOfStackPopOnEOF(this);
 
   if (!this->checkHeader()) return FALSE;
 
@@ -1154,7 +1199,7 @@ SoInput::read(char & c)
 SbBool
 SoInput::read(char & c, SbBool skip)
 {
-  SoInput_FileInfo * fi = PRIVATE(this)->getTopOfStackPopOnEOF();
+  SoInput_FileInfo * fi = SoInputP::getTopOfStackPopOnEOF(this);
 
   if (!this->checkHeader()) return FALSE;
 
@@ -1189,7 +1234,7 @@ SoInput::read(char & c, SbBool skip)
 SbBool
 SoInput::read(SbString & s)
 {
-  SoInput_FileInfo * fi = PRIVATE(this)->getTopOfStackPopOnEOF();
+  SoInput_FileInfo * fi = SoInputP::getTopOfStackPopOnEOF(this);
 
   if (!this->checkHeader()) return FALSE;
 
@@ -1347,7 +1392,10 @@ SoInput::read(SbString & s)
 SbBool
 SoInput::read(SbName & n, SbBool validIdent)
 {
-  SoInput_FileInfo * fi = PRIVATE(this)->getTopOfStackPopOnEOF();  
+  SoInput_FileInfo * fi = SoInputP::getTopOfStackPopOnEOF(this);
+
+  const enum CodePath { INVENTOR, VRML1, VRML2 } codepath =
+    fi->isFileVRML2() ? VRML2 : (fi->isFileVRML1() ? VRML1 : INVENTOR);
 
   if (!this->checkHeader()) return FALSE;
 
@@ -1358,10 +1406,34 @@ SoInput::read(SbName & n, SbBool validIdent)
 
     n = s;
 
-    if (validIdent && s.getLength() > 0) {
-      if (!PRIVATE(this)->isNameStartChar(s[0], validIdent)) return FALSE;
-      for (int i = 1; i < s.getLength(); i++)
-        if (!PRIVATE(this)->isNameChar(s[i], validIdent)) return FALSE;
+    const int strlength = s.getLength();
+    if (validIdent && strlength > 0) {
+      switch (codepath) {
+      case INVENTOR:
+      {
+        if (!SoInputP::isNameStartChar(s[0], validIdent)) return FALSE;
+        for (int i = 1; i < strlength; ++i)
+          if (!SoInputP::isNameChar(s[i], validIdent)) return FALSE;
+        break;
+      }
+      case VRML1:
+      {
+        if (!SoInputP::isNameStartCharVRML1(s[0], validIdent)) return FALSE;
+        for (int i = 1; i < strlength; ++i)
+          if (!SoInputP::isNameCharVRML1(s[i], validIdent)) return FALSE;
+        break;
+      }
+      case VRML2:
+      {
+        if (!SoInputP::isNameStartCharVRML2(s[0], validIdent)) return FALSE;
+        for (int i = 1; i < strlength; ++i)
+          if (!SoInputP::isNameCharVRML2(s[i], validIdent)) return FALSE;
+        break;
+      }
+      default:
+        assert(!"invalid code path");
+        break;
+      }
     }
 
     return TRUE;
@@ -1376,18 +1448,57 @@ SoInput::read(SbName & n, SbBool validIdent)
     char c;
     SbBool gotchar;
     
-    if ((gotchar = fi->get(c)) && PRIVATE(this)->isNameStartChar(c, validIdent)) {
-      *b++ = c;
-      
-      while ((gotchar = fi->get(c)) && PRIVATE(this)->isNameChar(c, validIdent)) {
+    switch (codepath) {
+    case INVENTOR:
+    {
+      if ((gotchar = fi->get(c)) && SoInputP::isNameStartChar(c, validIdent)) {
         *b++ = c;
-        if (b - buf == 255) {
-          *b = '\0';
-          s += buf;
-          b = buf;
+        while ((gotchar = fi->get(c)) && SoInputP::isNameChar(c, validIdent)) {
+          *b++ = c;
+          if (b - buf == 255) {
+            *b = '\0';
+            s += buf;
+            b = buf;
+          }
         }
       }
+      break;
     }
+    case VRML1:
+    {
+      if ((gotchar = fi->get(c)) && SoInputP::isNameStartCharVRML1(c, validIdent)) {
+        *b++ = c;
+        while ((gotchar = fi->get(c)) && SoInputP::isNameCharVRML1(c, validIdent)) {
+          *b++ = c;
+          if (b - buf == 255) {
+            *b = '\0';
+            s += buf;
+            b = buf;
+          }
+        }
+      }
+      break;
+    }
+    case VRML2:
+    {
+      if ((gotchar = fi->get(c)) && SoInputP::isNameStartCharVRML2(c, validIdent)) {
+        *b++ = c;
+        while ((gotchar = fi->get(c)) && SoInputP::isNameCharVRML2(c, validIdent)) {
+          *b++ = c;
+          if (b - buf == 255) {
+            *b = '\0';
+            s += buf;
+            b = buf;
+          }
+        }
+      }
+      break;
+    }
+    default:
+      assert(!"invalid code path");
+      break;
+    }
+
     // This behavior is pretty silly, but this is how it is supposed
     // to work, apparently -- _not_ returning FALSE upon end-of-file.
     if (gotchar) fi->putBack(c);
@@ -2247,11 +2358,12 @@ SoInput::checkHeader(SbBool bValidateBufferHeader)
   // <stdin> is being read from or not, we have a pretty good way of
   // telling if the user want to use <stdin> to read from or not when
   // a new file is pushed on the stack.
+  SoInputP * pimpl;
   if (this->filestack.getLength() == 1 && 
       fi->ivFilePointer() == coin_get_stdin() && 
-      !PRIVATE(this)->usingstdin) {
+      !(pimpl = PRIVATE(this))->usingstdin) {
 
-    PRIVATE(this)->usingstdin = TRUE;
+    pimpl->usingstdin = TRUE;
   }
 
   return fi->readHeader(this) && (!bValidateBufferHeader || fi->ivVersion() != 0.0f);
@@ -2333,26 +2445,9 @@ SoInput::freeBytesInBuf(void) const
 SbBool
 SoInput::readInteger(int32_t & l)
 {
-  // FIXME: write code for reading from binary stream? 19990628 mortene.
-  assert(!this->isBinary());
-
-  // FIXME: fixed size buffer for input of unknown
-  // length. Ouch. 19990530 mortene.
-  char str[32];
-  char * s = str;
-
-  if (this->readChar(s, '-') || this->readChar(s, '+'))
-    s++;
-
-  if (! this->readUnsignedIntegerString(s))
-    return FALSE;
-
-  // FIXME: check man page of strtol and exploit the functionality
-  // provided better -- it looks like we are duplicating some of the
-  // effort. 19990530 mortene.
-  l = strtol(str, NULL, 0);
-
-  return TRUE;
+  SoInput_FileInfo * fi = SoInputP::getTopOfStackPopOnEOF(this);
+  if (!fi) return FALSE;
+  return fi->readInteger(l);
 }
 
 /*!
@@ -2362,21 +2457,9 @@ SoInput::readInteger(int32_t & l)
 SbBool
 SoInput::readUnsignedInteger(uint32_t & l)
 {
-  // FIXME: write code for reading from binary stream? 19990628 mortene.
-  assert(!this->isBinary());
-
-  // FIXME: fixed size buffer for input of unknown
-  // length. Ouch. 19990530 mortene.
-  char str[32];
-  if (! this->readUnsignedIntegerString(str))
-    return FALSE;
-
-  // FIXME: check man page of strtoul and exploit the functionality
-  // provided better -- it looks like we are duplicating some of the
-  // effort. 19990530 mortene.
-  l = strtoul(str, NULL, 0);
-
-  return TRUE;
+  SoInput_FileInfo * fi = SoInputP::getTopOfStackPopOnEOF(this);
+  if (!fi) return FALSE;
+  return fi->readUnsignedInteger(l);
 }
 
 /*!
@@ -2387,66 +2470,9 @@ SoInput::readUnsignedInteger(uint32_t & l)
 SbBool
 SoInput::readReal(double & d)
 {
-  // FIXME: write code for reading from binary stream? 19990628 mortene.
-  assert(!this->isBinary());
-
-  // FIXME: fixed size buffer for input of unknown
-  // length. Ouch. 19990530 mortene.
-  const int BUFSIZE = 2048;
-  char str[BUFSIZE];
-  int n;
-  char * s = str;
-  SbBool gotNum = FALSE;
-
-  n = this->readChar(s, '-');
-  if (n == 0)
-    n = this->readChar(s, '+');
-  s += n;
-
-  if ((n = this->readDigits(s)) > 0) {
-    gotNum = TRUE;
-    s += n;
-  }
-
-  if (this->readChar(s, '.') > 0) {
-    s++;
-
-    if ((n = this->readDigits(s)) > 0) {
-      gotNum = TRUE;
-      s += n;
-    }
-  }
-
-  if (! gotNum)
-    return FALSE;
-
-  n = this->readChar(s, 'e');
-  if (n == 0)
-    n = this->readChar(s, 'E');
-
-  if (n > 0) {
-    s += n;
-
-    n = this->readChar(s, '-');
-    if (n == 0)
-      n = this->readChar(s, '+');
-    s += n;
-
-    if ((n = this->readDigits(s)) > 0)
-      s += n;
-
-    else
-      return FALSE; }
-
-  *s = '\0';
-
-  // FIXME: using the wrapped atof() call like this might have serious
-  // consequences for performance upon import of large
-  // iv/wrl-files. This should be checked. 20030407 mortene.
-
-  d = coin_atof(str);
-
-  return TRUE;
+  SoInput_FileInfo * fi = SoInputP::getTopOfStackPopOnEOF(this);
+  if (!fi) return FALSE;
+  return fi->readReal(d);
 }
 
 /*!
@@ -2459,29 +2485,9 @@ SoInput::readReal(double & d)
 SbBool
 SoInput::readUnsignedIntegerString(char * str)
 {
-  // FIXME: write code for reading from binary stream? 19990628 mortene.
-  assert(!this->isBinary());
-
-  int minSize = 1;
-  char * s = str;
-
-  if (this->readChar(s, '0')) {
-    if (this->readChar(s + 1, 'x')) {
-      s += 2 + this->readHexDigits(s + 2);
-      minSize = 3;
-    }
-    else
-      s += 1 + this->readDigits(s + 1);
-  }
-  else
-    s += this->readDigits(s);
-
-  if (s - str < minSize)
-    return FALSE;
-
-  *s = '\0';
-
-  return TRUE;
+  SoInput_FileInfo * fi = SoInputP::getTopOfStackPopOnEOF(this);
+  if (!fi) return FALSE;
+  return fi->readUnsignedIntegerString(str);
 }
 
 /*!
@@ -2491,25 +2497,9 @@ SoInput::readUnsignedIntegerString(char * str)
 int
 SoInput::readDigits(char * str)
 {
-  // FIXME: write code for reading from binary stream? 19990628 mortene.
-  assert(!this->isBinary());
-
-  SoInput_FileInfo * fi = this->getTopOfStack();
-  assert(fi);
-
-  char c, * s = str;
-
-  while (fi->get(c)) {
-    if (isdigit(c))
-      *s++ = c;
-    else {
-      fi->putBack(c);
-      break;
-    }
-  }
-
-  const ptrdiff_t offset = s - str;
-  return (int)offset;
+  SoInput_FileInfo * fi = SoInputP::getTopOfStackPopOnEOF(this);
+  if (!fi) return FALSE;
+  return fi->readDigits(str);
 }
 
 /*!
@@ -2519,24 +2509,9 @@ SoInput::readDigits(char * str)
 int
 SoInput::readHexDigits(char * str)
 {
-  // FIXME: write code for reading from binary stream? 19990628 mortene.
-  assert(!this->isBinary());
-
-  SoInput_FileInfo * fi = this->getTopOfStack();
-  assert(fi);
-
-  char c, * s = str;
-  while (fi->get(c)) {
-
-    if (isxdigit(c)) *s++ = c;
-    else {
-      fi->putBack(c);
-      break;
-    }
-  }
-
-  const ptrdiff_t offset = s - str;
-  return (int)offset;
+  SoInput_FileInfo * fi = SoInputP::getTopOfStackPopOnEOF(this);
+  if (!fi) return FALSE;
+  return fi->readHexDigits(str);
 }
 
 /*!
@@ -2548,22 +2523,9 @@ SoInput::readHexDigits(char * str)
 int
 SoInput::readChar(char * s, char charToRead)
 {
-  SoInput_FileInfo * fi = this->getTopOfStack();
-  assert(fi);
-
-  int ret = 0;
-  char c;
-  if (fi->get(c)) {
-    if (c == charToRead) {
-      *s = c;
-      ret = 1;
-    }
-    else {
-      fi->putBack(c);
-    }
-  }
-
-  return ret;
+  SoInput_FileInfo * fi = SoInputP::getTopOfStackPopOnEOF(this);
+  if (!fi) return FALSE;
+  return fi->readChar(s, charToRead);
 }
 
 /*!
@@ -2791,6 +2753,7 @@ SoInput::setDirectories(SbStringList * dirs)
 
   Return the currently active stream.
 */
+#if 0
 SoInput_FileInfo *
 SoInput::getTopOfStack(void) const
 {
@@ -2800,6 +2763,7 @@ SoInput::getTopOfStack(void) const
   }
   return this->filestack[0];
 }
+#endif
 
 /*!
   Scan through the include directories in order and try to find a file
