@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -352,8 +352,12 @@ SoDirectionalLightManip::valueChangedCB(void * m, SoDragger * dragger)
   SbMatrix matrix = dragger->getMotionMatrix();
   SbVec3f direction(0.0f, 0.0f, -1.0f);
   matrix.multDirMatrix(direction, direction);
-  direction.normalize();
-
+  if (direction.normalize() == 0.0f) {
+#if COIN_DEBUG
+    SoDebugError::post("SoDirectionalLightManip::valueChangedCB",
+                       "Invalid motion matrix.");
+#endif // debug
+  }
   thisp->attachSensors(FALSE);
   if (thisp->direction.getValue() != direction) {
     thisp->direction = direction;
