@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -177,6 +177,7 @@ public:
 
   static void emptyName2ObjHash(const char * const & n, SbPList * const & l, void * closure);
 
+  static void check_for_leaks(void);
   static SbBool trackbaseobjects;
   static void * allbaseobj_mutex;
   static SoBaseSet * allbaseobj; // maps from SoBase * to NULL
@@ -226,6 +227,36 @@ SoBaseP::cleanup_auditordict(void)
   }
 }
 
+void
+SoBaseP::check_for_leaks(void)
+{
+#if COIN_DEBUG
+  if (SoBaseP::trackbaseobjects) {
+    SbList<const SoBase *> keys;
+    SoBaseP::allbaseobj->makeKeyList(keys);
+    const unsigned int len = keys.getLength();
+    if (len > 0) {
+      // Use printf()s, in case SoDebugError has been made defunct by
+      // previous coin_atexit() work.
+      (void)printf("\nSoBase-derived instances not deallocated:\n");
+
+      for (unsigned int i=0; i < len; i++) {
+        const SoBase * base = keys[i];
+        base->assertAlive();
+        const SbName name = base->getName();
+        const SoType t = base->getTypeId();
+        SbString s;
+        s.sprintf("\"%s\"", name.getString());
+        (void)printf("\t%p type==(0x%04x, '%s') name=='%s'\n",
+                     base, t.getKey(), t.getName().getString(),
+                     name == "" ? "no name" : s.getString());
+      }
+      (void)printf("\n");
+    }
+  }
+#endif // COIN_DEBUG
+}
+
 // *************************************************************************
 
 // Strings and character tokens used in parsing.
@@ -425,6 +456,18 @@ SoBase::initClass(void)
 {
   coin_atexit((coin_atexit_f *)SoBase::cleanClass, CC_ATEXIT_NORMAL);
 
+  // check_for_leaks() goes through the allocation list, and checks if
+  // all allocated SoBase-derived instances was deallocated before the
+  // atexit-routines were run.
+  //
+  // Set up to run before other atexit-code, with NORMAL+1 priority,
+  // since we depend on misc parts of Coin still up and running (and
+  // no other code should depend on the allocation list, so this
+  // should be safe).
+  //
+  // -mortene.
+  coin_atexit((coin_atexit_f *)SoBaseP::check_for_leaks, CC_ATEXIT_NORMAL + 1);
+
   // Avoid multiple attempts at initialization.
   assert(SoBase::classTypeId == SoType::badType());
 
@@ -454,31 +497,6 @@ SoBase::initClass(void)
 void
 SoBase::cleanClass(void)
 {
-#if COIN_DEBUG
-  if (SoBaseP::trackbaseobjects) {
-    SbList<const SoBase *> keys;
-    SoBaseP::allbaseobj->makeKeyList(keys);
-    const unsigned int len = keys.getLength();
-    if (len > 0) {
-      // Use printf()s, in case SoDebugError has been made defunct by
-      // previous coin_atexit() work.
-      (void)printf("\nSoBase-derived instances not deallocated:\n");
-
-      for (unsigned int i=0; i < len; i++) {
-        const SoBase * base = keys[i];
-        const SbName name = base->getName();
-        SbString s;
-        s.sprintf("\"%s\"", name.getString());
-        (void)printf("\t%p %s (%s)\n",
-                     base,
-                     base->getTypeId().getName().getString(),
-                     name == "" ? "no name" : s.getString());
-      }
-      (void)printf("\n");
-    }
-  }
-#endif // COIN_DEBUG
-
   assert(SoBaseP::name2obj);
   assert(SoBaseP::obj2name);
 
@@ -560,11 +578,10 @@ SoBase::ref(void) const
   // Cast away constness.
   SoBase * base = (SoBase *)this;
 
+  CC_MUTEX_LOCK(SoBaseP::mutex);
 #if COIN_DEBUG
   int32_t currentrefcount = base->objdata.referencecount;
 #endif // COIN_DEBUG
-
-  CC_MUTEX_LOCK(SoBaseP::mutex);
   base->objdata.referencecount++;
   CC_MUTEX_UNLOCK(SoBaseP::mutex);
 
@@ -615,6 +632,8 @@ SoBase::unref(void) const
   SoBase * base = (SoBase *)this;
   CC_MUTEX_LOCK(SoBaseP::mutex);
   base->objdata.referencecount--;
+  int refcount = base->objdata.referencecount;
+ 
   CC_MUTEX_UNLOCK(SoBaseP::mutex);
 
 #if COIN_DEBUG
@@ -624,7 +643,7 @@ SoBase::unref(void) const
                            this, this->getTypeId().getName().getString(),
                            this->objdata.referencecount);
   }
-  if (this->objdata.referencecount < 0) {
+  if (refcount < 0) {
     // Do the debug output in two calls, since the getTypeId() might
     // cause a crash, and then there'd be no output at all with a
     // single SoDebugError::postWarning() call.
@@ -633,7 +652,7 @@ SoBase::unref(void) const
                               this->getTypeId().getName().getString(), this);
   }
 #endif // COIN_DEBUG
-  if (this->objdata.referencecount == 0) base->destroy();
+  if (refcount == 0) base->destroy();
 }
 
 /*!
