@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -335,6 +335,8 @@
 #include <Inventor/details/SoDetail.h>
 #include <Inventor/elements/SoElement.h>
 #include <Inventor/engines/SoEngine.h>
+#include <Inventor/engines/SoNodeEngine.h>
+#include <Inventor/engines/SoEngineOutput.h>
 #include <Inventor/errors/SoReadError.h>
 #include <Inventor/events/SoEvent.h>
 #include <Inventor/fields/SoGlobalField.h>
@@ -348,6 +350,7 @@
 #include <Inventor/sensors/SoTimerSensor.h>
 #include <Inventor/annex/HardCopy/SoHardCopy.h>
 #include <Inventor/errors/SoDebugError.h>
+#include "SoVBO.h"
 
 #include <coindefs.h> // COIN_STUB()
 
@@ -372,7 +375,7 @@ static SbRWMutex * sodb_globalmutex = NULL;
 #endif // HAVE_3DS_IMPORT_CAPABILITIES
 
 #include "CoinStaticObjectInDLL.h"
-
+#include "PointerMap.h"
 #include "systemsanity.icc"
 
 static SbString * coin_versionstring = NULL;
@@ -381,6 +384,7 @@ static SbString * coin_versionstring = NULL;
 static void cleanup_func(void)
 {
   delete coin_versionstring;
+  coin_versionstring = NULL;
 }
 
 // *************************************************************************
@@ -630,6 +634,9 @@ SoDB::init(void)
   }
 #endif // OBSOLETED sanity checks
 
+  // initialize the pimpl pointer mapper
+  PointerMap::initClass();
+
   SoInput::init();
   SoBase::initClass();
   SoDetail::initClass();
@@ -654,6 +661,7 @@ SoDB::init(void)
   SoGLBigImage::initClass();
 
   SoHardCopy::init();
+  SoVBO::init();
 
 #ifdef HAVE_VRML97
   so_vrml_init();
@@ -764,9 +772,7 @@ SoDB::init(void)
   // resource usage. This needs to be done last in init(), so we get
   // called _before_ clean() methods in other classes with the same
   // priority.
-  //
-  // FIXME: should use "cross-Coin" enum for priority. 20050627 mortene.
-  coin_atexit((coin_atexit_f *)SoDBP::clean, 0);
+  coin_atexit((coin_atexit_f *)SoDBP::clean, CC_ATEXIT_NORMAL);
 }
 
 // This will free all resources which has been allocated by the SoDB
@@ -795,7 +801,7 @@ SoDBP::clean(void)
 
   // Must be done /before/ killing off the SoSensorManager instance,
   // in case there is an SoFieldSensor attached to the realTime global
-  // field.
+  // field. Will delete the field.
   SoDB::renameGlobalField("realTime", "");
 
   delete SoDBP::sensormanager;
@@ -853,7 +859,7 @@ SoDB::getVersion(void)
 {
   if (coin_versionstring == NULL) {
     coin_versionstring = new SbString("SIM Coin " COIN_VERSION);
-    coin_atexit((coin_atexit_f *)cleanup_func, 0);
+    coin_atexit((coin_atexit_f *)cleanup_func, CC_ATEXIT_NORMAL);
   }
   return coin_versionstring->getString();
 }
@@ -1863,6 +1869,37 @@ SoDB::writeunlock(void)
 
 // *************************************************************************
 
+//
+// helper function for createRoute(). First test the actual fieldname,
+// then set set_<fieldname>, then <fieldname>_changed.
+//
+static SoField *
+find_route_field(SoNode * node, const SbName & fieldname)
+{
+  SoField * field = node->getField(fieldname);
+
+  if (!field) {
+    if (strncmp(fieldname.getString(), "set_", 4) == 0) {
+      SbName newname = fieldname.getString() + 4;
+      field = node->getField(newname);
+    }
+    else {
+      SbString str = fieldname.getString();
+      int len = str.getLength();
+      const char CHANGED[] = "_changed";
+      const int changedsize = sizeof(CHANGED) - 1;
+
+      if (len > changedsize && strcmp(str.getString()+len-changedsize,
+                                      CHANGED) == 0) {
+        SbString substr = str.getSubString(0, len-(changedsize+1));
+        SbName newname = substr.getString();
+        field = node->getField(newname);
+      }
+    }
+  }
+  return field;
+}
+
 /*!
   Create a connection from one VRML97 node field to another.
 
@@ -1878,27 +1915,93 @@ SoDB::writeunlock(void)
   \since TGS Inventor 2.6
 */
 void
-SoDB::createRoute(SoNode * from, const char * eventout,
-                  SoNode * to, const char * eventin)
+SoDB::createRoute(SoNode * fromnode, const char * eventout,
+                  SoNode * tonode, const char * eventin)
 {
-  assert(from && to && eventout && eventin);
-  SoField * out = from->getEventOut(eventout);
-  SoField * in = to->getEventIn(eventin);
+  assert(fromnode && tonode && eventout && eventin);
 
-  if (out == NULL) {
-    SoDebugError::postWarning("SoDB::createRoute",
-                              "no eventout field named '%s' for '%s'",
-                              eventout, from->getTypeId().getName().getString());
-    return;
+  SbName fromfieldname(eventout);
+  SbName tofieldname(eventin);
+
+  SoField * from = find_route_field(fromnode, fromfieldname);
+  SoField * to = find_route_field(tonode, tofieldname);
+
+  SbName fromnodename = fromnode->getName();
+  if (fromnodename == "") {
+    fromnodename = "<noname>";
   }
-  if (in == NULL) {
-    SoDebugError::postWarning("SoDB::createRoute",
-                              "no eventin field named '%s' for '%s'",
-                              eventin, to->getTypeId().getName().getString());
-    return;
+  SbName tonodename = tonode->getName();
+  if (tonodename == "") {
+    tonodename = "<noname>";
+  }
+  SoEngineOutput * output = NULL;
+  if (from == NULL && fromnode->isOfType(SoNodeEngine::getClassTypeId())) {
+    output = ((SoNodeEngine*) fromnode)->getOutput(fromfieldname);
   }
 
-  in->connectFrom(out);
+  if (to && (from || output)) {
+    SbBool notnotify = FALSE;
+    SbBool append = FALSE;
+    if (output || from->getFieldType() == SoField::EVENTOUT_FIELD) {
+      notnotify = TRUE;
+    }
+#if 0 // seems like (reading the VRML97 spec.) fanIn in allowed even to regular fields
+    if (to->getFieldType() == SoField::EVENTIN_FIELD) append = TRUE;
+#else // fanIn
+    append = TRUE;
+#endif // fanIn fix
+    
+    // Check if we're already connected.
+    SoFieldList fl;
+    if (from) from->getForwardConnections(fl);
+    else output->getForwardConnections(fl);
+    int idx = fl.find(to);
+    if (idx != -1) {
+#if COIN_DEBUG
+      SoDebugError::postWarning("SoDB::createRoute",
+                                "Tried to connect a ROUTE multiple times "
+                                "(from %s.%s to %s.%s)",
+                                fromnodename.getString(), fromfieldname.getString(),
+                                tonodename.getString(), tofieldname.getString());
+#endif // COIN_DEBUG
+      return;
+    }
+
+    // Check that there exists a field converter, if one is needed.
+    SoType totype = to->getTypeId();
+    SoType fromtype = from ? from->getTypeId() : output->getConnectionType();
+    if (totype != fromtype) {
+      SoType convtype = SoDB::getConverter(fromtype, totype);
+      if (convtype == SoType::badType()) {
+#if COIN_DEBUG
+        SoDebugError::postWarning("SoDB::createRoute",
+                                  "Tried to connect a ROUTE between entities "
+                                  "that can not be connected (due to lack of "
+                                  "field type converter): %s.%s is of type "
+                                  "%s, and %s.%s is of type %s",
+                                  fromnodename.getString(), fromfieldname.getString(),
+                                  fromtype.getName().getString(),
+                                  tonodename.getString(), tofieldname.getString(),
+                                  totype.getName().getString());
+#endif // COIN_DEBUG
+        return;
+      }
+    }
+    
+    SbBool ok;
+    if (from) ok = to->connectFrom(from, notnotify, append);
+    else ok = to->connectFrom(output, notnotify, append);
+    // Both known possible failure points are caught above.
+    assert(ok && "unexpected connection error");
+  }
+#if COIN_DEBUG
+  else {
+    SoDebugError::postWarning("SoDB::createRoute",
+                              "Unable to create route: %s.%s TO %s.%s",
+                              fromnodename.getString(), eventout,
+                              tonodename.getString(), eventin);
+  }
+#endif  // COIN_DEBUG
 }
 
 /*!
@@ -1911,14 +2014,43 @@ SoDB::createRoute(SoNode * from, const char * eventout,
   \since TGS Inventor 2.6
 */
 void
-SoDB::removeRoute(SoNode * from, const char * eventout,
-                  SoNode * to, const char * eventin)
+SoDB::removeRoute(SoNode * fromnode, const char * eventout,
+                  SoNode * tonode, const char * eventin)
 {
-  assert(from && to && eventout && eventin);
-  SoField * out = from->getEventOut(eventout);
-  SoField * in = to->getEventIn(eventin);
-  assert(out && in);
-  in->disconnect(out);
+  assert(fromnode && tonode && eventout && eventin);
+
+  SbName fromfieldname(eventout);
+  SbName tofieldname(eventin);
+  
+  SoField * from = find_route_field(fromnode, fromfieldname);
+  SoField * to = find_route_field(tonode, tofieldname);
+  
+  SoEngineOutput * output = NULL;
+  if (from == NULL && fromnode->isOfType(SoNodeEngine::getClassTypeId())) {
+    output = ((SoNodeEngine*) fromnode)->getOutput(fromfieldname);
+  }
+
+  SbName fromnodename = fromnode->getName();
+  if (fromnodename == "") {
+    fromnodename = "<noname>";
+  }
+  SbName tonodename = tonode->getName();
+  if (tonodename == "") {
+    tonodename = "<noname>";
+  }
+
+  if (to && (from || output)) {
+    if (from) to->disconnect(from);
+    else to->disconnect(output);
+  }
+#if COIN_DEBUG
+  else { // some error occured
+    SoDebugError::postWarning("SoDB::removeRoute",
+                              "Unable to remove route: %s.%s TO %s.%s",
+                              fromnodename.getString(), eventout,
+                              tonodename.getString(), eventin);
+  }
+#endif // COIN_DEBUG
 }
 
 // *************************************************************************
