@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -87,6 +87,12 @@
   unreproducable cases.  It could be a bug in our glTexSubImage()
   code, of course. :)
 
+  \li COIN_TEX2_USE_SGIS_GENERATE_MIPMAP: When set, use the
+  GL_SGIS_generate_mip extension (if available) to generate mipmaps,
+  otherwise use a fast internal routine to generate them. Use of
+  SGIS_generate_mipmap is not enabled by default since we suspect some
+  ATi drivers have problems with this extensions.
+
   \li COIN_ENABLE_CONFORMANT_GL_CLAMP: When set, GL_CLAMP will be used
   when SoGLImage::CLAMP is specified as the texture wrap mode. By
   default GL_CLAMP_TO_EDGE is used, since this is usually what people
@@ -94,10 +100,13 @@
   http://www.opengl.org/discussion_boards/ubb/Forum3/HTML/007306.html
   for a discussion regarding GL_CLAMP and GL_CLAMP_TO_EDGE.  
 
+  \li COIN_TEX2_ANISOTROPIC_LIMIT: Anisotropic filtering is enabled
+  for textures when the texture quality is higher than this value.
+  Default value is 0.85 
+
   \COIN_CLASS_EXTENSION
 
-  \since Coin 2.0
-*/
+  \since Coin 2.0 */
 
 // *************************************************************************
 
@@ -211,16 +220,23 @@ static float DEFAULT_LINEAR_LIMIT = 0.2f;
 static float DEFAULT_MIPMAP_LIMIT = 0.5f;
 static float DEFAULT_LINEAR_MIPMAP_LIMIT = 0.8f;
 static float DEFAULT_SCALEUP_LIMIT = 0.7f;
+static float DEFAULT_ANISOTROPIC_LIMIT = 0.85f;
 
 static float COIN_TEX2_LINEAR_LIMIT = -1.0f;
 static float COIN_TEX2_MIPMAP_LIMIT = -1.0f;
 static float COIN_TEX2_LINEAR_MIPMAP_LIMIT = -1.0f;
 static float COIN_TEX2_SCALEUP_LIMIT = -1.0f;
+static float COIN_TEX2_ANISOTROPIC_LIMIT = -1.0f;
 static int COIN_TEX2_USE_GLTEXSUBIMAGE = -1;
+static int COIN_TEX2_USE_SGIS_GENERATE_MIPMAP = -1;
 static int COIN_ENABLE_CONFORMANT_GL_CLAMP = -1; 
+static int COIN_TEX2_ALLOW_ALPHA_TEST = -1;
 
 // *************************************************************************
 
+// buffer used for creating mipmap images
+static SbStorage * glimage_bufferstorage = NULL;
+
 typedef struct {
   unsigned char * buffer;
   int buffersize;
@@ -247,24 +263,12 @@ glimage_buffer_destruct(void * buffer)
 }
 
 
-static SbStorage * glimage_bufferstorage = NULL;
-
-static void
-glimage_buffer_cleanup(void)
-{
-  delete glimage_bufferstorage;
-  glimage_bufferstorage = NULL;
-}
-
 static unsigned char *
 glimage_get_buffer(const int buffersize, const SbBool mipmap)
 {
   soglimage_buffer * buf = NULL;
-  if (glimage_bufferstorage == NULL) {
-    coin_atexit((coin_atexit_f*) glimage_buffer_cleanup, 0);
-    glimage_bufferstorage = new SbStorage(sizeof(soglimage_buffer),
-                                          glimage_buffer_construct, glimage_buffer_destruct);
-  }
+  assert(glimage_bufferstorage != NULL);
+  
   buf = (soglimage_buffer*)
     glimage_bufferstorage->get();
   if (mipmap) {
@@ -600,6 +604,10 @@ fast_image_resize3d(const unsigned char * src,
 
 class SoGLImageP {
 public:
+#ifdef COIN_THREADSAFE
+  static SbMutex * mutex;
+#endif // COIN_THREADSAFE
+
   static SoType classTypeId;
   static uint32_t current_glimageid;
   static uint32_t getNextGLImageId(void);
@@ -661,16 +669,12 @@ public:
   void unrefOldDL(SoState *state, const uint32_t maxage);
   SoGLImage *owner;
   uint32_t glimageid;
-#ifdef COIN_THREADSAFE
-  SbMutex mutex;
-#endif // COIN_THREADSAFE
   void init(void);
   static void contextCleanup(uint32_t context, void * closure);
-
 #ifdef COIN_NEXT_MINOR
   static SoGLImage::SoGLImageResizeCB * resizecb;
   static void * resizeclosure;
-#endif
+#endif // COIN_NEXT_MINOR
 };
 
 SoType SoGLImageP::classTypeId STATIC_SOTYPE_INIT;
@@ -678,7 +682,10 @@ uint32_t SoGLImageP::current_glimageid = 1;
 #ifdef COIN_NEXT_MINOR
 SoGLImage::SoGLImageResizeCB * SoGLImageP::resizecb = NULL;
 void * SoGLImageP::resizeclosure = NULL;
-#endif
+#endif // COIN_NEXT_MINOR
+#ifdef COIN_THREADSAFE
+SbMutex * SoGLImageP::mutex;
+#endif // COIN_THREADSAFE
 
 #undef PRIVATE
 #define PRIVATE(p) ((p)->pimpl)
@@ -697,12 +704,15 @@ void * SoGLImageP::resizeclosure = NULL;
 // SoGLImage::getGLDisplayList() use a mutex so that several
 // threads can call this method safely.
 
+
+// we now share one mutex among all glimages to avoid allocating too
+// many mutexes.
 #ifdef COIN_THREADSAFE
-#define LOCK_DLISTS(_thisp_)  (_thisp_)->pimpl->mutex.lock()
-#define UNLOCK_DLISTS(_thisp_)  (_thisp_)->pimpl->mutex.unlock()
+#define LOCK_GLIMAGE SoGLImageP::mutex->lock()
+#define UNLOCK_GLIMAGE SoGLImageP::mutex->unlock()
 #else // COIN_THREADSAFE
-#define LOCK_DLISTS(_thisp_)
-#define UNLOCK_DLISTS(_thisp_)
+#define LOCK_GLIMAGE
+#define UNLOCK_GLIMAGE
 #endif // !COIN_THREADSAFE
 
 // *************************************************************************
@@ -755,6 +765,14 @@ SoGLImage::SoGLImage(void)
     }
     else COIN_TEX2_USE_GLTEXSUBIMAGE = 0;
   }
+  if (COIN_TEX2_USE_SGIS_GENERATE_MIPMAP < 0) {
+    const char *env = coin_getenv("COIN_TEX2_USE_SGIS_GENERATE_MIPMAP");
+    if (env && atoi(env) == 1) {
+      COIN_TEX2_USE_SGIS_GENERATE_MIPMAP = 1;
+    }
+    else COIN_TEX2_USE_SGIS_GENERATE_MIPMAP = 0;
+  }
+
   if (COIN_ENABLE_CONFORMANT_GL_CLAMP < 0) {
     const char * env = coin_getenv("COIN_ENABLE_CONFORMANT_GL_CLAMP");
     if (env && atoi(env) == 1) {
@@ -762,6 +780,19 @@ SoGLImage::SoGLImage(void)
     }
     else COIN_ENABLE_CONFORMANT_GL_CLAMP = 0;
   }
+  if (COIN_TEX2_ANISOTROPIC_LIMIT < 0.0f) {
+    const char *env = coin_getenv("COIN_TEX2_ANISOTROPIC_LIMIT");
+    if (env) COIN_TEX2_ANISOTROPIC_LIMIT = (float) atof(env);
+    else COIN_TEX2_ANISOTROPIC_LIMIT = DEFAULT_ANISOTROPIC_LIMIT;
+  }
+
+  // disable alpha test rendering by default, since we've seen lots
+  // of problems with nVidia cards and alpha testing.
+  if (COIN_TEX2_ALLOW_ALPHA_TEST < 0) {
+    const char *env = coin_getenv("COIN_TEX2_ALLOW_ALPHA_TEST");
+    if (env) COIN_TEX2_ALLOW_ALPHA_TEST = atoi(env);
+    else COIN_TEX2_ALLOW_ALPHA_TEST = 0;
+  }
 }
 
 
@@ -774,10 +805,33 @@ SoGLImage::initClass(void)
   assert(SoGLImageP::classTypeId.isBad());
   SoGLImageP::classTypeId = SoType::createType(SoType::badType(),
                                                SbName("GLImage"));
+#ifdef COIN_THREADSAFE
+  SoGLImageP::mutex = new SbMutex;
+#endif // COIN_THREADSAFE
+  glimage_bufferstorage = new SbStorage(sizeof(soglimage_buffer),
+                                        glimage_buffer_construct, glimage_buffer_destruct);
+
+  coin_atexit(SoGLImage::cleanupClass, CC_ATEXIT_NORMAL);
+}
+
+//
+// called by atexit()
+//
+void
+SoGLImage::cleanupClass(void)
+{
+  delete glimage_bufferstorage;
+  glimage_bufferstorage = NULL;
+#ifdef COIN_THREADSAFE
+  delete SoGLImageP::mutex;
+  SoGLImageP::mutex = NULL;
+#endif // COIN_THREADSAFE
+  SoGLImageP::classTypeId STATIC_SOTYPE_INIT;
 #ifdef COIN_NEXT_MINOR
   SoGLImageP::resizecb = NULL;
   SoGLImageP::resizeclosure = NULL;
-#endif
+#endif // COIN_NEXT_MINOR
+  SoGLImageP::current_glimageid = 1;
 }
 
 /*!
@@ -1132,16 +1186,16 @@ SoGLImage::getImage(void) const
 SoGLDisplayList *
 SoGLImage::getGLDisplayList(SoState *state)
 {
-  LOCK_DLISTS(this);
+  LOCK_GLIMAGE;
   SoGLDisplayList *dl = PRIVATE(this)->findDL(state);
-  UNLOCK_DLISTS(this);
+  UNLOCK_GLIMAGE;
 
   if (dl == NULL) {
     dl = PRIVATE(this)->createGLDisplayList(state);
     if (dl) {
-      LOCK_DLISTS(this);
+      LOCK_GLIMAGE;
       PRIVATE(this)->dlists.append(SoGLImageP::dldata(dl));
-      UNLOCK_DLISTS(this);
+      UNLOCK_GLIMAGE;
     }
   }
   if (dl && !dl->isMipMapTextureObject() && PRIVATE(this)->image) {
@@ -1149,7 +1203,7 @@ SoGLImage::getGLDisplayList(SoState *state)
     float oldquality = PRIVATE(this)->quality;
     PRIVATE(this)->quality = quality;
     if (PRIVATE(this)->shouldCreateMipmap()) {
-      LOCK_DLISTS(this);
+      LOCK_GLIMAGE;
       // recreate DL to get a mipmapped image
       int n = PRIVATE(this)->dlists.getLength();
       for (int i = 0; i < n; i++) {
@@ -1160,7 +1214,7 @@ SoGLImage::getGLDisplayList(SoState *state)
           break;
         }
       }
-      UNLOCK_DLISTS(this);
+      UNLOCK_GLIMAGE;
     }
     else PRIVATE(this)->quality = oldquality;
   }
@@ -1198,6 +1252,8 @@ SoGLImage::useAlphaTest(void) const
   if (PRIVATE(this)->needtransparencytest) {
     ((SoGLImage*)this)->pimpl->checkTransparency();
   }
+  
+  if (COIN_TEX2_ALLOW_ALPHA_TEST == 0) return FALSE;
   return PRIVATE(this)->usealphatest;
 }
 
@@ -1228,6 +1284,19 @@ SoGLImage::getWrapR(void) const
   return PRIVATE(this)->wrapr;
 }
 
+#ifdef COIN_NEXT_MINOR
+/*!
+  Returns the texture quality for this texture image.
+
+  \since 2005-02-19
+*/
+float 
+SoGLImage::getQuality(void) const
+{
+  return PRIVATE(this)->quality;
+}
+#endif // COIN_NEXT_MINOR
+
 /*!  
   Returns an unique if for this GL image. This id can be used to
   test for changes in an SoGLImage's internal data.  
@@ -1286,29 +1355,43 @@ SoGLImageP::resizeImage(SoState * state, unsigned char *& imageptr,
   int numcomponents;
   unsigned char *bytes = this->image->getValue(size, numcomponents);
 
-  uint32_t newx = coin_geq_power_of_two(xsize - 2*this->border);
-  uint32_t newy = coin_geq_power_of_two(ysize - 2*this->border);
-  uint32_t newz = zsize ? coin_geq_power_of_two(zsize - 2*this->border) : 0;
+  uint32_t newx = xsize;
+  uint32_t newy = ysize;
+  uint32_t newz = zsize;
 
-  // if >= 256 and low quality, don't scale up unless size is
-  // close to an above power of two. This saves a lot of texture memory
+  uint32_t maxrectsize = 0;
 
-  if (this->flags & SoGLImage::SCALE_DOWN) {
-    // no use scaling down for very small images
-    if (newx > xsize && newx > 16) newx >>= 1;
-    if (newy > ysize && newy > 16) newy >>= 1;
-    if (newz > zsize && newz > 16) newz >>= 1;
-  }
-  else if (this->flags & SoGLImage::USE_QUALITY_VALUE) {
-    if (this->quality < COIN_TEX2_SCALEUP_LIMIT) {
-      if ((newx >= 256) && ((newx - (xsize-2*this->border)) > (newx>>3)))
-        newx >>= 1;
-      if ((newy >= 256) && ((newy - (ysize-2*this->border)) > (newy>>3)))
-        newy >>= 1;
-      if ((newz >= 256) && ((newz - (zsize-2*this->border)) > (newz>>3)))
-        newz >>= 1;
+  if (!(this->flags & SoGLImage::RECTANGLE)) {
+    newx = coin_geq_power_of_two(xsize - 2*this->border);
+    newy = coin_geq_power_of_two(ysize - 2*this->border);
+    newz = zsize ? coin_geq_power_of_two(zsize - 2*this->border) : 0;
+    
+    // if >= 256 and low quality, don't scale up unless size is
+    // close to an above power of two. This saves a lot of texture memory
+    
+    if (this->flags & SoGLImage::SCALE_DOWN) {
+      // no use scaling down for very small images
+      if (newx > xsize && newx > 16) newx >>= 1;
+      if (newy > ysize && newy > 16) newy >>= 1;
+      if (newz > zsize && newz > 16) newz >>= 1;
+    }
+    else if (this->flags & SoGLImage::USE_QUALITY_VALUE) {
+      if (this->quality < COIN_TEX2_SCALEUP_LIMIT) {
+        if ((newx >= 256) && ((newx - (xsize-2*this->border)) > (newx>>3)))
+          newx >>= 1;
+        if ((newy >= 256) && ((newy - (ysize-2*this->border)) > (newy>>3)))
+          newy >>= 1;
+        if ((newz >= 256) && ((newz - (zsize-2*this->border)) > (newz>>3)))
+          newz >>= 1;
+      }
     }
   }
+  else {
+    GLint maxr;
+    glGetIntegerv(GL_MAX_RECTANGLE_TEXTURE_SIZE_EXT, &maxr);
+    maxrectsize = (uint32_t) maxr;
+  }
+
 
   // downscale to legal GL size (implementation dependent)
   const cc_glglue * glw = sogl_glue_instance(state);
@@ -1320,10 +1403,17 @@ SoGLImageP::resizeImage(SoState * state, unsigned char *& imageptr,
     SbBool compressed = 
       (this->flags & SoGLImage::COMPRESSED) ? TRUE : FALSE &&
       cc_glue_has_texture_compression(glw);
-    sizeok = coin_glglue_is_texture_size_legal(glw, newx, newy, newz, 
-                                               numcomponents, 
-                                               this->shouldCreateMipmap(),
-                                               compressed);
+    
+    if (this->flags & SoGLImage::RECTANGLE) {
+      // FIXME: add support for rectangular textures in glglue proxy test
+      sizeok = (newx <= maxrectsize) && (newy <= maxrectsize); 
+    }
+    else {
+      sizeok = coin_glglue_is_texture_size_legal(glw, newx, newy, newz, 
+                                                 numcomponents, 
+                                                 this->shouldCreateMipmap(),
+                                                 compressed);
+    }
     if (!sizeok) {
       unsigned int max = SbMax(newx, SbMax(newy, newz));
       if (max==newz) newz >>= 1;
@@ -1368,7 +1458,6 @@ SoGLImageP::resizeImage(SoState * state, unsigned char *& imageptr,
 
   if ((newx != xsize) || (newy != ysize) || (newz != zsize)) {
     // We need to resize.
-
     int numbytes = newx * newy * ((newz==0)?1:newz) * numcomponents;
     unsigned char * glimage_tmpimagebuffer = glimage_get_buffer(numbytes, FALSE);
 
@@ -1384,7 +1473,6 @@ SoGLImageP::resizeImage(SoState * state, unsigned char *& imageptr,
                                               this->owner);
     }
 #endif // COIN_NEXT_MINOR
-
     if (!customresizedone) {
       // simage version 1.1.1 has a pretty high quality resize
       // function. We prefer to use that to avoid using GLU, since
@@ -1489,6 +1577,20 @@ SoGLImageP::createGLDisplayList(SoState *state)
                                             SoGLDisplayList::TEXTURE_OBJECT,
                                             1, mipmap);
   dl->ref();
+
+#ifdef COIN_NEXT_MINOR
+  if (bytes) {
+    SbBool is3D = (size[2]==0)?FALSE:TRUE;
+    if (is3D) {
+      dl->setTextureTarget((int) GL_TEXTURE_3D);
+    }
+    else {
+      dl->setTextureTarget((int) ((this->flags & SoGLImage::RECTANGLE) ?
+                                  GL_TEXTURE_RECTANGLE_EXT : GL_TEXTURE_2D));
+    }
+  }
+#endif // COIN_NEXT_MINOR
+
   dl->open(state);
 
 
@@ -1603,7 +1705,7 @@ SoGLImageP::reallyCreateTexture(SoState *state,
                                 const int w, const int h, const int d,
                                 const SbBool dlist, //FIXME: Not in use (kintel 20011129)
                                 const SbBool mipmap,
-                                const int borderarg)
+                                const int border)
 {
   const cc_glglue * glw = sogl_glue_instance(state);
   this->glsize = SbVec3s((short) w, (short) h, (short) d);
@@ -1633,7 +1735,7 @@ SoGLImageP::reallyCreateTexture(SoState *state,
     if (!mipmap) {
       if (cc_glglue_has_3d_textures(glw)) {
         cc_glglue_glTexImage3D(glw, GL_TEXTURE_3D, 0, internalFormat, w, h, d,
-                               borderarg, dataFormat, GL_UNSIGNED_BYTE, texture);
+                               border, dataFormat, GL_UNSIGNED_BYTE, texture);
       }
     }
     else { // mipmaps
@@ -1670,12 +1772,26 @@ SoGLImageP::reallyCreateTexture(SoState *state,
       }
       else mipmapfilter = FALSE;
     }
-    this->applyFilter(mipmapfilter);
     
+    else if (mipmap && COIN_TEX2_USE_SGIS_GENERATE_MIPMAP && 
+             cc_glglue_glext_supported(glw, "GL_SGIS_generate_mipmap")) {
+      glTexParameteri(target, GL_GENERATE_MIPMAP_SGIS, GL_TRUE);        
+      mipmapimage = FALSE;
+    }
+
+    this->applyFilter(mipmapfilter);
+#ifdef COIN_NEXT_MINOR
+    if ((this->quality > COIN_TEX2_ANISOTROPIC_LIMIT) && 
+        cc_glglue_can_do_anisotropic_filtering(glw)) {
+      glTexParameterf(target, GL_TEXTURE_MAX_ANISOTROPY_EXT, 
+                      cc_glglue_get_max_anisotropy(glw));
+    }
+#endif // COIN_NEXT_MINOR
     if (!mipmapimage) {
       // Create only level 0 texture. Mimpamps might be created by GL_SGIS_generate_mipmap
+
       glTexImage2D(target, 0, internalFormat, w, h,
-                   borderarg, dataFormat, GL_UNSIGNED_BYTE, texture);
+                   border, dataFormat, GL_UNSIGNED_BYTE, texture);
     }
     else { // mipmaps
       // The GLU function invocation has been disabled, for the
@@ -1685,9 +1801,7 @@ SoGLImageP::reallyCreateTexture(SoState *state,
       //   (void)GLUWrapper()->gluBuild2DMipmaps(GL_TEXTURE_2D, internalFormat,
       //                                         w, h, dataFormat,
       //                                         GL_UNSIGNED_BYTE, texture);
-
       fast_mipmap(state, w, h, numComponents, texture, FALSE, compress);
-
     }
   }
   glPixelStorei(GL_UNPACK_ALIGNMENT, 4);
@@ -1865,13 +1979,13 @@ SoGLImageP::getNextGLImageId(void)
 
 static SbList <SoGLImage*> * glimage_reglist;
 static uint32_t glimage_maxage = 60;
-static void * glimage_reglist_mutex = NULL;
 
 static void
 regimage_cleanup(void)
 {
-  CC_MUTEX_DESTRUCT(glimage_reglist_mutex);
   delete glimage_reglist;
+  glimage_reglist = NULL;
+  glimage_maxage = 60;
 }
 
 /*!
@@ -1899,10 +2013,10 @@ SoGLImage::tagImage(SoState *state, SoGLImage *image)
 {
   assert(image);
   if (image) {
-    LOCK_DLISTS(image);
+    LOCK_GLIMAGE;
     image->resetAge();
     image->pimpl->tagDL(state);
-    UNLOCK_DLISTS(image);
+    UNLOCK_GLIMAGE;
   }
 }
 
@@ -1917,7 +2031,7 @@ void
 SoGLImage::endFrame(SoState *state)
 {
   if (glimage_reglist) {
-    CC_MUTEX_LOCK(glimage_reglist_mutex);
+    LOCK_GLIMAGE;
     int n = glimage_reglist->getLength();
     for (int i = 0; i < n; i++) {
       SoGLImage *img = (*glimage_reglist)[i];
@@ -1925,7 +2039,7 @@ SoGLImage::endFrame(SoState *state)
       if (img->pimpl->endframecb)
         img->pimpl->endframecb(img->pimpl->endframeclosure);
     }
-    CC_MUTEX_UNLOCK(glimage_reglist_mutex);
+    UNLOCK_GLIMAGE;
   }
 }
 
@@ -1976,18 +2090,14 @@ SoGLImage::setDisplayListMaxAge(const uint32_t maxage)
 void
 SoGLImage::registerImage(SoGLImage *image)
 {
-  if (!glimage_reglist_mutex) {
-    CC_MUTEX_CONSTRUCT(glimage_reglist_mutex);
-  }
-
-  CC_MUTEX_LOCK(glimage_reglist_mutex);
+  LOCK_GLIMAGE;
   if (glimage_reglist == NULL) {
-    coin_atexit((coin_atexit_f *)regimage_cleanup, 0);
+    coin_atexit((coin_atexit_f *)regimage_cleanup, CC_ATEXIT_NORMAL);
     glimage_reglist = new SbList<SoGLImage*>;
   }
   assert(glimage_reglist->find(image) < 0);
   glimage_reglist->append(image);
-  CC_MUTEX_UNLOCK(glimage_reglist_mutex);
+  UNLOCK_GLIMAGE;
 }
 
 // used internally to keep track of the SoGLImages
@@ -1996,20 +2106,20 @@ SoGLImage::unregisterImage(SoGLImage *image)
 {
   assert(glimage_reglist);
 
-  CC_MUTEX_LOCK(glimage_reglist_mutex);
+  LOCK_GLIMAGE;
   int idx = glimage_reglist->find(image);
   assert(idx >= 0);
   if (idx >= 0) {
     glimage_reglist->removeFast(idx);
   }
-  CC_MUTEX_UNLOCK(glimage_reglist_mutex);
+  UNLOCK_GLIMAGE;
 }
 
 #ifdef COIN_NEXT_MINOR
 /*!
   Sets a custom image resize function.
 
-  \since 2005-06-03
+  \since Coin 2.5
 */
 void
 SoGLImage::setResizeCallback(SoGLImageResizeCB * f, void * closure)
@@ -2029,7 +2139,7 @@ SoGLImageP::contextCleanup(uint32_t context, void * closure)
 {
   SoGLImageP * thisp = (SoGLImageP *) closure;
 #ifdef COIN_THREADSAFE
-  thisp->mutex.lock();
+  SoGLImageP::mutex->lock();
 #endif // COIN_THREADSAFE
   
   int n = thisp->dlists.getLength();
@@ -2044,13 +2154,13 @@ SoGLImageP::contextCleanup(uint32_t context, void * closure)
     else i++;
   }
 #ifdef COIN_THREADSAFE
-  thisp->mutex.unlock();
+  SoGLImageP::mutex->unlock();
 #endif // COIN_THREADSAFE
 }
 
 // *************************************************************************
 
 #undef PRIVATE
-#undef LOCK_DLISTS
-#undef UNLOCK_DLISTS
+#undef LOCK_GLIMAGE
+#undef UNLOCK_GLIMAGE
 
