@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -98,7 +98,6 @@
 #include <Inventor/C/glue/gl.h>
 #include <coindefs.h> // COIN_OBSOLETED()
 
-
 /*!
   \enum SoBumpMap::Wrap
 
@@ -220,7 +219,7 @@ convert_heightmap_to_normalmap(const unsigned char * srcptr,
       n[0] = -dx;
       n[1] = -dy;
       n[2] = 1.0f;
-      n.normalize();
+      (void) n.normalize();
             
       *dstptr++ = (unsigned char) SbMin((n[0]+1.0f) * 128.0f, 255.0f); 
       *dstptr++ = (unsigned char) SbMin((n[1]+1.0f) * 128.0f, 255.0f); 
@@ -284,6 +283,8 @@ SoBumpMap::initClass(void)
   SO_NODE_INTERNAL_INIT_CLASS(SoBumpMap, SO_FROM_COIN_2_2);
 
   SO_ENABLE(SoGLRenderAction, SoBumpMapElement);
+  SO_ENABLE(SoCallbackAction, SoBumpMapElement);
+  SO_ENABLE(SoRayPickAction, SoBumpMapElement);
 }
 
 
@@ -365,7 +366,18 @@ SoBumpMap::GLRender(SoGLRenderAction * action)
 void
 SoBumpMap::doAction(SoAction * action)
 {
-  // will be implemented when (if) more actions support SoBumpMap.
+  SoState * state = action->getState();
+
+  int nc;
+  SbVec2s size;
+  const unsigned char * bytes = this->image.getValue(size, nc);
+  
+  if (bytes && size != SbVec2s(0,0)) {
+    SoShapeStyleElement::setBumpmapEnabled(state, TRUE);
+  }
+  else {
+    SoShapeStyleElement::setBumpmapEnabled(state, FALSE);
+  }
 }
 
 // doc from parent
@@ -380,8 +392,7 @@ SoBumpMap::callback(SoCallbackAction * action)
 void
 SoBumpMap::rayPick(SoRayPickAction * action)
 {
-  // not supported for SoRayPickAction yet
-  // SoBumpMap::doAction(action);
+  SoBumpMap::doAction(action);
 }
 
 // Documented in superclass. Overridden to detect when fields change.
