@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -339,6 +339,9 @@ public:
   void selectAndReset(SoHandleEventAction * action);
   void performSelection(SoHandleEventAction * action);
 
+  void validateViewportBBox(SbBox2s & bbox, 
+                            const SbVec2s & vpsize);
+
   // This keeps track of the state of a rectangle or lasso selection
   // operation.
   struct SelectionState {
@@ -1485,6 +1488,7 @@ SoExtSelectionP::cameraCB(void * data,
   for (int i = 0; i < PRIVATE(thisp)->runningselection.coords.getLength(); i++) {
     rectbbox.extendBy(PRIVATE(thisp)->runningselection.coords[i]);
   }
+  PRIVATE(thisp)->validateViewportBBox(rectbbox, vp.getViewportSizePixels());
 
   SbVec2s org = vp.getViewportOriginPixels();
   SbVec2s siz = vp.getViewportSizePixels();
@@ -2508,8 +2512,10 @@ SoExtSelectionP::scanOffscreenBuffer(SoNode *sceneRoot)
                (this->maximumcolorcounter + 7) / 8);
 
   SbBox2s rectbbox;
-  for (int k = 0; k < this->runningselection.coords.getLength(); k++)
+  for (int k = 0; k < this->runningselection.coords.getLength(); k++) {
     rectbbox.extendBy(this->runningselection.coords[k]);
+  }
+  this->validateViewportBBox(rectbbox, SbVec2s(offscreenSizeX, offscreenSizeY));
 
   // We must also deform lassocoords if offscreen size is changed
   const int minx = (int) (rectbbox.getMin()[0] * ((float) offscreenSizeX/requestedsize[0]));
@@ -2755,3 +2761,25 @@ SoExtSelectionP::performSelection(SoHandleEventAction * action)
   PUBLIC(this)->finishCBList->invokeCallbacks(PUBLIC(this));
   PUBLIC(this)->touch();
 }
+
+//
+// avoid an empty viewport bbox (support for a single click and 
+// a 1-pixel-size rectangles/lassos).
+//
+void 
+SoExtSelectionP::validateViewportBBox(SbBox2s & bbox, 
+                                     const SbVec2s & vpsize)
+{
+  const SbVec2s bmin = bbox.getMin();
+  const SbVec2s bmax = bbox.getMax();
+
+  if (bmin[0] == bmax[0]) {
+    int add = (bmin[0] < (vpsize[0] - 1)) ? 1 : -1;
+    bbox.extendBy(SbVec2s(bmin[0] + add, bmin[1]));
+  }
+
+  if (bmin[1] == bmax[1]) {
+    int add = (bmin[1] < (vpsize[1] - 1)) ? 1 : -1;
+    bbox.extendBy(SbVec2s(bmin[0], bmin[1] + add));
+  }
+}
