@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -77,9 +77,15 @@
 class SoFileP {
 public:
   static const char UNDEFINED_FILE[];
+#ifdef COIN_NEXT_MINOR
+  static SbBool searchok;
+#endif // COIN_NEXT_MINOR
 };
 
 const char SoFileP::UNDEFINED_FILE[] = "<Undefined file>";
+#ifdef COIN_NEXT_MINOR
+SbBool SoFileP::searchok = FALSE;
+#endif // COIN_NEXT_MINOR
 
 // *************************************************************************
 
@@ -409,6 +415,14 @@ SoFile::audioRender(SoAudioRenderAction * action)
   SoFile::doAction((SoAction *)action);
 }
 
+#ifdef COIN_NEXT_MINOR
+void
+SoFile::search(SoSearchAction * action)
+{
+  if (SoFileP::searchok) SoFile::doAction((SoAction *)action);
+}
+#endif // COIN_NEXT_MINOR
+
 // Doc from superclass.
 void
 SoFile::copyContents(const SoFieldContainer * from, SbBool copyconnections)
@@ -427,3 +441,23 @@ SoFile::copyContents(const SoFieldContainer * from, SbBool copyconnections)
     this->children->append(cp);
   }
 }
+
+#ifdef COIN_NEXT_MINOR
+/*!
+  Enables/disables searching children using SoSearchAction.
+*/
+void
+SoFile::setSearchOK(SbBool dosearch)
+{
+  SoFileP::searchok = dosearch;
+}
+
+/*!
+  Returns whether searching children using SoSearchAction is enabled.
+*/
+SbBool
+SoFile::getSearchOK()
+{
+  return SoFileP::searchok;
+}
+#endif // COIN_NEXT_MINOR
