@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -55,6 +55,10 @@
 #include <Inventor/actions/SoGLRenderAction.h>
 #include <Inventor/elements/SoOverrideElement.h>
 #include <Inventor/elements/SoGLLazyElement.h>
+#include <Inventor/elements/SoGLVBOElement.h>
+#include <Inventor/C/tidbits.h>
+#include "../misc/SoVBO.h"
+#include "../misc/PointerMap.h"
 
 // *************************************************************************
 
@@ -72,6 +76,17 @@
 
 // *************************************************************************
 
+class SoPackedColorP {
+ public:
+  SoPackedColorP() : vbo(NULL) { }
+  ~SoPackedColorP() { delete this->vbo; }
+  SbBool transparent;
+  SbBool checktransparent;
+  SoVBO * vbo;
+};
+
+#define PRIVATE(obj) ((SoPackedColorP*) PointerMap::get(obj))
+
 SO_NODE_SOURCE(SoPackedColor);
 
 /*!
@@ -79,12 +94,14 @@ SO_NODE_SOURCE(SoPackedColor);
 */
 SoPackedColor::SoPackedColor()
 {
+  SoPackedColorP * pimpl = new SoPackedColorP;
+  PointerMap::add(this, pimpl);
   SO_NODE_INTERNAL_CONSTRUCTOR(SoPackedColor);
-
+  
   SO_NODE_ADD_FIELD(orderedRGBA, (0xccccccff));
 
-  this->checktransparent = FALSE;
-  this->transparent = FALSE;
+  pimpl->checktransparent = FALSE;
+  pimpl->transparent = FALSE;
 }
 
 /*!
@@ -92,6 +109,9 @@ SoPackedColor::SoPackedColor()
 */
 SoPackedColor::~SoPackedColor()
 {
+  SoPackedColorP * pimpl = PRIVATE(this);
+  PointerMap::remove(this);
+  delete pimpl;
 }
 
 // Doc from superclass.
@@ -115,16 +135,66 @@ SoPackedColor::GLRender(SoGLRenderAction * action)
 void
 SoPackedColor::doAction(SoAction * action)
 {
+  SoPackedColorP * pimpl = PRIVATE(this);
+  
   (void) this->isTransparent(); // update cached value for transparent
-
+  
   SoState * state = action->getState();
+  const int num = this->orderedRGBA.getNum();
   if (!this->orderedRGBA.isIgnored() &&
-      this->orderedRGBA.getNum() > 0 &&
+      num > 0 &&
       !SoOverrideElement::getDiffuseColorOverride(state)) {
     SoLazyElement::setPacked(state, this,
-                             this->orderedRGBA.getNum(),
+                             num,
                              this->orderedRGBA.getValues(0),
-                             this->transparent);
+                             pimpl->transparent);
+    
+#ifdef COIN_NEXT_MINOR
+    if (state->isElementEnabled(SoGLVBOElement::getClassStackIndex())) {
+      SoBase::staticDataLock();
+      SbBool setvbo = FALSE;
+      if (SoGLVBOElement::shouldCreateVBO(state, num)) {
+        SbBool dirty = FALSE;
+        setvbo = TRUE;
+        if (pimpl->vbo == NULL) {
+          pimpl->vbo = new SoVBO(GL_ARRAY_BUFFER, GL_STATIC_DRAW);
+          dirty = TRUE;
+        }
+        else if (pimpl->vbo->getBufferDataId() != this->getNodeId()) {
+          dirty = TRUE;
+        }
+        if (dirty) {
+          if (coin_host_get_endianness() == COIN_HOST_IS_BIGENDIAN) {
+            pimpl->vbo->setBufferData(this->orderedRGBA.getValues(0),
+                                      num*sizeof(uint32_t),
+                                      this->getNodeId());
+          }
+          else {
+            // sigh. Need to swap bytes on little endian systems.
+            const uint32_t * src = this->orderedRGBA.getValues(0);
+            uint32_t * dst = (uint32_t*)
+              pimpl->vbo->allocBufferData(num*sizeof(uint32_t),
+                                          this->getNodeId());
+            for (int i = 0; i < num; i++) {
+              uint32_t tmp = src[i];
+              dst[i] =
+                (tmp << 24) |
+                ((tmp & 0xff00) << 8) |
+                ((tmp & 0xff0000) >> 8) |
+                (tmp >> 24);
+            }
+          }
+        }
+      }
+      else if (pimpl->vbo) {
+        pimpl->vbo->setBufferData(NULL, 0, 0);
+      }
+      SoBase::staticDataUnlock();
+      if (setvbo) {
+        SoGLVBOElement::setColorVBO(state, pimpl->vbo);
+      }
+    }
+#endif // COIN_NEXT_MINOR
     if (this->isOverride()) {
       SoOverrideElement::setDiffuseColorOverride(state, this, TRUE);
     }
@@ -145,18 +215,20 @@ SoPackedColor::callback(SoCallbackAction * action)
 SbBool
 SoPackedColor::isTransparent(void)
 {
-  if (this->checktransparent) {
-    this->checktransparent = FALSE;
-    this->transparent = FALSE;
+  SoPackedColorP * pimpl = PRIVATE(this);
+
+  if (pimpl->checktransparent) {
+    pimpl->checktransparent = FALSE;
+    pimpl->transparent = FALSE;
     int n = this->orderedRGBA.getNum();
     for (int i = 0; i < n; i++) {
       if ((this->orderedRGBA[i] & 0xff) != 0xff) {
-        this->transparent = TRUE;
+        pimpl->transparent = TRUE;
         break;
       }
     }
   }
-  return this->transparent;
+  return pimpl->transparent;
 }
 
 // Documented in superclass.
@@ -167,7 +239,9 @@ SoPackedColor::notify(SoNotList *list)
 
   SoField *f = list->getLastField();
   if (f == &this->orderedRGBA) {
-    this->checktransparent = TRUE;
+    PRIVATE(this)->checktransparent = TRUE;
   }
   inherited::notify(list);
 }
+
+#undef PRIVATE
