@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2005 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -14,10 +14,10 @@
  *  support services, please contact Systems in Motion about acquiring
  *  a Coin Professional Edition License.
  *
- *  See <URL:http://www.coin3d.org/> for more information.
+ *  See http://www.coin3d.org/ for more information.
  *
  *  Systems in Motion, Postboks 1283, Pirsenteret, 7462 Trondheim, NORWAY.
- *  <URL:http://www.sim.no/>.
+ *  http://www.sim.no/  sales@sim.no  coin-support@coin3d.org
  *
 \**************************************************************************/
 
@@ -312,7 +312,7 @@
   Note that in Coin-2 this is a hidden field and can only be accessed
   through SoFieldContainer::getField().
 
-  \since 2005-06-03
+  \since Coin 2.4.2
   \since TGS Inventor 4.0
 */
 
@@ -473,14 +473,20 @@ SoTexture2::initClass(void)
   SO_ENABLE(SoGLRenderAction, SoGLTextureEnabledElement);
   SO_ENABLE(SoGLRenderAction, SoGLTexture3EnabledElement);
 
+  SO_ENABLE(SoGLRenderAction, SoGLMultiTextureImageElement);
+  SO_ENABLE(SoGLRenderAction, SoGLMultiTextureEnabledElement);
+
   SO_ENABLE(SoCallbackAction, SoTextureImageElement);
   SO_ENABLE(SoCallbackAction, SoTextureEnabledElement);
+  SO_ENABLE(SoCallbackAction, SoMultiTextureEnabledElement);
+  SO_ENABLE(SoCallbackAction, SoMultiTextureImageElement);
   SO_ENABLE(SoCallbackAction, SoTexture3EnabledElement);
 
   SO_ENABLE(SoRayPickAction, SoTextureImageElement);
   SO_ENABLE(SoRayPickAction, SoTextureEnabledElement);
   SO_ENABLE(SoRayPickAction, SoTexture3EnabledElement);
-
+  SO_ENABLE(SoRayPickAction, SoMultiTextureEnabledElement);
+  SO_ENABLE(SoRayPickAction, SoMultiTextureImageElement);
 }
 
 
@@ -523,29 +529,28 @@ SoTexture2::GLRender(SoGLRenderAction * action)
   float quality = SoTextureQualityElement::get(state);
 
   const cc_glglue * glue = cc_glglue_instance(SoGLCacheContextElement::get(state));
-
+  SoTextureScalePolicyElement::Policy scalepolicy =
+    SoTextureScalePolicyElement::get(state);
+  SbBool needbig = (scalepolicy == SoTextureScalePolicyElement::FRACTURE);
+  SoType glimagetype = PRIVATE(this)->glimage ? PRIVATE(this)->glimage->getTypeId() : SoType::badType();
+  
   LOCK_GLIMAGE(this);
-
-  if (!PRIVATE(this)->glimagevalid) {
+  
+  if (!PRIVATE(this)->glimagevalid || 
+      (needbig && glimagetype != SoGLBigImage::getClassTypeId()) ||
+      (!needbig && glimagetype != SoGLImage::getClassTypeId())) {
     int nc;
     SbVec2s size;
     const unsigned char * bytes =
       this->image.getValue(size, nc);
     
-    SoTextureScalePolicyElement::Policy scalepolicy =
-      SoTextureScalePolicyElement::get(state);
-      
-    SbBool needbig = (scalepolicy == SoTextureScalePolicyElement::FRACTURE);
-
     if (needbig &&
-        (PRIVATE(this)->glimage == NULL ||
-         PRIVATE(this)->glimage->getTypeId() != SoGLBigImage::getClassTypeId())) {
+        (glimagetype != SoGLBigImage::getClassTypeId())) {
       if (PRIVATE(this)->glimage) PRIVATE(this)->glimage->unref(state);
       PRIVATE(this)->glimage = new SoGLBigImage();
     }
     else if (!needbig &&
-             (PRIVATE(this)->glimage == NULL ||
-              PRIVATE(this)->glimage->getTypeId() != SoGLImage::getClassTypeId())) {
+             (glimagetype != SoGLImage::getClassTypeId())) {
       if (PRIVATE(this)->glimage) PRIVATE(this)->glimage->unref(state);
       PRIVATE(this)->glimage = new SoGLImage();
     }
@@ -637,42 +642,58 @@ SoTexture2::doAction(SoAction * action)
 {
   SoState * state = action->getState();
 
-  if (SoTextureOverrideElement::getImageOverride(state))
+  int unit = SoTextureUnitElement::get(state);
+  if ((unit == 0) && SoTextureOverrideElement::getImageOverride(state))
     return;
 
   int nc;
   SbVec2s size;
   const unsigned char * bytes = this->image.getValue(size, nc);
-
-  SoTexture3EnabledElement::set(state, this, FALSE);
-
-  if (size != SbVec2s(0,0)) {
-    SoTextureImageElement::set(state, this,
-                               size, nc, bytes,
-                               (int)this->wrapT.getValue(),
-                               (int)this->wrapS.getValue(),
-                               (SoTextureImageElement::Model) model.getValue(),
-                               this->blendColor.getValue());
-    SoTextureEnabledElement::set(state, this, TRUE);
-  }
+  
   // if a filename has been set, but the file has not been loaded, supply
   // a dummy texture image to make sure texture coordinates are generated.
-  else if (this->image.isDefault() && this->filename.getValue().getLength()) {
+  if ((size == SbVec2s(0,0)) &&
+      this->image.isDefault() && 
+      this->filename.getValue().getLength()) {
     static const unsigned char dummytex[] = {0xff,0xff,0xff,0xff};
-    SoTextureImageElement::set(state, this,
-                               SbVec2s(2,2), 1, dummytex,
-                               (int)this->wrapT.getValue(),
-                               (int)this->wrapS.getValue(),
-                               (SoTextureImageElement::Model) model.getValue(),
-                               this->blendColor.getValue());
-    SoTextureEnabledElement::set(state, this, TRUE);
+    bytes = dummytex;
+    size = SbVec2s(2,2);
+    nc = 1;
+  }
+
+  if (unit == 0) {
+    SoTexture3EnabledElement::set(state, this, FALSE);
+    if (size != SbVec2s(0,0)) {
+      SoTextureImageElement::set(state, this,
+                                 size, nc, bytes,
+                                 (int)this->wrapT.getValue(),
+                                 (int)this->wrapS.getValue(),
+                                 (SoTextureImageElement::Model) model.getValue(),
+                                 this->blendColor.getValue());
+      SoTextureEnabledElement::set(state, this, TRUE); 
+    }
+    else {
+      SoTextureImageElement::setDefault(state, this);
+      SoTextureEnabledElement::set(state, this, FALSE);
+    }
+    if (this->isOverride()) {
+      SoTextureOverrideElement::setImageOverride(state, TRUE);
+    }
   }
   else {
-    SoTextureImageElement::setDefault(state, this);
-    SoTextureEnabledElement::set(state, this, FALSE);
-  }
-  if (this->isOverride()) {
-    SoTextureOverrideElement::setImageOverride(state, TRUE);
+    if (size != SbVec2s(0,0)) {
+      SoMultiTextureImageElement::set(state, this, unit,
+                                      size, nc, bytes,
+                                      (SoTextureImageElement::Wrap)this->wrapT.getValue(),
+                                      (SoTextureImageElement::Wrap)this->wrapS.getValue(),
+                                      (SoTextureImageElement::Model) model.getValue(),
+                                      this->blendColor.getValue());
+      SoMultiTextureEnabledElement::set(state, this, unit, TRUE); 
+    }
+    else {
+      SoMultiTextureImageElement::setDefault(state, this, unit);
+      SoMultiTextureEnabledElement::set(state, this, unit, FALSE);
+    }
   }
 }
 
