@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -1806,14 +1806,16 @@ SoShape::finishVertexArray(SoGLRenderAction * action,
     cc_glglue_glDisableClientState(glue, GL_NORMAL_ARRAY);
   }
   if (texpervertex) {
-    const SoTextureCoordinateElement * telem = NULL;
-    const SoMultiTextureCoordinateElement * mtelem = NULL;
-    const SbBool * enabledunits = NULL;
     int lastenabled;
+    const SbBool * enabledunits = 
+      SoMultiTextureEnabledElement::getEnabledUnits(state, lastenabled);
 
-    telem = SoTextureCoordinateElement::getInstance(state);
-    enabledunits = SoMultiTextureEnabledElement::getEnabledUnits(state, lastenabled);
+    const SoMultiTextureCoordinateElement * mtelem = 
+      SoMultiTextureCoordinateElement::getInstance(state);
     
+    const SoTextureCoordinateElement * telem = 
+      SoTextureCoordinateElement::getInstance(state);
+
     for (int i = 1; i <= lastenabled; i++) {
       if (enabledunits[i] && mtelem->getNum(i)) {
         cc_glglue_glClientActiveTexture(glue, GL_TEXTURE0 + i);
