@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -605,7 +605,7 @@ coin_getenv(const char * envname)
       envptr = (struct envvar_data *) malloc(sizeof(struct envvar_data));
       if (envptr == NULL) {
         /* Augh. Could we handle this any better? */
-	/* We can alternatively ignore the bookkeeping and leak the buffer
+        /* We can alternatively ignore the bookkeeping and leak the buffer
            - 20030205 larsa */
         free(valbuf);
         return NULL;
@@ -613,11 +613,11 @@ coin_getenv(const char * envname)
       envptr->name = strdup(envname);
       if (envptr->name == NULL) {
         /* Augh. Could we handle this any better? */
-	/* We can alternatively ignore the bookkeeping and leak the buffer
+        /* We can alternatively ignore the bookkeeping and leak the buffer
            - 20030205 larsa */
-	free(envptr);
-	free(valbuf);
-	return NULL;
+        free(envptr);
+        free(valbuf);
+        return NULL;
       }
       envptr->val = valbuf;
       envlist_append(envptr);
@@ -708,15 +708,15 @@ coin_setenv(const char * name, const char * value, int overwrite)
 
     if (putenv(envptr->string) == -1) { /* denied! */
       if (oldbuf) {
-	/* we had old value - setting new didn't work, so we do a rollback and assume we
-	   still need to do bookkeeping of the old value */
-	free(envptr->string);
-	envptr->string = oldbuf;
-	return FALSE;
+        /* we had old value - setting new didn't work, so we do a rollback and assume we
+           still need to do bookkeeping of the old value */
+        free(envptr->string);
+        envptr->string = oldbuf;
+        return FALSE;
       }
       else {
-	free(envptr->string);
-	free(envptr);
+        free(envptr->string);
+        free(envptr);
       }
       return FALSE;
     }
@@ -814,6 +814,11 @@ coin_strncasecmp(const char * s1, const char * s2, int len)
 
 /**************************************************************************/
 
+#define COIN_BSWAP_8(x)  ((x) & 0xff)
+#define COIN_BSWAP_16(x) ((COIN_BSWAP_8(x)  << 8)  | COIN_BSWAP_8((x)  >> 8))
+#define COIN_BSWAP_32(x) ((COIN_BSWAP_16(x) << 16) | COIN_BSWAP_16((x) >> 16))
+#define COIN_BSWAP_64(x) ((COIN_BSWAP_32(x) << 32) | COIN_BSWAP_32((x) >> 32))
+
 static int coin_endianness = COIN_HOST_IS_UNKNOWNENDIAN;
 
 int
@@ -824,43 +829,32 @@ coin_host_get_endianness(void)
     uint8_t  bytes[4];
   } temp;
 
-  if (coin_endianness != COIN_HOST_IS_UNKNOWNENDIAN)
+  if (coin_endianness != COIN_HOST_IS_UNKNOWNENDIAN) {
     return coin_endianness;
+  }
 
   temp.bytes[0] = 0x00;
   temp.bytes[1] = 0x01;
   temp.bytes[2] = 0x02;
   temp.bytes[3] = 0x03;
   switch (temp.value) {
-  case 0x03020100: return COIN_HOST_IS_LITTLEENDIAN;
-  case 0x00010203: return COIN_HOST_IS_BIGENDIAN;
-  /* might be more variations here for some obscure CPU architectures */
+    case 0x03020100: return coin_endianness = COIN_HOST_IS_LITTLEENDIAN;
+    case 0x00010203: return coin_endianness = COIN_HOST_IS_BIGENDIAN;
+    /* might be more variations here for some obscure CPU architectures */
   }
   assert(0 && "system has unknown endianness");
   return COIN_HOST_IS_UNKNOWNENDIAN; /* maybe just as well exit()? */
 }
 
-static void
-coin_swap_16bit_word(uint8_t * block)
-{
-  uint8_t tmp;
-
-  tmp = block[1];
-  block[1] = block[0];
-  block[0] = tmp;
-}
-
 uint16_t
 coin_hton_uint16(uint16_t value)
 {
-  if (coin_endianness == COIN_HOST_IS_UNKNOWNENDIAN)
-    coin_endianness = coin_host_get_endianness();
-  switch (coin_endianness) {
+  switch (coin_host_get_endianness()) {
   case COIN_HOST_IS_BIGENDIAN:
     /* value = value */
     break;
   case COIN_HOST_IS_LITTLEENDIAN:
-    coin_swap_16bit_word((uint8_t *)&value);
+    value = COIN_BSWAP_16(value);
     break;
   default:
     assert(0 && "system has unknown endianness");
@@ -874,31 +868,15 @@ coin_ntoh_uint16(uint16_t value)
   return coin_hton_uint16(value);
 }
 
-static void
-coin_swap_32bit_word(uint8_t * block)
-{
-  uint8_t tmp;
-
-  tmp = block[3];
-  block[3] = block[0];
-  block[0] = tmp;
-
-  tmp = block[2];
-  block[2] = block[1];
-  block[1] = tmp;
-}
-
 uint32_t
 coin_hton_uint32(uint32_t value)
 {
-  if (coin_endianness == COIN_HOST_IS_UNKNOWNENDIAN)
-    coin_endianness = coin_host_get_endianness();
-  switch (coin_endianness) {
+  switch (coin_host_get_endianness()) {
   case COIN_HOST_IS_BIGENDIAN:
     /* big-endian is the same order as network order */
     break;
   case COIN_HOST_IS_LITTLEENDIAN:
-    coin_swap_32bit_word((uint8_t *)&value);
+    value = COIN_BSWAP_32(value);
     break;
   default:
     assert(0 && "system has unknown endianness");
@@ -915,19 +893,26 @@ coin_ntoh_uint32(uint32_t value)
 float
 coin_hton_float(float value)
 {
-  if (coin_endianness == COIN_HOST_IS_UNKNOWNENDIAN)
-    coin_endianness = coin_host_get_endianness();
-  switch (coin_endianness) {
+  union f32 {
+      float f32;
+      uint32_t u32;
+  } val;
+
+  val.f32 = value;
+
+  assert(4 == sizeof(float));
+
+  switch (coin_host_get_endianness()) {
   case COIN_HOST_IS_BIGENDIAN:
     /* big-endian is the same order as network order */
     break;
   case COIN_HOST_IS_LITTLEENDIAN:
-    coin_swap_32bit_word((uint8_t *)&value);
+    val.u32 = COIN_BSWAP_32(val.u32);
     break;
   default:
     assert(0 && "system has unknown endianness");
   }
-  return value;
+  return val.f32;
 }
 
 float
@@ -936,6 +921,37 @@ coin_ntoh_float(float value)
   return coin_hton_float(value);
 }
 
+double
+coin_hton_double(double value)
+{
+  union d64 {
+      double d64;
+      uint64_t u64;
+  } val;
+
+  val.d64 = value;
+
+  assert(8 == sizeof(double));
+
+  switch (coin_host_get_endianness()) {
+  case COIN_HOST_IS_BIGENDIAN:
+    /* big-endian is the same order as network order */
+    break;
+  case COIN_HOST_IS_LITTLEENDIAN:
+    val.u64 = COIN_BSWAP_64(val.u64);
+    break;
+  default:
+    assert(0 && "system has unknown endianness");
+  }
+  return val.d64;
+}
+
+double
+coin_ntoh_double(double value)
+{
+  return coin_hton_double(value);
+}
+
 /**************************************************************************/
 
 /*
@@ -957,8 +973,8 @@ coin_isspace(const char c)
 {
   /* This is what isspace() does under the POSIX and C locales,
      according to the GNU libc man page. */
-  return (c==' ') || (c=='\n') || (c=='\r') ||
-         (c=='\f') || (c=='\t') || (c=='\v');
+  return (c==' ') || (c=='\n') || (c=='\t') ||
+         (c=='\r') || (c=='\f') || (c=='\v');
 }
 
 /**************************************************************************/
