@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -100,6 +100,41 @@
   description of the bboxCenter and bboxSize fields is contained in
   4.6.4, Bounding boxes
   (<http://www.web3d.org/x3d/specifications/vrml/ISO-IEC-14772-VRML97/part1/concepts.html#4.6.4>),
+
+  \ENDWEB3D
+
+  The following example VRML scene is a simple illustration of how
+  SoVRMLBillboard::axisOfRotation constrains rotation around the given
+  vector:
+
+  \verbatim
+  #VRML V2.0 utf8
+  
+  Transform {
+     translation -2 0 0
+     children [
+        Billboard {
+  	 children [ Box { } ]
+        }
+     ]
+  }
+  
+  Transform {
+     translation 2 0 0
+     children [
+        Billboard {
+  	 axisOfRotation 0 1 0
+  	 children [ Box { } ]
+        }
+     ]
+  }
+  
+  Transform {
+     translation 0 -2 0
+     children [ Box { size 10 0.1 10 } ]
+  }
+  \endverbatim
+
 */
 
 // *************************************************************************
@@ -125,6 +160,7 @@
 #include <Inventor/VRMLnodes/SoVRMLBillboard.h>
 
 #include <math.h>
+#include <float.h>
 
 #include <Inventor/VRMLnodes/SoVRMLMacros.h>
 #include <Inventor/nodes/SoSubNodeP.h>
@@ -145,16 +181,6 @@
 #include <Inventor/system/gl.h>
 #include <Inventor/C/glue/glp.h>
 
-#include "../misc/PointerMap.h"
-
-// *************************************************************************
-
-class SoVRMLBillboardP {
-public:
-  SbRotation previousrotation;
-};
-
-#define PRIVATE(obj) ((SoVRMLBillboardP *) PointerMap::get(this))
 
 // *************************************************************************
 
@@ -174,8 +200,6 @@ SoVRMLBillboard::initClass(void)
 */
 SoVRMLBillboard::SoVRMLBillboard(void)
 {
-  PointerMap::add(this, new SoVRMLBillboardP);
-
   SO_VRMLNODE_INTERNAL_CONSTRUCTOR(SoVRMLBillboard);
 
   SO_VRMLNODE_ADD_EXPOSED_FIELD(axisOfRotation, (0.0f, 0.0f, 0.0f));
@@ -196,8 +220,6 @@ SoVRMLBillboard::SoVRMLBillboard(int numchildren)
 */
 SoVRMLBillboard::~SoVRMLBillboard()
 {
-  delete PRIVATE(this);
-  PointerMap::remove(this);
 }
 
 // *************************************************************************
@@ -327,7 +349,10 @@ SoVRMLBillboard::GLRenderInPath(SoGLRenderAction * action )
 {
   int numindices;
   const int * indices;
-  if (action->getPathCode(numindices, indices) == SoAction::IN_PATH) {
+
+  SoAction::PathCode pathcode = action->getPathCode(numindices, indices);
+
+  if (pathcode == SoAction::IN_PATH) {
     SoState * state = action->getState();
     SoCacheElement::invalidate(state);
     SoNode ** childarray = (SoNode**) this->getChildren()->getArrayPtr();
@@ -336,7 +361,6 @@ SoVRMLBillboard::GLRenderInPath(SoGLRenderAction * action )
 
     int childidx = 0;
     for (int i = 0; i < numindices; i++) {
-      SoAction::PathCode pathcode = action->getCurPathCode();
       for (; childidx < indices[i] && !action->hasTerminated(); childidx++) {
         SoNode * offpath = childarray[childidx];
         if (offpath->affectsState()) {
@@ -384,28 +408,33 @@ SoVRMLBillboard::notify(SoNotList * list)
 void
 SoVRMLBillboard::performRotation(SoState * state) const
 {
-  SbRotation rot;
-  const SbMatrix & mm = SoModelMatrixElement::get(state);
-
-  SbVec3f toviewer;
+  SbMatrix imm = SoModelMatrixElement::get(state).inverse();
   const SbViewVolume & vv = SoViewVolumeElement::get(state);
+  SbVec3f up, toviewer;
+  imm.multDirMatrix(vv.getViewUp(), up);
+  imm.multVecMatrix(vv.getProjectionPoint(), toviewer);
 
-  toviewer = - vv.getProjectionDirection();
-  (void) toviewer.normalize();
+  if (toviewer.normalize() < FLT_EPSILON) {
+    // we're not going to be able to calculate anything if the
+    // projection point is in origo
+    return;
+  }
 
   SbVec3f rotaxis = this->axisOfRotation.getValue();
+  SbVec3f yaxis(0.0f, 1.0f, 0.0f);
+  SbVec3f zaxis(0.0f, 0.0f, 1.0f);
+  
+  SbRotation rot = SbRotation::identity();
+  
   if (rotaxis == SbVec3f(0.0f, 0.0f, 0.0f)) {
     // 1. Compute the billboard-to-viewer vector.
     // 2. Rotate the Z-axis of the billboard to be collinear with the
     //    billboard-to-viewer vector and pointing towards the viewer's position.
     // 3. Rotate the Y-axis of the billboard to be parallel and oriented in the
     //    same direction as the Y-axis of the viewer.
-    rot.setValue(SbVec3f(0.0f, 0.0f, 1.0f), toviewer);
-    SbVec3f viewup = vv.getViewUp();
-    SbVec3f yaxis(0.0f, 1.0f, 0.0f);
+    rot.setValue(zaxis, toviewer);
     rot.multVec(yaxis, yaxis);
-    SbRotation rot2(yaxis, viewup);
-    rot = rot * rot2;
+    rot *= SbRotation(yaxis, up);
   }
   else {
     // 1. Compute the vector from the Billboard node's origin to the viewer's
@@ -415,21 +444,11 @@ SoVRMLBillboard::performRotation(SoState * state) const
     // 3. Rotate the local Z-axis of the billboard into the plane from 2.,
     //    pivoting around the axisOfRotation.
     SbVec3f planenormal(rotaxis.cross(toviewer));
-    float length = planenormal.length();
-    if (length == 0) {
-      // Resulting rotation of billboard is undefined here; we are going
-      // to apply previous rotation so that eventually no sudden changes
-      // in orientation arise when this situation occurs while smoothly
-      // moving camera.
-      rot = PRIVATE(this)->previousrotation;
-    }
-    else {
+    if (planenormal.normalize() > FLT_EPSILON) {
       // Calculate projection vector of local Z-axis to plane defined by
       // the axisOfRotation and the billboard-to-viewer vector and then
       // apply rotation around axisOfRotation that will rotate Z-axis to
       // this projection.
-      planenormal /= length;
-      SbVec3f zaxis(0.0f, 0.0f, 1.0f);
       SbVec3f vecinplane = zaxis - planenormal * planenormal[2];
       (void) vecinplane.normalize();
       if (vecinplane.dot(toviewer) < 0.0f) vecinplane = - vecinplane;
@@ -437,16 +456,8 @@ SoVRMLBillboard::performRotation(SoState * state) const
       rot.setValue(rotaxis, planenormal[2] < 0.0f ? angle : - angle);
     }
   }
-  SbVec3f translation, scale;
-  SbRotation rotation, scaleorientation;
-  SbMatrix mm2;
-  mm.getTransform(translation, rotation, scale, scaleorientation);
-  mm2.setTransform(translation, rot, scale);
-  SoModelMatrixElement::set(state, (SoNode*) this, mm2);
-
-  PRIVATE(this)->previousrotation = rot;
+  
+  SoModelMatrixElement::rotateBy(state, (SoNode*) this, rot);
 }
 
-#undef PRIVATE
-
 #endif // HAVE_VRML97
