@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -112,7 +112,10 @@ SoVRMLGeometry::shouldGLRender(SoGLRenderAction * action)
 void
 SoVRMLGeometry::setupShapeHints(SoState * state, const SbBool ccw, const SbBool solid)
 {
-  SoGLShapeHintsElement::forceSend(state, ccw, solid, !solid);
+  SoShapeHintsElement::set(state, this, 
+                           ccw ? SoShapeHintsElement::COUNTERCLOCKWISE : SoShapeHintsElement::CLOCKWISE,
+                           solid ? SoShapeHintsElement::SOLID : SoShapeHintsElement::UNKNOWN_SHAPE_TYPE,
+                           SoShapeHintsElement::FACE_TYPE_AS_IS);
 }
 
 // Doc in parent
