@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -432,13 +432,18 @@ SoVRMLIndexedFaceSet::GLRender(SoGLRenderAction * action)
   if (this->coordIndex.getNum() < 3 || this->coord.getValue() == NULL) return;
   SoState * state = action->getState();
 
+  state->push();
   // update state with coordinates, normals and texture information
   SoVRMLVertexShape::GLRender(action);
 
-  if (!this->shouldGLRender(action)) return;
+  if (!this->shouldGLRender(action)) { 
+    state->pop();
+    return;
+  }
+
+  this->setupShapeHints(state, this->ccw.getValue(), this->solid.getValue());
 
   Binding mbind = this->findMaterialBinding(state);
-
   Binding nbind = this->findNormalBinding(state);
 
   const SoCoordinateElement * coords;
@@ -516,8 +521,6 @@ SoVRMLIndexedFaceSet::GLRender(SoGLRenderAction * action)
 
   mb.sendFirst(); // make sure we have the correct material
 
-  this->setupShapeHints(state, this->ccw.getValue(), this->solid.getValue());
-
 #ifdef COIN_NEXT_MINOR
   SoGLLazyElement * lelem = NULL;
   const uint32_t contextid = action->getCacheContext();
@@ -629,6 +632,8 @@ SoVRMLIndexedFaceSet::GLRender(SoGLRenderAction * action)
 
   // send approx number of triangles for autocache handling
   sogl_autocache_update(state, this->coordIndex.getNum() / 4);
+
+  state->pop();
 }
 
 // Doc in parent
@@ -768,6 +773,7 @@ SoVRMLIndexedFaceSet::generatePrimitives(SoAction * action)
     }
   }
   
+  SbBool convexcacheused = FALSE;
   if (this->useConvexCache(action, normals, nindices, normalCacheUsed)) {
     cindices = PRIVATE(this)->convexCache->getCoordIndices();
     numindices = PRIVATE(this)->convexCache->getNumCoordIndices();
@@ -781,6 +787,7 @@ SoVRMLIndexedFaceSet::generatePrimitives(SoAction * action)
     else if (nbind == PER_FACE) nbind = PER_FACE_INDEXED;
 
     if (tbind != NONE) tbind = PER_VERTEX_INDEXED;
+    convexcacheused = TRUE;
   }
 
   int texidx = 0;
@@ -884,6 +891,9 @@ SoVRMLIndexedFaceSet::generatePrimitives(SoAction * action)
   if (normalCacheUsed) {
     this->readUnlockNormalCache();
   }
+  if (convexcacheused) {
+    PRIVATE(this)->readUnlockConvexCache();
+  }
   state->pop();
 }
 
