@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -250,15 +250,18 @@ SoVRMLIndexedLineSet::GLRender(SoGLRenderAction * action)
   if (this->coordIndex.getNum() < 2) return;
   
   SoState * state = action->getState();
-  
+  state->push();
+
   SoLazyElement::setLightModel(state, SoLazyElement::BASE_COLOR);
   SoGLTextureEnabledElement::set(state, this, FALSE);
   SoGLTexture3EnabledElement::set(state, this, FALSE);
-
+  
   SoVRMLVertexLine::GLRender(action);
 
-  if (!this->shouldGLRender(action)) 
+  if (!this->shouldGLRender(action)) {
+    state->pop();
     return;
+  }
 
   SoVRMLIndexedLineSetP * pimpl = PRIVATE(this);
   
@@ -414,6 +417,7 @@ SoVRMLIndexedLineSet::GLRender(SoGLRenderAction * action)
   }
   // send approx number of triangles for autocache handling
   sogl_autocache_update(state, this->coordIndex.getNum() / 2);
+  state->pop();
 }
 
 void
