@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -25,12 +25,14 @@
 
 #ifdef COIN_HAVE_JAVASCRIPT
 
+#include <Inventor/actions/SoWriteAction.h>
 #include <Inventor/misc/SoJavaScriptEngine.h>
 #include <Inventor/misc/SbHash.h>
 #include <Inventor/C/glue/spidermonkey.h>
 
 #include <Inventor/fields/SoSFBool.h>
 #include <Inventor/fields/SoSFInt32.h>
+#include <Inventor/fields/SoSFEnum.h>
 #include <Inventor/fields/SoSFFloat.h>
 #include <Inventor/fields/SoSFString.h>
 #include <Inventor/fields/SoSFTime.h>
@@ -39,6 +41,7 @@
 #include <Inventor/fields/SoSFImage.h>
 #include <Inventor/fields/SoSFVec2f.h>
 #include <Inventor/fields/SoSFVec3f.h>
+#include <Inventor/fields/SoSFVec3d.h>
 #include <Inventor/fields/SoSFRotation.h>
 
 #include <Inventor/fields/SoMFInt32.h>
@@ -49,10 +52,12 @@
 #include <Inventor/fields/SoMFNode.h>
 #include <Inventor/fields/SoMFVec2f.h>
 #include <Inventor/fields/SoMFVec3f.h>
+#include <Inventor/fields/SoMFVec3d.h>
 #include <Inventor/fields/SoMFRotation.h>
 
 #include <Inventor/SoDB.h>
 #include <Inventor/SoInput.h>
+#include <Inventor/SoOutput.h> 
 #include <Inventor/VRMLnodes/SoVRMLGroup.h>
 #include <Inventor/errors/SoDebugError.h>
 #include <Inventor/sensors/SoNodeSensor.h>
@@ -72,6 +77,7 @@ struct CoinVrmlJs {
   static ClassDescriptor SFRotation;
   static ClassDescriptor SFVec2f;
   static ClassDescriptor SFVec3f;
+  static ClassDescriptor SFVec3d;
 
   static ClassDescriptor MFColor;
   static ClassDescriptor MFFloat;
@@ -82,6 +88,7 @@ struct CoinVrmlJs {
   static ClassDescriptor MFTime;
   static ClassDescriptor MFVec2f;
   static ClassDescriptor MFVec3f;
+  static ClassDescriptor MFVec3d;
 };
 
 // Struct and SbHash for keeping track of SoNodeSensors for recycling
@@ -95,19 +102,22 @@ SbHash <void *, unsigned long> * CoinVrmlJs_sensorinfohash = NULL;
 char * CoinVrmlJs_SFColorAliases[] = {"r", "g", "b"};
 char * CoinVrmlJs_SFRotationAliases[] = {"x", "y", "z", "angle"};
 float CoinVrmlJs_SFdefaultValues[] = {0.0, 0.0, 0.0, 0.0};
+double CoinVrmlJs_SFdefaultValuesDouble[] = {0.0, 0.0, 0.0, 0.0};
 float CoinVrmlJs_SFRotationDefaultValues[] = {0.0, 1.0, 0.0, 0.0};
 
 // Macros for instance checking
 #define JSVAL_IS_SFVEC2F(cx, jsval) (JSVAL_IS_OBJECT(jsval) && spidermonkey()->JS_InstanceOf(cx, JSVAL_TO_OBJECT(jsval), &CoinVrmlJs::SFVec2f.cls, NULL))
 #define JSVAL_IS_SFVEC3F(cx, jsval) (JSVAL_IS_OBJECT(jsval) && spidermonkey()->JS_InstanceOf(cx, JSVAL_TO_OBJECT(jsval), &CoinVrmlJs::SFVec3f.cls, NULL))
+#define JSVAL_IS_SFVEC3D(cx, jsval) (JSVAL_IS_OBJECT(jsval) && spidermonkey()->JS_InstanceOf(cx, JSVAL_TO_OBJECT(jsval), &CoinVrmlJs::SFVec3d.cls, NULL))
 #define JSVAL_IS_SFCOLOR(cx, jsval) (JSVAL_IS_OBJECT(jsval) && spidermonkey()->JS_InstanceOf(cx, JSVAL_TO_OBJECT(jsval), &CoinVrmlJs::SFColor.cls, NULL))
 #define JSVAL_IS_SFROTATION(cx, jsval) (JSVAL_IS_OBJECT(jsval) && spidermonkey()->JS_InstanceOf(cx, JSVAL_TO_OBJECT(jsval), &CoinVrmlJs::SFRotation.cls, NULL))
 
 // Handlers
-#define SFColorHandler CoinVrmlJsSFHandler<SbColor, 3, CoinVrmlJs_SFColorAliases, CoinVrmlJs_SFdefaultValues>
-#define SFRotationHandler CoinVrmlJsSFHandler<SbVec4f, 4, CoinVrmlJs_SFRotationAliases, CoinVrmlJs_SFRotationDefaultValues>
-#define SFVec2fHandler CoinVrmlJsSFHandler<SbVec2f, 2, CoinVrmlJs_SFRotationAliases, CoinVrmlJs_SFdefaultValues>
-#define SFVec3fHandler CoinVrmlJsSFHandler<SbVec3f, 3, CoinVrmlJs_SFRotationAliases, CoinVrmlJs_SFdefaultValues>
+#define SFColorHandler CoinVrmlJsSFHandler<SbColor, 3, CoinVrmlJs_SFColorAliases, float, CoinVrmlJs_SFdefaultValues>
+#define SFRotationHandler CoinVrmlJsSFHandler<SbVec4f, 4, CoinVrmlJs_SFRotationAliases, float, CoinVrmlJs_SFRotationDefaultValues>
+#define SFVec2fHandler CoinVrmlJsSFHandler<SbVec2f, 2, CoinVrmlJs_SFRotationAliases, float, CoinVrmlJs_SFdefaultValues>
+#define SFVec3fHandler CoinVrmlJsSFHandler<SbVec3f, 3, CoinVrmlJs_SFRotationAliases, float, CoinVrmlJs_SFdefaultValues>
+#define SFVec3dHandler CoinVrmlJsSFHandler<SbVec3d, 3, CoinVrmlJs_SFRotationAliases, double, CoinVrmlJs_SFdefaultValuesDouble>
 
 #define MFColorHandler CoinVrmlJsMFHandler<SoMFColor, SoSFColor, &CoinVrmlJs::MFColor>
 #define MFFloatHandler CoinVrmlJsMFHandler<SoMFFloat, SoSFFloat, &CoinVrmlJs::MFFloat>
@@ -118,6 +128,7 @@ float CoinVrmlJs_SFRotationDefaultValues[] = {0.0, 1.0, 0.0, 0.0};
 #define MFTimeHandler CoinVrmlJsMFHandler<SoMFTime, SoSFTime, &CoinVrmlJs::MFTime>
 #define MFVec2fHandler CoinVrmlJsMFHandler<SoMFVec2f, SoSFVec2f, &CoinVrmlJs::MFVec2f>
 #define MFVec3fHandler CoinVrmlJsMFHandler<SoMFVec3f, SoSFVec3f, &CoinVrmlJs::MFVec3f>
+#define MFVec3dHandler CoinVrmlJsMFHandler<SoMFVec3d, SoSFVec3d, &CoinVrmlJs::MFVec3d>
 
 static JSFunctionSpec MFFunctions[] = {
 //  {"toString", MF_toString, 0, 0, 0},
@@ -133,6 +144,7 @@ static JSObject * SFNodeFactory(JSContext * cx, SoNode * container);
 static JSObject * SFRotationFactory(JSContext * cx, const SbRotation & self);
 static JSObject * SFVec2fFactory(JSContext * cx, const SbVec2f & self);
 static JSObject * SFVec3fFactory(JSContext * cx, const SbVec3f & self);
+static JSObject * SFVec3dFactory(JSContext * cx, const SbVec3d & self);
 
 static SbList <JSObject *> * garbagecollectedobjects = NULL;
 static SbList <SoNodeSensor *> * nodesensorstobedeleted = NULL;
@@ -163,23 +175,45 @@ static JSBool getIndex(JSContext * cx, jsval id, char * aliases[], int max)
   }
 }
 
+bool jsval2int(JSContext *cx, const jsval v, int32_t &value) 
+{
+  if (JSVAL_IS_NULL(v)) return false;
+  int32_t tempval;
+  if (!spidermonkey()->JS_ValueToECMAInt32(cx, v, &tempval)) {
+    return false;
+  }
+  value = tempval;
+  return true;
+}
+
+bool jsval2double(JSContext *cx, const jsval v, double &value) 
+{
+  if (JSVAL_IS_NULL(v)) return false;
+  double tempval;
+  if (!spidermonkey()->JS_ValueToNumber(cx, v, &tempval)) {
+    return false;
+  }
+  value = tempval;
+  return true;
+}
+
 // *************************************************************************
 // handlers
 
 // FIXME: number of aliases must not be lower than max. This may lead to
 // unsafe programming. 20050721 erikgors.
-template <class Base, int max, char * aliases[], float defaultValues[]>
+template <class Base, int max, char * aliases[], class basetype, basetype defaultValues[]>
 struct CoinVrmlJsSFHandler {
   static JSBool get(JSContext * cx, JSObject * obj, jsval id, jsval * rval)
   {
     int index = getIndex(cx, id, aliases, max);
     if (index == -1) {
-      return JS_FALSE;
+      return JS_TRUE;
     }
 
     Base * data = (Base *)spidermonkey()->JS_GetPrivate(cx, obj);
     assert(data != NULL);
-    float var = (*data)[index];
+    basetype var = (*data)[index];
     SbBool ok = spidermonkey()->JS_NewDoubleValue(cx, (double)var, rval);
     assert(ok && "JS_NewDoubleValue failed");
     return JS_TRUE;
@@ -192,7 +226,7 @@ struct CoinVrmlJsSFHandler {
       return JS_FALSE;
     }
 
-    SbColor * data = (SbColor *)spidermonkey()->JS_GetPrivate(cx, obj);
+    Base * data = (Base *)spidermonkey()->JS_GetPrivate(cx, obj);
     assert(data != NULL);
 
     // FIXME: number may be NaN, PositiveInfinity and NegativeInfinity.
@@ -200,14 +234,14 @@ struct CoinVrmlJsSFHandler {
     // ie: "blipp" will become NaN. 20050720 erikgors.
     double number;
     spidermonkey()->JS_ValueToNumber(cx, *val, &number);
-    (*data)[index] = (float)number;
+    (*data)[index] = (basetype)number;
     return JS_TRUE;
   }
 
   static JSBool constructor(JSContext * cx, JSObject * obj,
                             uintN argc, jsval * argv, jsval * rval)
   {
-    float vals[max];
+    basetype vals[max];
 
     // convert all arguments to numbers or use defaultValues if missing
     uint32_t i;
@@ -216,7 +250,7 @@ struct CoinVrmlJsSFHandler {
       if (i<argc) {
         double val;
         if (spidermonkey()->JS_ValueToNumber(cx, argv[i], &val)) {
-          vals[i] = (float)val;
+          vals[i] = (basetype)val;
         }
         else {
           spidermonkey()->JS_ReportError(cx, "WARNING: failed converting argument %d "
@@ -341,6 +375,13 @@ struct CoinVrmlJsMFHandler {
           SFVec3fHandler::constructor(cx, newObj, 0, NULL, &val);
           val = OBJECT_TO_JSVAL(newObj);
         }
+        else if (type == SoMFVec3d::getClassTypeId()) {
+          JSObject * newObj =
+            spidermonkey()->JS_NewObject(cx, &CoinVrmlJs::SFVec3d.cls, NULL, NULL);
+          assert(newObj != NULL);
+          SFVec3dHandler::constructor(cx, newObj, 0, NULL, &val);
+          val = OBJECT_TO_JSVAL(newObj);
+        }
         else {
           assert(0 && "this should not happen");
         }
@@ -574,6 +615,47 @@ static JSBool SFNode_unref(JSContext * cx, JSObject * obj, uintN argc,
   return JSVAL_TRUE;
 }
 
+static void *
+buffer_realloc(void * bufptr, size_t size)
+{
+  char *buffer = (char *)realloc(bufptr, size);
+  return buffer;
+}
+
+static JSBool SFNode_toString(JSContext * cx, JSObject * obj, uintN argc,
+                              jsval * argv, jsval * rval)
+{
+  // Check if the JS object has already been garbage collected. This
+  // must be done to prevent a Java script from crashing the
+  // application.
+  if (garbagecollectedobjects->find(obj) != -1) {
+    if (SoJavaScriptEngine::debug())
+      SoDebugError::postInfo("SFNode_toString", "WARNING! Trying to access "
+                             "an already deleted node.");
+    return JSVAL_FALSE;
+  }
+
+  SoNode *node = (SoNode *)spidermonkey()->JS_GetPrivate(cx, obj);
+
+  SoOutput out;
+  out.setHeaderString("#VRML V2.0 utf8");
+  size_t buffer_size = 1024;
+  void *buffer = (void *)malloc(buffer_size);
+  out.setBuffer(buffer, buffer_size, buffer_realloc);
+  
+  SoWriteAction wa(&out);
+  wa.apply(node);
+  
+  out.getBuffer(buffer, buffer_size);
+
+  *rval = STRING_TO_JSVAL(spidermonkey()->JS_NewStringCopyZ(cx, 
+    (char *)buffer));
+
+  free(buffer);
+
+  return JSVAL_TRUE;
+}
+
 static void SFNode_deleteCB(void * data, SoSensor * sensor)
 {
   SoNode * node = ((SoNodeSensor *) sensor)->getAttachedNode();
@@ -638,15 +720,28 @@ static JSBool SFVec3f_add(JSContext * cx, JSObject * obj, uintN argc,
   return JS_FALSE;
 }
 
+static JSBool SFVec3d_add(JSContext * cx, JSObject * obj, uintN argc,
+                          jsval * argv, jsval * rval)
+{
+  SbVec3d & vec1 = *(SbVec3d *)spidermonkey()->JS_GetPrivate(cx, obj);
+
+  if (argc >= 1 && JSVAL_IS_SFVEC3D(cx, argv[0])) {
+    SbVec3d & vec2 = *(SbVec3d *)spidermonkey()->JS_GetPrivate(cx, JSVAL_TO_OBJECT(argv[0]));
+    SbVec3d result = vec1 + vec2;
+    *rval = OBJECT_TO_JSVAL(SFVec3dFactory(cx, result));
+    return JS_TRUE;
+  }
+
+  return JS_FALSE;
+}
+
 static JSBool SFVec2f_divide(JSContext * cx, JSObject * obj, uintN argc,
                              jsval * argv, jsval * rval)
 {
   SbVec2f & vec = *(SbVec2f *)spidermonkey()->JS_GetPrivate(cx, obj);
 
-  if (argc >= 1 && JSVAL_IS_NUMBER(argv[0])) {
-    double number;
-    spidermonkey()->JS_ValueToNumber(cx, argv[0], &number);
-
+  double number;
+  if (argc >= 1 && jsval2double(cx, argv[0], number)) {
     SbVec2f newVec = vec / (float)number;
     *rval = OBJECT_TO_JSVAL(SFVec2fFactory(cx, newVec));
     return JS_TRUE;
@@ -659,10 +754,8 @@ static JSBool SFVec3f_divide(JSContext * cx, JSObject * obj, uintN argc,
 {
   SbVec3f & vec = *(SbVec3f *)spidermonkey()->JS_GetPrivate(cx, obj);
 
-  if (argc >= 1 && JSVAL_IS_NUMBER(argv[0])) {
-    double number;
-    spidermonkey()->JS_ValueToNumber(cx, argv[0], &number);
-
+  double number;
+  if (argc >= 1 && jsval2double(cx, argv[0], number)) {
     SbVec3f newVec = vec / (float)number;
     *rval = OBJECT_TO_JSVAL(SFVec3fFactory(cx, newVec));
     return JS_TRUE;
@@ -670,6 +763,20 @@ static JSBool SFVec3f_divide(JSContext * cx, JSObject * obj, uintN argc,
   return JS_FALSE;
 }
 
+static JSBool SFVec3d_divide(JSContext * cx, JSObject * obj, uintN argc,
+                             jsval * argv, jsval * rval)
+{
+  SbVec3d & vec = *(SbVec3d *)spidermonkey()->JS_GetPrivate(cx, obj);
+
+  double number;
+  if (argc >= 1 && jsval2double(cx, argv[0], number)) {
+    SbVec3d newVec = vec / number;
+    *rval = OBJECT_TO_JSVAL(SFVec3dFactory(cx, newVec));
+    return JS_TRUE;
+  }
+  return JS_FALSE;
+}
+
 static JSBool SFVec2f_dot(JSContext *cx, JSObject *obj, uintN argc,
                           jsval *argv, jsval *rval)
 {
@@ -701,6 +808,22 @@ static JSBool SFVec3f_dot(JSContext *cx, JSObject *obj, uintN argc,
   return JS_FALSE;
 }
 
+static JSBool SFVec3d_dot(JSContext *cx, JSObject *obj, uintN argc,
+                          jsval *argv, jsval *rval)
+{
+  SbVec3d & vec = *(SbVec3d *)spidermonkey()->JS_GetPrivate(cx, obj);
+
+  if (argc >= 1 && JSVAL_IS_SFVEC3D(cx, argv[0])) {
+
+    SbVec3d & vec2 = *(SbVec3d *)spidermonkey()->JS_GetPrivate(cx, JSVAL_TO_OBJECT(argv[0]));
+
+    double dot = vec.dot(vec2);
+    JSBool ok = spidermonkey()->JS_NewDoubleValue(cx, dot, rval);
+    return JS_TRUE;
+  }
+  return JS_FALSE;
+}
+
 static JSBool SFVec2_length(JSContext * cx, JSObject * obj, uintN argc,
                             jsval * argv, jsval * rval)
 {
@@ -709,7 +832,7 @@ static JSBool SFVec2_length(JSContext * cx, JSObject * obj, uintN argc,
   return JS_TRUE;
 }
 
-static JSBool SFVec3_length(JSContext * cx, JSObject * obj, uintN argc,
+static JSBool SFVec3f_length(JSContext * cx, JSObject * obj, uintN argc,
                             jsval * argv, jsval * rval)
 {
   SbVec3f * vec = (SbVec3f *)spidermonkey()->JS_GetPrivate(cx, obj);
@@ -717,6 +840,14 @@ static JSBool SFVec3_length(JSContext * cx, JSObject * obj, uintN argc,
   return JS_TRUE;
 }
 
+static JSBool SFVec3d_length(JSContext * cx, JSObject * obj, uintN argc,
+                            jsval * argv, jsval * rval)
+{
+  SbVec3d * vec = (SbVec3d *)spidermonkey()->JS_GetPrivate(cx, obj);
+  JSBool ok = spidermonkey()->JS_NewDoubleValue(cx, vec->length(), rval);
+  return JS_TRUE;
+}
+
 static JSBool SFVec2f_multiply(JSContext * cx, JSObject * obj, uintN argc,
                                jsval * argv, jsval * rval)
 {
@@ -741,10 +872,8 @@ static JSBool SFVec3f_multiply(JSContext * cx, JSObject * obj, uintN argc,
 {
   SbVec3f & vec = *(SbVec3f *)spidermonkey()->JS_GetPrivate(cx, obj);
 
-  if (argc >= 1 && JSVAL_IS_NUMBER(argv[0])) {
-    double number;
-    spidermonkey()->JS_ValueToNumber(cx, argv[0], &number);
-
+  double number;
+  if (argc >= 1 && jsval2double(cx, argv[0], number)) {
     SbVec3f newVec = vec * (float)number;
     *rval = OBJECT_TO_JSVAL(SFVec3fFactory(cx, newVec));
     return JS_TRUE;
@@ -752,6 +881,20 @@ static JSBool SFVec3f_multiply(JSContext * cx, JSObject * obj, uintN argc,
   return JS_FALSE;
 }
 
+static JSBool SFVec3d_multiply(JSContext * cx, JSObject * obj, uintN argc,
+                               jsval * argv, jsval * rval)
+{
+  SbVec3d & vec = *(SbVec3d *)spidermonkey()->JS_GetPrivate(cx, obj);
+
+  double number;
+  if (argc >= 1 && jsval2double(cx, argv[0], number)) {
+    SbVec3d newVec = vec * number;
+    *rval = OBJECT_TO_JSVAL(SFVec3dFactory(cx, newVec));
+    return JS_TRUE;
+  }
+  return JS_FALSE;
+}
+
 static JSBool SFVec2f_normalize(JSContext * cx, JSObject * obj, uintN argc,
                                 jsval * argv, jsval * rval)
 {
@@ -770,6 +913,15 @@ static JSBool SFVec3f_normalize(JSContext * cx, JSObject * obj, uintN argc,
   return JS_TRUE;
 }
 
+static JSBool SFVec3d_normalize(JSContext * cx, JSObject * obj, uintN argc,
+                                jsval * argv, jsval * rval)
+{
+  SbVec3d vec = *(SbVec3d *)spidermonkey()->JS_GetPrivate(cx, obj);
+  vec.normalize();
+  *rval = OBJECT_TO_JSVAL(SFVec3dFactory(cx, vec));
+  return JS_TRUE;
+}
+
 static JSBool SFVec3f_negate(JSContext * cx, JSObject * obj, uintN argc,
                                      jsval * argv, jsval * rval)
 {
@@ -779,6 +931,15 @@ static JSBool SFVec3f_negate(JSContext * cx, JSObject * obj, uintN argc,
   return JS_TRUE;
 }
 
+static JSBool SFVec3d_negate(JSContext * cx, JSObject * obj, uintN argc,
+                                     jsval * argv, jsval * rval)
+{
+  SbVec3d vec = *(SbVec3d *)spidermonkey()->JS_GetPrivate(cx, obj);
+  vec.negate();
+  *rval = OBJECT_TO_JSVAL(SFVec3dFactory(cx, vec));
+  return JS_TRUE;
+}
+
 static JSBool SFVec2f_substract(JSContext * cx, JSObject * obj, uintN argc,
                                 jsval * argv, jsval * rval)
 {
@@ -808,6 +969,21 @@ static JSBool SFVec3f_substract(JSContext * cx, JSObject * obj, uintN argc,
   return JS_FALSE;
 }
 
+static JSBool SFVec3d_substract(JSContext * cx, JSObject * obj, uintN argc,
+                                jsval * argv, jsval * rval)
+{
+  SbVec3d & vec1 = *(SbVec3d *)spidermonkey()->JS_GetPrivate(cx, obj);
+
+  if (argc >= 1 && JSVAL_IS_SFVEC3D(cx, argv[0])) {
+    SbVec3d & vec2 = *(SbVec3d *)spidermonkey()->JS_GetPrivate(cx, JSVAL_TO_OBJECT(argv[0]));
+    SbVec3d result = vec1 - vec2;
+    *rval = OBJECT_TO_JSVAL(SFVec3dFactory(cx, result));
+    return JS_TRUE;
+  }
+
+  return JS_FALSE;
+}
+
 static JSBool SFColor_setHSV(JSContext * cx, JSObject * obj, uintN argc,
                              jsval * argv, jsval * rval)
 {
@@ -930,14 +1106,12 @@ static JSBool SFRotation_slerp(JSContext * cx, JSObject * obj, uintN argc,
   SbVec3f axis(vec[0], vec[1], vec[2]);
   SbRotation rot(axis, vec[3]);
 
-  if (argc >= 2 && JSVAL_IS_SFROTATION(cx, argv[0]) && JSVAL_IS_NUMBER(argv[1])) {
+  double number;
+  if (argc >= 2 && JSVAL_IS_SFROTATION(cx, argv[0]) && jsval2double(cx, argv[1], number)) {
     SbVec4f & vec2 = *(SbVec4f *)spidermonkey()->JS_GetPrivate(cx, JSVAL_TO_OBJECT(argv[0]));
     SbVec3f axis2(vec2[0], vec2[1], vec2[2]);
     SbRotation dest(axis2, vec2[3]);
 
-    double number;
-    spidermonkey()->JS_ValueToNumber(cx, argv[1], &number);
-
     SbRotation result = SbRotation::slerp(rot, dest, (float)number);
 
     *rval = OBJECT_TO_JSVAL(SFRotationFactory(cx, result));
@@ -949,6 +1123,7 @@ static JSBool SFRotation_slerp(JSContext * cx, JSObject * obj, uintN argc,
 static JSFunctionSpec SFNodeFunctions[] = {
   {"ref", SFNode_ref, 0, 0, 0},
   {"unref", SFNode_unref, 0, 0, 0},
+  {"toString", SFNode_toString, 0, 0, 0},
   {NULL, NULL, 0, 0, 0}
 };
 
@@ -967,7 +1142,7 @@ static JSFunctionSpec SFVec3fFunctions[] = {
   {"add", SFVec3f_add, 1, 0, 0},
   {"divide", SFVec3f_divide, 1, 0, 0},
   {"dot", SFVec3f_dot, 1, 0, 0},
-  {"length", SFVec3_length, 0, 0, 0},
+  {"length", SFVec3f_length, 0, 0, 0},
   {"multiply", SFVec3f_multiply, 1, 0, 0},
   {"normalize", SFVec3f_normalize, 0, 0, 0},
   {"negate", SFVec3f_negate, 0, 0, 0},
@@ -975,6 +1150,18 @@ static JSFunctionSpec SFVec3fFunctions[] = {
   {NULL, NULL, 0, 0, 0}
 };
 
+static JSFunctionSpec SFVec3dFunctions[] = {
+  {"add", SFVec3d_add, 1, 0, 0},
+  {"divide", SFVec3d_divide, 1, 0, 0},
+  {"dot", SFVec3d_dot, 1, 0, 0},
+  {"length", SFVec3d_length, 0, 0, 0},
+  {"multiply", SFVec3d_multiply, 1, 0, 0},
+  {"normalize", SFVec3d_normalize, 0, 0, 0},
+  {"negate", SFVec3d_negate, 0, 0, 0},
+  {"substract", SFVec3d_substract, 1, 0, 0},
+  {NULL, NULL, 0, 0, 0}
+};
+
 static JSFunctionSpec SFColorFunctions[] = {
   {"setHSV", SFColor_setHSV, 3, 0, 0},
   {"getHSV", SFColor_getHSV, 0, 0, 0},
@@ -1044,6 +1231,18 @@ static JSObject * SFVec3fFactory(JSContext * cx, const SbVec3f & self)
   return obj;
 }
 
+static JSObject * SFVec3dFactory(JSContext * cx, const SbVec3d & self)
+{
+  JSObject * obj =
+    spidermonkey()->JS_NewObject(cx, &CoinVrmlJs::SFVec3d.cls, NULL, NULL);
+  spidermonkey()->JS_DefineFunctions(cx, obj, SFVec3dFunctions);
+
+  SbVec3d * data = new SbVec3d(self);
+  spidermonkey()->JS_SetPrivate(cx, obj, data);
+
+  return obj;
+}
+
 static JSObject * SFVec2f_init(JSContext * cx, JSObject * obj)
 {
   return spidermonkey()->JS_InitClass(cx, obj, NULL, &CoinVrmlJs::SFVec2f.cls,
@@ -1054,10 +1253,17 @@ static JSObject * SFVec2f_init(JSContext * cx, JSObject * obj)
 static JSObject * SFVec3f_init(JSContext * cx, JSObject * obj)
 {
   return spidermonkey()->JS_InitClass(cx, obj, NULL, &CoinVrmlJs::SFVec3f.cls,
-                                      SFVec2fHandler::constructor, 0,
+                                      SFVec3fHandler::constructor, 0,
                                       NULL, SFVec3fFunctions, NULL, NULL);
 }
 
+static JSObject * SFVec3d_init(JSContext * cx, JSObject * obj)
+{
+  return spidermonkey()->JS_InitClass(cx, obj, NULL, &CoinVrmlJs::SFVec3d.cls,
+                                      SFVec3dHandler::constructor, 0,
+                                      NULL, SFVec3dFunctions, NULL, NULL);
+}
+
 static JSObject * SFColor_init(JSContext * cx, JSObject * obj)
 {
   return spidermonkey()->JS_InitClass(cx, obj, NULL, &CoinVrmlJs::SFColor.cls,
@@ -1129,6 +1335,14 @@ static JSBool SFNode_get(JSContext * cx, JSObject * obj, jsval id, jsval * rval)
      dynamically add and remove properties for any object at any time.
 
      2005-11-23 thammer.
+
+     Update: We should look more closely into the return values 
+     JS_TRUE / JS_FALSE for all getters and setters, and possibly for
+     other functions as well. It might be more robust to report the
+     error (using JS_ReportError) and return JS_TRUE to allow the
+     script to continue running than to abort the script.
+
+     2006-06-21 thammer.
   */
   return JS_TRUE;
 }
@@ -1161,7 +1375,8 @@ static JSBool SFNode_set(JSContext * cx, JSObject * obj, jsval id, jsval * rval)
       if (SoJavaScriptEngine::debug()) {
         SoDebugError::postInfo("SFNode_set", "setting field %s", str.getString());
       }
-    }
+    } else if (SoJavaScriptEngine::debug())
+      SoDebugError::postWarning("SFNode_set", "no such field %s", str.getString());
   }
 
   // See note in SFNode_get() about return value. 2005-11-23 thammer.
@@ -1232,6 +1447,8 @@ static JSObject * SFNodeFactory(JSContext * cx, SoNode * container)
   spidermonkey()->JS_SetPrivate(cx, obj, container);
   spidermonkey()->JS_DefineFunctions(cx, obj, SFNodeFunctions);
 
+  // FIXME: If the node has enums, define them here. 2007-03-08 thammer.
+
   attachSensorToNode(container, obj);
 
   if (SoJavaScriptEngine::getEngine(cx)->getAutoNodeUnrefState())
@@ -1301,7 +1518,6 @@ static JSObject * SFNode_init(JSContext * cx, JSObject * obj)
                                       NULL, NULL, NULL, NULL);
 }
 
-
 // *************************************************************************
 // jsval2field
 
@@ -1335,9 +1551,8 @@ static SbBool SFColor_jsval2field(JSContext * cx, const jsval v, SoField * f)
 
 static SbBool SFFloat_jsval2field(JSContext * cx, const jsval v, SoField * f)
 {
-  if (JSVAL_IS_NUMBER(v)) {
-    double number;
-    spidermonkey()->JS_ValueToNumber(cx, v, &number);
+  double number;
+  if (jsval2double(cx, v, number)) {
     ((SoSFFloat *)f)->setValue((float)number);
     return TRUE;
   }
@@ -1346,8 +1561,18 @@ static SbBool SFFloat_jsval2field(JSContext * cx, const jsval v, SoField * f)
 
 static SbBool SFInt32_jsval2field(JSContext * cx, const jsval v, SoField * f)
 {
-  if (JSVAL_IS_INT(v)) {
-    const int32_t val = JSVAL_TO_INT(v);
+  int32_t val;
+  if (jsval2int(cx, v, val)) {
+    ((SoSFInt32 *)f)->setValue(val);
+    return TRUE;
+  }
+  return FALSE;
+}
+
+static SbBool SFEnum_jsval2field(JSContext * cx, const jsval v, SoField * f)
+{
+  int32_t val;
+  if (jsval2int(cx, v, val)) {
     ((SoSFInt32 *)f)->setValue(val);
     return TRUE;
   }
@@ -1356,6 +1581,10 @@ static SbBool SFInt32_jsval2field(JSContext * cx, const jsval v, SoField * f)
 
 static SbBool SFNode_jsval2field(JSContext * cx, const jsval v, SoField * f)
 {
+  if (JSVAL_IS_NULL(v)) {
+    ((SoSFNode *)f)->setValue(NULL);
+    return TRUE;
+  }
   if (JSVAL_IS_OBJECT(v) &&
       spidermonkey()->JS_InstanceOf(cx, JSVAL_TO_OBJECT(v), &CoinVrmlJs::SFNode.cls, NULL)) {
     SoNode * node = (SoNode *)spidermonkey()->JS_GetPrivate(cx, JSVAL_TO_OBJECT(v));
@@ -1389,8 +1618,8 @@ static SbBool SFString_jsval2field(JSContext * cx, const jsval v, SoField * f)
 
 static SbBool SFTime_jsval2field(JSContext * cx, const jsval v, SoField * f)
 {
-  if (JSVAL_IS_NUMBER(v)) {
-    double number;
+  double number;
+  if (jsval2double(cx, v, number)) {
     spidermonkey()->JS_ValueToNumber(cx, v, &number);
     ((SoSFTime*)f)->setValue(SbTime(number));
     return TRUE;
@@ -1420,6 +1649,17 @@ static SbBool SFVec3f_jsval2field(JSContext * cx, const jsval v, SoField * f)
   return FALSE;
 }
 
+static SbBool SFVec3d_jsval2field(JSContext * cx, const jsval v, SoField * f)
+{
+  if (JSVAL_IS_SFVEC3D(cx, v)) {
+    SbVec3d * vec = (SbVec3d *)spidermonkey()->JS_GetPrivate(cx, JSVAL_TO_OBJECT(v));
+    assert(vec != NULL);
+    ((SoSFVec3d *)f)->setValue(*vec);
+    return TRUE;
+  }
+  return FALSE;
+}
+
 // *************************************************************************
 // field2jsval
 
@@ -1448,11 +1688,21 @@ static void SFInt32_field2jsval(JSContext * cx, const SoField * f, jsval * v)
   *v = INT_TO_JSVAL(val);
 }
 
+static void SFEnum_field2jsval(JSContext * cx, const SoField * f, jsval * v)
+{
+  const int32_t val = ((SoSFInt32 *)f)->getValue();
+  *v = INT_TO_JSVAL(val);
+}
+
 static void SFNode_field2jsval(JSContext * cx, const SoField * f, jsval * v)
 {
   SoNode * node = ((SoSFNode *)f)->getValue();
-  JSObject * obj = SFNodeFactory(cx, node);
-  *v = OBJECT_TO_JSVAL(obj);
+  if (node == NULL)
+    *v = JSVAL_NULL;
+  else {
+    JSObject * obj = SFNodeFactory(cx, node);
+    *v = OBJECT_TO_JSVAL(obj);
+  }
 }
 
 static void SFRotation_field2jsval(JSContext * cx, const SoField * f, jsval *v)
@@ -1489,6 +1739,13 @@ static void SFVec3f_field2jsval(JSContext * cx, const SoField * f, jsval *v)
   *v = OBJECT_TO_JSVAL(obj);
 }
 
+static void SFVec3d_field2jsval(JSContext * cx, const SoField * f, jsval *v)
+{
+  const SbVec3d & val = ((SoSFVec3d *)f)->getValue();
+  JSObject * obj = SFVec3dFactory(cx, val);
+  *v = OBJECT_TO_JSVAL(obj);
+}
+
 // *************************************************************************
 // classes
 
@@ -1548,6 +1805,17 @@ CoinVrmlJs::ClassDescriptor CoinVrmlJs::SFVec3f = {
   SFVec3fFunctions
 };
 
+CoinVrmlJs::ClassDescriptor CoinVrmlJs::SFVec3d = {
+  {
+    "SFVec3d", JSCLASS_HAS_PRIVATE, NULL, NULL,
+    SFVec3dHandler::get, SFVec3dHandler::set,
+    NULL, NULL, NULL,
+    SFVec3dHandler::destructor,
+    NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0
+  },
+  SFVec3dFunctions
+};
+
 CoinVrmlJs::ClassDescriptor CoinVrmlJs::MFColor = {
   {
     "MFColor", JSCLASS_HAS_PRIVATE, NULL, NULL,
@@ -1647,13 +1915,25 @@ CoinVrmlJs::ClassDescriptor CoinVrmlJs::MFVec3f = {
   MFFunctions,
 };
 
+CoinVrmlJs::ClassDescriptor CoinVrmlJs::MFVec3d = {
+  {
+    "MFVec3d", JSCLASS_HAS_PRIVATE, NULL, NULL,
+    MFVec3dHandler::get, MFVec3dHandler::set,
+    NULL, NULL, NULL,
+    MFVec3dHandler::destructor,
+    NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0
+  },
+  MFFunctions,
+};
+
 
 CoinVrmlJs::ClassDescriptor * CLASSDESCRIPTORS[] = {
   &CoinVrmlJs::SFColor, &CoinVrmlJs::SFNode, &CoinVrmlJs::SFRotation,
-  &CoinVrmlJs::SFVec2f, &CoinVrmlJs::SFVec3f, &CoinVrmlJs::MFColor,
+  &CoinVrmlJs::SFVec2f, &CoinVrmlJs::SFVec3f, &CoinVrmlJs::SFVec3d, 
+  &CoinVrmlJs::MFColor, 
   &CoinVrmlJs::MFFloat, &CoinVrmlJs::MFInt32, &CoinVrmlJs::MFNode,
   &CoinVrmlJs::MFRotation, &CoinVrmlJs::MFString, &CoinVrmlJs::MFTime,
-  &CoinVrmlJs::MFVec2f, &CoinVrmlJs::MFVec3f
+  &CoinVrmlJs::MFVec2f, &CoinVrmlJs::MFVec3f, &CoinVrmlJs::MFVec3d
 };
 
 // *************************************************************************
@@ -1730,6 +2010,11 @@ JS_addVRMLclasses(SoJavaScriptEngine * engine)
     MFInt32Handler::field2jsval,
     MFInt32Handler::jsval2field);
 
+  // Enum 
+  engine->addHandler(
+    SoSFEnum::getClassTypeId(), NULL,
+    SFEnum_field2jsval, SFEnum_jsval2field);
+
   // Node
   engine->addHandler(
     SoSFNode::getClassTypeId(), SFNode_init,
@@ -1789,6 +2074,16 @@ JS_addVRMLclasses(SoJavaScriptEngine * engine)
     MFVec3fHandler::init,
     MFVec3fHandler::field2jsval,
     MFVec3fHandler::jsval2field);
+
+  // Vec3d
+  engine->addHandler(
+    SoSFVec3d::getClassTypeId(), SFVec3d_init,
+    SFVec3d_field2jsval, SFVec3d_jsval2field);
+  engine->addHandler(
+    SoMFVec3d::getClassTypeId(),
+    MFVec3dHandler::init,
+    MFVec3dHandler::field2jsval,
+    MFVec3dHandler::jsval2field);
 }
 
 #else // !COIN_HAVE_JAVASCRIPT
