@@ -1,7 +1,7 @@
 /**************************************************************************\
  *
  *  This file is part of the Coin 3D visualization library.
- *  Copyright (C) 1998-2006 by Systems in Motion.  All rights reserved.
+ *  Copyright (C) 1998-2007 by Systems in Motion.  All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -1593,11 +1593,11 @@ void SoVRMLSoundP::fillBuffers()
         else {
           char statestr[20];
           switch (state) {
-          case AL_INITIAL : sprintf(statestr, "initial"); break;
-          case AL_PLAYING : sprintf(statestr, "playing"); break;
-          case AL_PAUSED : sprintf(statestr, "paused"); break;
-          case AL_STOPPED : sprintf(statestr, "stopped"); break;
-          default : sprintf(statestr, "unknown"); break;
+          case AL_INITIAL : strcpy(statestr, "initial"); break;
+          case AL_PLAYING : strcpy(statestr, "playing"); break;
+          case AL_PAUSED : strcpy(statestr, "paused"); break;
+          case AL_STOPPED : strcpy(statestr, "stopped"); break;
+          default : strcpy(statestr, "unknown"); break;
           };
           // 20021007 thammer fixme: deal with this properly!
 #if COIN_DEBUG && DEBUG_AUDIO // debug
